using System;
using System.Collections;
using System.Runtime;
using System.Runtime.Serialization;

using jvms.datamodel.dcd;
using jvms.datamodel.prf;
using jvms.datamodel.sad;
using jvms.datamodel.spd;

namespace jvms.datamodel
{
	[Serializable]
	/// <summary>
	/// 
	/// </summary>
	public class Project
	{
		private IList components = new ArrayList();
		private IList deviceConfigurations = new ArrayList();
		private IList applications = new ArrayList();
		private IList properties = new ArrayList();
		private IList applicationInstantiations = new ArrayList();
		private IList deviceConfigInstantiations = new ArrayList();


		private String projectDirectory = "";
		private String company = "";
		private String author = "";
		private String name = "";

		/// <summary>
		/// 
		/// </summary>
		public IList DeviceConfigInstantiations
		{
			get
			{
				return deviceConfigInstantiations;
			}
			set
			{
				deviceConfigInstantiations = value;
			}
		}

		/// <summary>
		/// 
		/// </summary>
		public IList ApplicationInstantiations
		{
			get
			{
				return applicationInstantiations;
			}
			set
			{
				applicationInstantiations = value;
			}
		}

		/// <summary>
		/// 
		/// </summary>
		public IList Components
		{
			get
			{
				return components;
			}
			set
			{
				components = value;
			}
		}

		/// <summary>
		/// 
		/// </summary>
		public IList DeviceConfigurations
		{
			get
			{
				return deviceConfigurations;
			}
			set
			{
				deviceConfigurations = value;
			}
		}

		/// <summary>
		/// 
		/// </summary>
		public IList Applications
		{
			get
			{
				return applications;
			}
			set
			{
				applications = value;
			}
		}

		/// <summary>
		/// 
		/// </summary>
		public IList Properties
		{
			get
			{
				return properties;
			}
			set
			{
				properties = value;
			}
		}

		/// <summary>
		/// 
		/// </summary>
		public String ProjectDirectory
		{
			get
			{
				return projectDirectory;
			}

			set
			{
				projectDirectory = value;
			}
		}

		/// <summary>
		/// 
		/// </summary>
		public String Company
		{
			get
			{
				return company;
			}
			set
			{
				company = value;
			}
		}

		/// <summary>
		/// 
		/// </summary>
		public String Name
		{
			get
			{
				return name;
			}
			set
			{
				name = value;
			}
		}

		/// <summary>
		/// 
		/// </summary>
		public String Author
		{
			get
			{
				return author;
			}

			set
			{
				author = value;
			}
		}
	
		/// <summary>
		/// 
		/// </summary>
		public Project()
		{
		}

		/// <summary>
		/// 
		/// </summary>
		/// <returns></returns>
		public bool requiresSave()
		{
			return false;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="app"></param>		
		public void deleteApplication( Application app )
		{
			//TODO remove the application
		}

		/// <summary>
		/// This will add a component to a project and initialize any project wide properties.
		/// </summary>
		/// <param name="component">The component that is being added.</param>
		public void addComponent( NonSCAComponent component )
		{
			bool foundGoodName = false;
			int potentialNameSuffix = 0;
			String potentialNamePrefix = "component";
			if( component is SCAComponent && 
				!((SCAComponent)component).Descriptor.ComponentType.Equals( "" ) )
			{
				potentialNamePrefix = ((SCAComponent)component).Descriptor.ComponentType;
			}

			while( !foundGoodName )
			{
				foundGoodName = true;
				potentialNameSuffix++;
				foreach( NonSCAComponent next in components )
				{
					if( next.Name.Equals( potentialNamePrefix + potentialNameSuffix ) )
					{
						foundGoodName = false;
					}
					
				}
			}
			component.Name = potentialNamePrefix + potentialNameSuffix;
			components.Add( component );
		}

		/// <summary>
		/// This will add a properties to a project and initialize any project wide properties.
		/// </summary>
		/// <param name="properties">The properties that are being added.</param>
		public void addProperties( Properties propertySet )
		{
			bool foundGoodName = false;
			int potentialNameSuffix = 0;
			String potentialNamePrefix = "properties";

			while( !foundGoodName )
			{
				foundGoodName = true;
				potentialNameSuffix++;
				foreach( Properties next in properties )
				{
					if( next.Description.Equals( potentialNamePrefix + potentialNameSuffix ) )
					{
						foundGoodName = false;
					}
					
				}
			}
			propertySet.Description = potentialNamePrefix + potentialNameSuffix;
			properties.Add( propertySet );
		}

		/// <summary>
		/// This will add an application to a project and initialize any project wide properties.
		/// </summary>
		/// <param name="application">The application that are being added.</param>
		public void addApplication( Application application )
		{
			bool foundGoodName = false;
			int potentialNameSuffix = 0;
			String potentialNamePrefix = "application";

			while( !foundGoodName )
			{
				foundGoodName = true;
				potentialNameSuffix++;
				foreach( Application next in applications )
				{
					if( next.Name.Equals( potentialNamePrefix + potentialNameSuffix ) )
					{
						foundGoodName = false;
					}
					
				}
			}
			application.Name = potentialNamePrefix + potentialNameSuffix;
			applications.Add( application );
		}

		/// <summary>
		/// This will add a device config to a project and initialize any project wide properties.
		/// </summary>
		/// <param name="deiceConfig">The application that are being added.</param>
		public void addDeviceConfiguration( DeviceConfiguration deviceConfig )
		{
			bool foundGoodName = false;
			int potentialNameSuffix = 0;
			String potentialNamePrefix = "deviceconfiguration";

			while( !foundGoodName )
			{
				foundGoodName = true;
				potentialNameSuffix++;
				foreach( DeviceConfiguration next in deviceConfigurations )
				{
					if( next.Name.Equals( potentialNamePrefix + potentialNameSuffix ) )
					{
						foundGoodName = false;
					}
					
				}
			}
			deviceConfig.Name = potentialNamePrefix + potentialNameSuffix;
			deviceConfigurations.Add( deviceConfig );
		}

		/// <summary>
		/// This will add a instantiation to a project and initialize any project wide properties.
		/// </summary>
		/// <param name="instantiation">The instantiation that are being added.</param>
		public void addApplicationInstantiation( ApplicationComponentInstantiation instantiation )
		{
			bool foundGoodName = false;
			int potentialNameSuffix = 0;
			String potentialNamePrefix = "instantiation";

			while( !foundGoodName )
			{
				foundGoodName = true;
				potentialNameSuffix++;
				foreach( ApplicationComponentInstantiation next in applicationInstantiations )
				{
					if( next.UsageName.Equals( potentialNamePrefix + potentialNameSuffix ) )
					{
						foundGoodName = false;
					}
					
				}
			}
			instantiation.UsageName = potentialNamePrefix + potentialNameSuffix;
			applicationInstantiations.Add( instantiation );
		}

		/// <summary>
		/// This will add a instantiation to a project and initialize any project wide properties.
		/// </summary>
		/// <param name="instantiation">The instantiation that are being added.</param>
		public void addDeviceConfigInstantiation( jvms.datamodel.dcd.ComponentPlacement instantiation )
		{
			bool foundGoodName = false;
			int potentialNameSuffix = 0;
			String potentialNamePrefix = "instantiation";

			while( !foundGoodName )
			{
				foundGoodName = true;
				potentialNameSuffix++;
				foreach( jvms.datamodel.dcd.ComponentPlacement next in deviceConfigInstantiations )
				{
					if( next.Instantiation.UsageName.Equals( potentialNamePrefix + potentialNameSuffix ) )
					{
						foundGoodName = false;
					}
					
				}
			}
			instantiation.Instantiation.UsageName = potentialNamePrefix + potentialNameSuffix;
			deviceConfigInstantiations.Add( instantiation );
		}
	}
}
