using System;
using System.Collections;
using System.Xml;

using jvms.xmlgeneration;
using jvms.datamodel.prf;
using jvms.datamodel.spd;

namespace jvms.datamodel.common
{
	[Serializable]
	public class ComponentProperties : XMLObject
	{
		/// <summary>
		/// If set then these are readonly properties that are used
		/// to set as allocation properties for a component instance.
		/// </summary>
		private NonSCAComponent baseComponent = null;

		private TypedArrayList simpleSequences = new TypedArrayList(typeof(SimpleSequenceRef));
		private TypedArrayList structSequences = new TypedArrayList(typeof(StructSequenceRef));
		private TypedArrayList simples = new TypedArrayList(typeof(SimpleRef));
		private TypedArrayList structs = new TypedArrayList(typeof(StructRef));

		/// <summary>
		/// This is a list of struct refs.
		/// </summary>
		public TypedArrayList Structs
		{
			get
			{
				return structs;
			}
			set
			{
				structs = value;
			}
		}

		/// <summary>
		/// This is a list of simple Refs.
		/// </summary>
		public TypedArrayList Simples
		{
			get
			{
				return simples;
			}
			set
			{
				simples = value;
			}
		}

		/// <summary>
		/// This is the list of simple sequences refs.
		/// </summary>
		/// <value>The simple sequence list.</value>
		public TypedArrayList SimpleSequences
		{
			get
			{
				return simpleSequences;
			}
			set
			{
				simpleSequences = value;
			}
		}

		/// <summary>
		/// This is the list of StructSequenceRefs.
		/// </summary>
		public TypedArrayList StructSequences
		{
			get
			{
				return structSequences;
			}

			set
			{
				structSequences = value;
			}
		}

		public Properties BaseProperties
		{
			get
			{ 
				if( baseComponent != null )
				{
					return baseComponent.Properties;
				}
				return null;
			}
		}

		public NonSCAComponent BaseComponent
		{
			get{ return baseComponent; }
			set { baseComponent = value; }
		}

		/// <summary>
		/// This will return a string representation of the 
		/// ComponentProperties class.
		/// </summary>
		/// <returns>This author represented in a human readable form.</returns>
		public override String ToString()
		{
			return "Component Properties";
		}

		/// <summary>
		/// This will convert this object to an XML element.
		/// </summary>
		/// <param name="document">The document that this element will go into.</param>
		/// <returns>The element representation of the object.</returns>
		public XmlElement toXML( XmlDocument document )
		{
			XmlElement element = document.CreateElement( "componentproperties" );
			XMLHelper.writeListOfXMLElements( document, element, simples );
			XMLHelper.writeListOfXMLElements( document, element, simpleSequences );
			XMLHelper.writeListOfXMLElements( document, element, structs );
			XMLHelper.writeListOfXMLElements( document, element, structSequences );
			return element;
		}
	}
}
