using System;
using System.Xml;

using jvms.datamodel.sad;
using jvms.datamodel.scd;
using jvms.xmlgeneration;


namespace jvms.datamodel.common
{
	[Serializable]
	/// <summary>
	/// Either the componentInstantiation or the findBy must be used
	/// </summary>
	public class ComponentSupportedInterface : PortConnection, XMLObject
	{
		private ComponentInstantiation componentInstantiation;
		private Interface identifier;

		/// <summary>
		/// This component instantiation for this
		/// interface.
		/// </summary>
		public ComponentInstantiation Instance
		{
			get
			{
				return componentInstantiation;
			}
			set
			{
				componentInstantiation = value;
			}
		}

		/// <summary>
		/// This component interface identifier.
		/// </summary>
		public Interface SupportedIdentifier
		{
			get
			{
				return identifier;
			}

			set
			{
				identifier = value;
			}
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="document"></param>
		/// <returns></returns>
		public override XmlElement toXML( XmlDocument document )
		{
			XmlElement supportedInterface = document.CreateElement( "componentsupportedinterface" );
			XMLHelper.writeElement( document, supportedInterface, "supportedidentifier", identifier.RepositoryID );
			if( componentInstantiation != null )
			{
				XMLHelper.writeElement( document, supportedInterface, "componentinstantiationref", componentInstantiation.ID.DCEUUID );
			}
			else
			{
				throw new Exception( "An instantiation or findby is required on the component supported interface" );
			}
			return supportedInterface;
		}
	}
}
