using System;
using System.Xml;

using jvms.xmlgeneration;

namespace jvms.datamodel.common
{
	/// <summary>
	/// 
	/// </summary>
	[Serializable]
	public class ConnectInterface : XMLObject
	{
		//required
		private PortConnection usesPort;

		//one of the following three are required.
		private FindBy findBy;
		private PortConnection providesPort;
		
		private ID id = new ID();

		/// <summary>
		/// This is the find by to find the connect interface.
		/// </summary>
		public FindBy FindBy
		{
			get
			{
				return findBy;
			}
			set
			{
				findBy = value;
			}
		}

		/// <summary>
		/// This is the provides port for this component.
		/// </summary>
		public PortConnection ProvidesPort
		{
			get
			{
				return providesPort;
			}
			set
			{
				providesPort = value;
			}
		}

		/// <summary>
		/// This is the uses port for this interface.
		/// </summary>
		public PortConnection UsesPort
		{
			get
			{
				return usesPort;
			}
			set
			{
				usesPort = value;
			}
		}

		/// <summary>
		/// This is the id that identifies this interface.
		/// </summary>
		public ID ID
		{
			get
			{
				return id;
			}
			set
			{
				id = value;
			}
		}

		/// <summary>
		/// This will convert this object to an XML element.
		/// </summary>
		/// <param name="document">The document that this element will go into.</param>
		/// <returns>The element representation of the object.</returns>
		public XmlElement toXML( XmlDocument document )
		{
			XmlElement element = document.CreateElement( "connectinterface" );
			element.SetAttribute( "id", id.DCEUUID );
			if( usesPort != null )
			{
				element.AppendChild( usesPort.toXML( document ) );
			}

			if( providesPort != null )
			{
				element.AppendChild( providesPort.toXML( document ) );
			}
			else if( findBy != null )
			{
				XmlElement findByElement = document.CreateElement( "findby" );
				findByElement.AppendChild( findBy.toXML( document ) );
				element.AppendChild( findByElement );
			}
			return element;
		}

		public override String ToString()
		{
			String retval = "";
			if( usesPort != null )
			{
				retval += usesPort.Port.UsesName;
			}
			else if( providesPort != null )
			{
				retval += providesPort.Port.UsesName;
			}
			else if( findBy != null )
			{
				retval += findBy.ToString();
			}
			return retval;
		}
	}
}
