using System;
using System.Xml;

using jvms.datamodel.spd;
using jvms.xmlgeneration;

namespace jvms.datamodel.common
{
	/// <summary>
	/// Summary description for DeviceUsedByThisComponentPortConnection.
	/// </summary>
	[Serializable]
	public class DeviceUsedByThisComponentPortConnection : PortConnection
	{
		private SCAComponent component;
		private UsesDevice usesDevice;
		private bool usesPort = false;

		public DeviceUsedByThisComponentPortConnection( bool usesPortConnection )
		{
			usesPort = usesPortConnection;
		}

		/// <summary>
		/// 
		/// </summary>
		public SCAComponent Component
		{
			get
			{
				return component;
			}
			set
			{
				component = value;
			}
		}

		/// <summary>
		/// 
		/// </summary>
		public UsesDevice UsesDevice
		{
			get
			{
				return usesDevice;
			}
			set
			{
				usesDevice = value;
			}
		}

		/// <summary>
		/// This will convert this object to XML.
		/// </summary>
		/// <param name="document"></param>
		/// <returns></returns>
		public override XmlElement toXML( XmlDocument document )
		{
			XmlElement element = null;
			if( usesPort )
			{
				element = document.CreateElement( "usesport" );
				if( Port != null && Port.RepositoryID != null )
				{
					XMLHelper.writeElement( document, element, "usesidentifier", Port.UsesName );
				}
			}
			else
			{
				element = document.CreateElement( "providesport" );
				if( Port != null && Port.RepositoryID != null )
				{
					XMLHelper.writeElement( document, element, "providesidentifier", Port.UsesName );
				}
			}
			XmlElement usedByElement = document.CreateElement( "deviceusedbythiscomponentref" );
			usedByElement.SetAttribute( "refid", component.ID.DCEUUID );
			if( usesDevice != null )
			{
				usedByElement.SetAttribute( "usesrefid", usesDevice.ID );			
			}
			element.AppendChild( usedByElement );
			XmlComment comment = document.CreateComment( component.Name );
			element.AppendChild( comment );
			return element;
		}
	}
}
