using System;
using System.Collections;
using System.Xml;

using jvms.datamodel.common;
using jvms.datamodel.spd;

using jvms.xmlgeneration;

namespace jvms.datamodel.dcd
{
	[Serializable]
	public class DeviceConfiguration : FileSystemObject
	{
		/// <summary>
		/// BJL - This is a list of FileSystemName objects.
		/// </summary>
		private TypedArrayList fileSystemNames = new TypedArrayList(typeof(FileSystemName));
		/// <summary>
		/// This is a list of ConnectInterface objects.
		/// </summary>
		private TypedArrayList connections = new TypedArrayList(typeof(ConnectInterface));
		private NamingService domainManager = new NamingService();
		/// <summary>
		/// This is a list of ComponentPlacement objects.
		/// </summary>
		private TypedArrayList partitioning = new TypedArrayList(typeof(ComponentPlacement));
		private SCAComponent deviceManager;
		private String description = "";
		private String name = "";
		private ID id = new ID();

		/// <summary>
		/// This is the list of file system names.
		/// </summary>
		public TypedArrayList FileSystemNames
		{
			get
			{
				return fileSystemNames;
			}
			set
			{
				fileSystemNames = value;
			}
		}
		
		/// <summary>
		/// This will get the list of connections.
		/// </summary>
		public TypedArrayList Connections
		{
			get
			{
				return connections;
			}
			set
			{
				connections = value;
			}
		}


		/// <summary>
		/// This is the domain manager.
		/// </summary>
		public NamingService DomainManager
		{
			get
			{
				return domainManager;
			}
			set
			{
				domainManager = value;
			}
		}


		/// <summary>
		/// This is the list of partitionings.
		/// </summary>
		public TypedArrayList Partitioning
		{
			get
			{
				return partitioning;
			}
			set
			{
				partitioning = value;
			}
		}

		/// <summary>
		/// This is the device manager.
		/// </summary>
		public SCAComponent DeviceManager
		{
			get
			{
				return deviceManager;
			}
					
			set
			{
				deviceManager = value;
			}
		}
		/// <summary>
		/// The description of the DCD.
		/// </summary>
		public String Description
		{
			get
			{
				return description;
			}

			set
			{
				description = value;
			}
		}

		/// <summary>
		/// The name of the DCD.
		/// </summary>
		public String Name
		{
			get
			{
				return name;
			}

			set
			{
				name = value;
			}
		}

			/// <summary>
			/// The id for this DCD.
			/// </summary>
		public ID ID
		{
			get
			{
				return id;
			}

			set
			{
				id = value;
			}
		}

		/// <summary>
		/// This will generate the list of component files that are necessary.
		/// </summary>
		/// <returns>A list of NonSCAComponent objects.</returns>
		public ICollection getComponentFiles()
		{
			Hashtable componentFiles = new Hashtable();
				
			foreach( ComponentPlacement instance in partitioning )
			{
				componentFiles[ instance.ComponentFile ] = instance.ComponentFile;
			}
			return componentFiles.Keys;
		}

		/// <summary>
		/// This will convert this object to an XML element.
		/// </summary>
		/// <param name="document">The document that this element will go into.</param>
		/// <returns>The element representation of the object.</returns>
		public override XmlElement toXML( XmlDocument document )
		{
			document.XmlResolver = new JVMSXMLResolver();

			XmlDocumentType type = document.CreateDocumentType( "deviceconfiguration", "", "deviceconfiguration.2.2.dtd", null );
			XmlDeclaration decl = document.CreateXmlDeclaration( "1.0", "ISO-8859-1", "no" );
			document.AppendChild( decl );
			document.AppendChild( type );

			XmlElement element = document.CreateElement( "deviceconfiguration" );
			element.SetAttribute( "id", id.DCEUUID );
			element.SetAttribute( "name", name );

			XMLHelper.writeElement( document, element, "description", description );
			XMLHelper.writeFileReference( document, element, "devicemanagersoftpkg", this, deviceManager );
			//partitioning
			if( partitioning.Count > 0 )
			{
				//ok now for the hard part we need to write out the list of component files that
				//we are using, this list will be determined via the instantiations in the 
				//partitioning.  We are using a hashtable because there is no Set, not sure why?
				ICollection componentFiles = getComponentFiles();
				
				if( componentFiles.Count > 0 )
				{
					XmlElement filesElement = document.CreateElement( "componentfiles" );
					element.AppendChild( filesElement );
					foreach( NonSCAComponent component in componentFiles )
					{
						XmlElement fileElement = 
							XMLHelper.writeFileReference( document, filesElement, "componentfile", this, component );					
						fileElement.SetAttribute( "id", component.Name );
					}
				}
			
			
				XmlElement partitioningElement = document.CreateElement( "partitioning" );
				element.AppendChild( partitioningElement );
				XMLHelper.writeListOfXMLElements( document, partitioningElement, partitioning );
			}
			
			//domainmanager
			XmlElement domainManagerElement = document.CreateElement( "domainmanager" );
			element.AppendChild( domainManagerElement );
			domainManagerElement.AppendChild( domainManager.toXML( document ) );

			//connections
			if( connections.Count > 0 )
			{
				XmlElement connectionsElement = document.CreateElement( "connections" );
				element.AppendChild( connectionsElement );
				XMLHelper.writeListOfXMLElements( document, connectionsElement, connections );
			}

			//filessytemnames
			if( fileSystemNames.Count > 0 )
			{
				XmlElement fsElement = document.CreateElement( "filesystemnames" );
				element.AppendChild( fsElement );
				XMLHelper.writeListOfXMLElements( document, fsElement, fileSystemNames );
			}
			return element;
		}

		/// <summary>
		/// This will get the relative name of the file to write to.  This
		/// string shall begin with a \
		/// </summary>
		/// <returns>The path to write this file to.</returns>
		public override String getFilename()
		{
			return "\\" + Name + ".dcd.xml";
		}

		public override String ToString()
		{
			return Name;
		}

	}
}
