using System;
using System.Xml;

using jvms.xmlgeneration;

namespace jvms.datamodel.prf
{
	[Serializable]
	/// <summary>
	/// Values of enumerations the property may take.  I think the CF 
	/// is responsible for translating these to their proper values (CORBA long).
	/// </summary>
	public class Enumeration : XMLObject
	{
		private String enumValue = "";
		private String label = "";

		/// <summary>
		/// This will get the value of the enum.
		/// </summary>
		public String Value
		{
			get
			{
				return enumValue;
			}
			set
			{
				enumValue = value;
			}
		}

		/// <summary>
		/// The label of the enumeration.
		/// </summary>
		public String Label
		{
			get
			{
				return label;
			}
			set
			{
				label = value;
			}
		}

		/// <summary>
		/// This will return a string representation of the Author class.
		/// </summary>
		/// <returns>This author represented in a human readable form.</returns>
		public override String ToString()
		{
			String retval = "<New Enumeration>";
			if( !Label.Equals( "" ) )
			{
				retval = Label;
			}
			return retval;
		}

		public XmlElement toXML( XmlDocument document )
		{
			XmlElement element = document.CreateElement( "enumeration" );
			element.SetAttribute( "label", label );
			element.SetAttribute( "value", enumValue );
			return element;
		}
	}
}
