using System;
using System.Collections;
using System.Xml;

using jvms.xmlgeneration;

namespace jvms.datamodel.prf
{
	[Serializable]
	public class Properties : FileSystemObject
	{
		private TypedArrayList simples = new TypedArrayList(typeof(Simple));
		private TypedArrayList simpleSequences = new TypedArrayList(typeof(SimpleSequence));
		private TypedArrayList tests = new TypedArrayList(typeof(Test));
		private TypedArrayList structs = new TypedArrayList(typeof(Struct));		
		private TypedArrayList structSequences = new TypedArrayList(typeof(StructSequence));
		private String description = "";
		
		/// <summary>
		/// This is a list of simples
		/// </summary>
		public TypedArrayList Simples
		{
			get
			{
				return simples;
			}
			set
			{
				simples = value;
			}
		}

		/// <summary>
		/// The list of simple sequence values.
		/// </summary>
		public TypedArrayList SimpleSequences
		{
			get
			{
				return simpleSequences;
			}
			set
			{
				simpleSequences = value;
			}
		}

		/// <summary>
		/// This is the list of test properties.
		/// </summary>
		public TypedArrayList Test
		{
			get
			{
				return tests;
			}
			set
			{
				tests = value;
			}
		}



		/// <summary>
		/// The list of struct properties.
		/// </summary>
		public TypedArrayList Structs
		{
			get
			{
				return structs;
			}
			set
			{
				structs = value;
			}
		}

		/// <summary>
		/// The list of struct properties.
		/// </summary>
		public TypedArrayList StructSequences
		{
			get
			{
				return structSequences;
			}
			set
			{
				structSequences = value;
			}
		}

		/// <summary>
		/// This is a description of the properties.
		/// </summary>
		public String Description
		{
			get
			{
				return description;
			}
			set
			{
				description = value;
			}
		}

		/// <summary>
		/// This will convert this object to an XML element.
		/// </summary>
		/// <param name="document">The document that this element will go into.</param>
		/// <returns>The element representation of the object.</returns>
		public override XmlElement toXML( XmlDocument document )
		{
			document.XmlResolver = new JVMSXMLResolver();
			XmlDocumentType type = document.CreateDocumentType( "properties", "", "properties.2.2.dtd", null );
			XmlDeclaration decl = document.CreateXmlDeclaration( "1.0", "ISO-8859-1", "no" );
			document.AppendChild( decl );
			document.AppendChild( type );
			

			XmlElement element = document.CreateElement( "properties" );
			XMLHelper.writeElement( document, element, "description", description );
			XMLHelper.writeListOfXMLElements( document, element, simples );
			XMLHelper.writeListOfXMLElements( document, element, simpleSequences );
			XMLHelper.writeListOfXMLElements( document, element, tests );
			XMLHelper.writeListOfXMLElements( document, element, structs );
			XMLHelper.writeListOfXMLElements( document, element, structSequences );
			return element;
		}

		/// <summary>
		/// This will get the relative name of the file to write to.  This
		/// string shall begin with a \
		/// </summary>
		/// <returns>The path to write this file to.</returns>
		public override String getFilename()
		{
			return "\\" + Description + ".prf.xml";
		}

		public override String ToString()
		{
			return Description;
		}
	}
}
