using System;
using System.Xml;

using jvms.xmlgeneration;

namespace jvms.datamodel.prf
{
	[Serializable]
	/// <summary>
	/// "The range element describes the specific min and max values that are legal for the simple element. The intent of the range element is to provide a means to perform range validation. This element is not used by the CF ApplicationFactory or CF Application implementations."
	/// 
	/// CL - Range of value.  To be used by the HMI, but not checked by the CF.
	/// </summary>
	public class Range : XMLObject
	{
		private String min = "";
		private String max = "";

		/// <summary>
		/// The min value allowed for the range.
		/// </summary>
		public String Min
		{
			get
			{
				return min;
			}
			set
			{
				min = value;
			}
		}

		/// <summary>
		/// The max allowed value of the range.
		/// </summary>
		public String Max
		{
			get
			{
				return max;
			}
			set
			{
				max = value;
			}
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="document"></param>
		/// <returns></returns>
		public XmlElement toXML( XmlDocument document )
		{
			XmlElement element = document.CreateElement( "range" );
			element.SetAttribute( "min", min );
			element.SetAttribute( "max", max );

			return element;
		}
	}
}
