using System;
using System.Collections;
using System.Xml;

using jvms.datamodel.common;
using jvms.xmlgeneration;

namespace jvms.datamodel.prf
{
	[Serializable]
	/// <summary>
	/// "The structsequence element (see Figure D-16) is used to specify a list of properties with the same struct characteristics. The structsequence element maps to a properties element having the CF Properties type. Each item in the CF Properties type will be the same struct definition as referenced by the structrefid attribute."
	/// </summary>
	public class StructSequence : XMLObject
	{
		/// <summary>
		/// configurekind is limited to configure and factoryParam types.
		/// 
		/// I am assuming that all of the elements of a struct by extension are also 
		/// limited to this same type.
		/// </summary>
		private String configurationKind = "";
		/// <summary>
		/// BJL - This is a list of SimpleRef objects
		/// </summary>
		private TypedArrayList structValues  = new TypedArrayList(typeof(StructValue));
		private String description = "";
		private String mode = "";
		private String name = "";
		private Struct structValue;
		private ID id = new ID();

		public StructSequence()
		{
		}


		public StructSequence( StructSequence other )
		{
			configurationKind = other.ConfigurationKind;
			foreach( StructValue sv in other.StructValues )
			{
				structValues.Add( new StructValue(sv) );
			}
			description = other.Description;
			mode = other.Mode;
			name = other.Name;
			structValue = new Struct(other.Struct);
		}

		/// <summary>
		/// The configuration kind.
		/// </summary>
		public String ConfigurationKind
		{
			get
			{
				return configurationKind;
			}
			set
			{
				configurationKind = value;
			}
		}
		/// <summary>
		/// The list of possible values.
		/// </summary>
		public TypedArrayList StructValues
		{
			get
			{
				return structValues;
			}
			set
			{
				structValues = value;
			}
		}
		/// <summary>
		/// The property description.
		/// </summary>
		public String Description
		{
			get
			{
				return description;
			}
			set
			{
				description = value;
			}
		}
		/// <summary>
		/// The mode of the property.
		/// </summary>
		public String Mode
		{
			get
			{
				return mode;
			}
			set
			{
				mode = value;
			}
		}
		/// <summary>
		/// The name of the property.
		/// </summary>
		public String Name
		{
			get
			{
				return name;
			}
			set
			{
				name = value;
			}
		}
		/// <summary>
		/// The struct Reference ID.
		/// BJL - This might be the wrong datatype, we need to think about
		/// this a little more.  I will leave it as a string for now until
		/// I have a little time to read Appendix D.
		/// </summary>
		public Struct Struct
		{
			get
			{
				return structValue;
			}
			set
			{
				structValue = value;
			}
		}
		/// <summary>
		/// The id of this property.
		/// </summary>
		public ID ID
		{
			get
			{
				return id;
			}
			set
			{
				id = value;
			}
		}
		/// <summary>
		/// This will return a string representation of the Author class.
		/// </summary>
		/// <returns>This author represented in a human readable form.</returns>
		public override String ToString()
		{
			String retval = "<New Struct Sequence>";
			if( !Name.Equals( "" ) )
			{
				retval = Name;
			}
			return retval;
		}

		/// <summary>
		/// Create an SCA XML representation of this element.
		/// </summary>
		/// <param name="document"></param>
		/// <returns></returns>
		public XmlElement toXML( XmlDocument document )
		{
			XmlElement element = document.CreateElement( "structsequence" );
			
			XMLHelper.writeAttribute( element, "id", id.DCEUUID );
			XMLHelper.writeAttribute( element, "structrefid", structValue.ID.DCEUUID );
			XMLHelper.writeAttribute( element, "name", name );
			XMLHelper.writeAttribute( element, "mode", mode );

			XMLHelper.writeElement( document, element, "description", description );

			XMLHelper.writeListOfXMLElements( document, element, structValues );
			if( !configurationKind.Equals( "" ) )
			{
				XmlElement ckElement = document.CreateElement( "configurationkind" );
				element.AppendChild( ckElement );
				ckElement.SetAttribute( "kindtype", configurationKind );
			}

			return element;
		}
	}
}
