using System;
using System.Collections;
using System.Xml;

using jvms.datamodel.common;
using jvms.xmlgeneration;

namespace jvms.datamodel.prf
{
	[Serializable]
	/// <summary>
	/// "The test element (see Figure D-14) is used to specify a list of test properties for executing the runTest() operation to perform a component specific test.  This definition contains inputvalue and resultvalue elements and it has a testid attribute for grouping test properties to a specific test.  Inputvalues are used to configure the test to be performed (e.g., frequency and RF power output level).  When the test has completed, resultvalues contain the results of the testing (e.g., Pass or a fault code/message)."
	/// 
	/// The SCA specifies no CF behavior on testing, other than Application delegating tests to the assemblyController.
	/// </summary>
	public class Test : XMLObject
	{
		/// <summary>
		/// The values (expected?) expected of the test, or perhaps just the 
		/// format of what the answer would take (more likely).  This could be 
		/// used by an HMI or self test application (somehow...).
		/// </summary>
		private TypedArrayList resultValues = new TypedArrayList(typeof(Simple));
		/// <summary>
		/// A list of test properties to be passed as parameters to the RunTest operation.
		/// </summary>
		private TypedArrayList inputValues = new TypedArrayList(typeof(Simple));
		private String description = "";
		private ID id = new ID();

		/// <summary>
		/// The list of result values.
		/// </summary>
		public TypedArrayList ResultValues
		{
			get
			{
				return resultValues;
			}
			set
			{
				resultValues = value;
			}
		}
		/// <summary>
		/// The list of input values.
		/// </summary>
		public TypedArrayList InputValues
		{
			get
			{
				return inputValues;
			}
			set
			{
				inputValues = value;
			}
		}
		/// <summary>
		/// The description of the property.
		/// </summary>
		public String Description
		{
			get
			{
				return description;
			}
			set
			{
				description = value;
			}
		}
		/// <summary>
		/// The id of the property.
		/// </summary>
		public ID ID
		{
			get
			{
				return id;
			}
			set
			{
				id = value;
			}
		}

		public XmlElement toXML( XmlDocument document )
		{
			XmlElement element = document.CreateElement( "test" );
			element.SetAttribute( "id", id.DCEUUID );

			XMLHelper.writeElement( document, element, "description", description );
			if( inputValues.Count > 0 )
			{
				XmlElement ivElement = document.CreateElement( "inputValue" );
				element.AppendChild( ivElement );
				XMLHelper.writeListOfXMLElements( document, ivElement, inputValues );
			}
			XmlElement rvElement = document.CreateElement( "resultValue" );
			element.AppendChild( rvElement );
			XMLHelper.writeListOfXMLElements( document, rvElement, resultValues );
			return element;
		}
	}
}
