using System;
using System.Collections;
using System.Xml;

using jvms.datamodel.common;
using jvms.datamodel.spd;
using jvms.xmlgeneration;

namespace jvms.datamodel.sad
{
	[Serializable]
	public class ApplicationComponentInstantiation : ComponentInstantiation
	{
		private FindComponent findComponent = new FindComponent();

		//this is actually the softwareassembly/componentplacement/componentfileref
		private NonSCAComponent component = null;
		//this is actually part of the application but we are storing it here
		//because it makes more sense
		private IList externalPorts = new ArrayList();

		/// <summary>
		/// A list of this instantiations external ports.
		/// </summary>
		public IList ExternalPorts
		{
			get
			{
				return externalPorts;
			}
			set
			{
				externalPorts = value;
			}
		}

		/// <summary>
		/// The find component property of the instantiation.
		/// </summary>
		public FindComponent FindComponent
		{
			get
			{
				return findComponent;
			}
			set
			{
				findComponent = value;
			}
		}

		/// <summary>
		/// The id of the instantiation.
		/// </summary>
		public NonSCAComponent Component
		{
			get
			{
				return component;
			}
			set
			{
				component = value;
			}
		}

		public override String ToString()
		{
			String retval = "<new instance>";
			if( !UsageName.Equals( "" ) )
			{
				retval = UsageName;
			}
			return retval;
		}

		/// <summary>
		/// This will create an XML element from this instantiation.
		/// </summary>
		/// <param name="document"></param>
		/// <returns></returns>
		public override XmlElement toXML( XmlDocument document )
		{
			XmlElement element = document.CreateElement( "componentplacement" );
			XmlElement fileElement = document.CreateElement( "componentfileref" );
			fileElement.SetAttribute( "refid", component.Name );
			element.AppendChild( fileElement );

			XmlElement instanceElement = base.toXML( document );
			element.AppendChild( instanceElement );


			XmlElement findElement = findComponent.toXML( document );
			if( findElement != null )
			{
				instanceElement.AppendChild( findElement );
			}
			return element;
		}
	}
}
