using System;
using System.Collections;
using System.Xml;

using jvms.datamodel.common;
using jvms.datamodel.spd;
using jvms.xmlgeneration;

namespace jvms.datamodel.sad
{
	[Serializable]
	public class ComponentInstantiation : XMLObject
	{
		private FindComponent findComponent = new FindComponent();
		private ComponentProperties properties = new ComponentProperties();
		private String usageName = "";
		private ID id = new ID();

		//this is actually the softwareassembly/componentplacement/componentfileref
		private NonSCAComponent component = null;
		//this is actually part of the application but we are storing it here
		//because it makes more sense
		private IList externalPorts = new ArrayList();

		/// <summary>
		/// A list of this instantiations external ports.
		/// </summary>
		public IList ExternalPorts
		{
			get
			{
				return externalPorts;
			}
			set
			{
				externalPorts = value;
			}
		}

		/// <summary>
		/// The find component property of the instantiation.
		/// </summary>
		public FindComponent FindComponent
		{
			get
			{
				return findComponent;
			}
			set
			{
				findComponent = value;
			}
		}
		/// <summary>
		/// The properties of the instantiation.
		/// </summary>
		public ComponentProperties Properties
		{
			get
			{
				return properties;
			}
			set
			{
				properties = value;
			}
		}
		/// <summary>
		/// The usage name of the instantiation.
		/// </summary>
		public String UsageName
		{
			get
			{
				return usageName;
			}
			set
			{
				usageName = value;
			}
		}

		/// <summary>
		/// The id of the instantiation.
		/// </summary>
		public NonSCAComponent Component
		{
			get
			{
				return component;
			}
			set
			{
				component = value;
			}
		}

		/// <summary>
		/// 
		/// </summary>
		public ID ID
		{
			get
			{
				return id;
			}
			set
			{
				id = value;
			}
		}

		/// <summary>
		/// This will create an XML element from this instantiation.
		/// </summary>
		/// <param name="document"></param>
		/// <returns></returns>
		public XmlElement toXML( XmlDocument document )
		{
			XmlElement element = document.CreateElement( "componentplacement" );
			XmlElement fileElement = document.CreateElement( "componentfileref" );
			fileElement.SetAttribute( "refid", component.ID.DCEUUID );
			element.AppendChild( fileElement );

			XmlElement instanceElement = document.CreateElement( "componentinstantiation" );
			element.AppendChild( instanceElement );
			instanceElement.SetAttribute( "id", id.DCEUUID );
			XMLHelper.writeElement( document, instanceElement, "usagename", usageName );

			if( properties.Simples.Count + 
				properties.SimpleSequences.Count +
				properties.Structs.Count +
				properties.StructSequences.Count > 0 )
			{
				instanceElement.AppendChild( properties.toXML( document ) );
			}

			if( !findComponent.NamingService.Name.Equals( "" ) ||
				findComponent.ResourceFactory.Instance != null )
			{
				instanceElement.AppendChild( findComponent.toXML( document ) );
			}
			return element;
		}
	}
}
