using System;
using System.Xml;

using jvms.datamodel.common;
using jvms.xmlgeneration;

namespace jvms.datamodel.sad
{
	[Serializable]
	public class ComponentResourceFactoryRef : XMLObject
	{
		private ComponentProperties resourceFactoryProperties = new ComponentProperties();

		//this will be used to create the refId
		private ComponentInstantiation instance = null;

		/// <summary>
		/// The resource factory properties.
		/// </summary>
		public ComponentProperties ResourceFactoryProperties
		{
			get
			{
				return resourceFactoryProperties;
			}
			set
			{
				resourceFactoryProperties = value;
			}
		}

		/// <summary>
		/// The referenced ID.
		/// </summary>
		public ComponentInstantiation Instance
		{
			get
			{
				return instance;
			}
			set
			{
				instance = value;
			}
		}

		/// <summary>
		/// This will convert this object to xml.
		/// </summary>
		/// <param name="document"></param>
		/// <returns></returns>
		public XmlElement toXML( XmlDocument document )
		{
			XmlElement element = document.CreateElement( "componentresourcefactoryref" );
			element.SetAttribute( "refid", instance.ID.DCEUUID );	
			if( resourceFactoryProperties != null )
			{
				XmlElement propsElement = document.CreateElement( "resourcefactoryproperties" );
				XMLHelper.writeListOfXMLElements( document, propsElement, resourceFactoryProperties.Simples );
				XMLHelper.writeListOfXMLElements( document, propsElement, resourceFactoryProperties.SimpleSequences );
				XMLHelper.writeListOfXMLElements( document, propsElement, resourceFactoryProperties.Structs );
				XMLHelper.writeListOfXMLElements( document, propsElement, resourceFactoryProperties.StructSequences );			
				if( propsElement.ChildNodes.Count > 0 )
				{
					//only add if properties have been defined.
					element.AppendChild( propsElement );
				}
			}
			return element;
		}

	}
}
