using System;

using jvms.datamodel.scd;

namespace jvms.datamodel.sad
{
	[Serializable]
	/// <summary>
	/// No SCA documentation.  This element represents a particular port that is to be 
	/// exported from the application via the Application::getPort() operation.
	/// </summary>
	public class Port
	{
		/// <summary>
		/// The supportedidentifier element identifies which supported 
		/// interface on the component is to participate in the connection 
		/// relationship.  This identifier will correspond with the repid attribute of 
		/// one of the component's supportsinterface elements, specified in the Software 
		/// Component Descriptor.
		/// 
		/// CL: Supported interfaces are those that the component inherits from, instead 
		/// of provides or uses ports.  Either way, and interface is an interface...
		/// </summary>
		private ComponentPortEndPoint supports;
		/// <summary>
		/// The providesidentifier element identifies which "provides port" 
		/// on the component is to participate in the connection relationship.  
		/// This identifier will correspond with a repid attribute for one 
		/// of the component ports elements, specified in the Software 
		/// Component Descriptor.
		/// </summary>
		private ComponentPortEndPoint provides;
		/// <summary>
		/// The usesidentifier element identifies which "uses port" on the component is to 
		/// participate in the connection relationship.  This identifier will correspond 
		/// with an id for one of the component ports specified in the Software Component 
		/// Descriptor.
		/// </summary>
		private ComponentPortEndPoint uses;
		private String description = "";

		/// <summary>
		/// The supported identifier.
		/// </summary>
		public ComponentPortEndPoint Supports
		{
			get
			{
				return supports;
				}
			set
			{
				supports = value;
			}
		}

		/// <summary>
		/// The provides identifier.
		/// </summary>
		public ComponentPortEndPoint Provides
		{
			get
			{
				return provides;
			}
			set
			{
				provides = value;
			}
		}

		/// <summary>
		/// The uses identifier.
		/// </summary>
		public ComponentPortEndPoint Uses
		{
			get
			{
				return uses;
			}
			set
			{
				uses = value;
			}
		}

		/// <summary>
		/// The description.
		/// </summary>
		public String Description
		{
			get
			{
				return description;
			}
			set
			{
				description = value;
			}
		}
	}
}
