using System;

namespace jvms.datamodel.scd
{
	[Serializable]
	/// <summary>
	/// "The ports element (see Figure D-19) describes what interfaces 
	/// a component provides and uses.  The provides elements are 
	/// interfaces that are not part of a components interface 
	/// but are independent interfaces known as facets (in CORBA 
	/// Components terminology) (i.e. a provides port at the end 
	/// of a path, like I/O Device or Modem Device, does not need 
	/// to be a CF Port type).  The uses element is a CF Port 
	/// interface type that is connected to a provides or 
	/// supportinterfaces interface. Any number of uses and 
	/// provides elements can be given in any order. Each ports 
	/// element has a name and references an interface by repid 
	/// (see interfaces section D.5.1.5). The port names are used 
	/// in the Software Assembly Descriptor to connect ports together. 
	/// A ports element also has an optional porttype element that 
	/// allows for identification of port classification.  Values 
	/// for porttype include data, control, responses, and 
	/// test. If a porttype is not given then control is assumed."
	/// 
	/// The ports element contains a list of all formal provides and uses ports.
	/// </summary>
	public class ComponentPort
	{
		private ComponentPortEndPoint usesPort = new ComponentPortEndPoint();
		private ComponentPortEndPoint providesPort = new ComponentPortEndPoint();
// 		ComponentPortEndPoint uses;
// 		ComponentPortEndPoint provides;
		/// <summary>
		/// The uses port.
		/// </summary>
		public ComponentPortEndPoint UsesPort
		{
			get
			{
				return usesPort;
			}
			set
			{
				usesPort = value;
			}
		}
		/// <summary>
		/// The provides port.
		/// </summary>
		public ComponentPortEndPoint ProvidesPort
		{
			get
			{
				return providesPort;
			}
			set
			{
				providesPort = value;
			}
		}
	}
}
