using System;
using System.Collections;
using System.Collections.Specialized;

namespace jvms.datamodel.scd
{
	[Serializable]
	/// <summary>
	/// This element describes the interfaces that this component can connect to "uses".
	/// </summary>
	public class ComponentPortEndPoint
	{
		/// <summary>
		/// This optional element provides additional information on the general use of the port.  It contains a single element with an enumeration on what the port is to used for.
		/// 
		/// "data | control | responses | test"
		/// 
		/// </summary>
		private IList portTypes = new ArrayList();
		/// <summary>
		/// This string gives the name (unique, at least for this component) that the port is called.  This same name will be provided by the components "getPorts" interface.
		/// </summary>
		private String usesName = "";
		/// <summary>
		/// This is the repository ID for the interface required by this port.  This refers to an interface repid in the interfaces section, where additional information on this interface is given (including inherited interfaces, etc.)
		/// 
		/// </summary>
		private Interface repID = null;

		/// <summary>
		/// The type of ports for this endpoint.
		/// </summary>
		public IList PortTypes
		{
			get
			{
				return portTypes;
			}
			set
			{
				portTypes = value;
			}
		}
		/// <summary>
		/// The uses name of this port.
		/// </summary>
		public String UsesName
		{
			get
			{
				return usesName;
			}
			set
			{
				usesName = value;
			}
		}

		/// <summary>
		/// The repository id for this port.
		/// </summary>
		public Interface RepositoryID
		{
			get
			{
				return repID;
			}
			set
			{
				repID = value;
			}
		}

		public ComponentPortEndPoint()
		{
			portTypes.Add( "control" );
		}

		public override bool Equals( Object obj )
		{
			bool retval = false;
			if( obj != null && obj.GetType().Equals( this.GetType() ) )
			{
				ComponentPortEndPoint other = (ComponentPortEndPoint)obj;
				retval = this.UsesName.Equals( other.UsesName ) && 
					((this.RepositoryID == null && other.RepositoryID == null) ||
					 (this.RepositoryID != null && this.RepositoryID.Equals( other.RepositoryID) ));
				retval = retval && this.PortTypes.Count == other.PortTypes.Count;
				for( int i=0; retval && i<this.PortTypes.Count; i++ )
				{
					retval = retval && this.PortTypes[i].Equals( other.PortTypes[i] );
				}				
			}
			return retval;
		}

		public override int GetHashCode()
		{
			return UsesName.GetHashCode() + (RepositoryID == null ? 0 : RepositoryID.GetHashCode() );
		}

		/// <summary>
		/// This will return a string representation of the Author class.
		/// </summary>
		/// <returns>This author represented in a human readable form.</returns>
		public override String ToString()
		{
			String retval = "<New Port End Point>";
			if( !UsesName.Equals( "" ) )
			{
				retval = UsesName;
			}
			return retval;
		}
	}
}
