using System;

namespace jvms.datamodel.scd
{
	[Serializable]
	/// <summary>
	/// "The supportsinterface element is used to identify an IDL interface that the 
	/// component supports.  These interfaces are distinct interfaces that were inherited 
	/// by the component's specific interface.  One can widen the component's interface 
	/// to be a supportsinterface.  The repid is used to refer to the interface element 
	/// (see interfaces section D.5.1.5)."
	/// 
	/// Normally a component uses ports to provide its communication.  However, it is 
	/// optionally allowed to extend its base interface (typically CF::Resource) to 
	/// provide extra IDL.  This would typically be done for an assemblyController that 
	/// interfaced directly with the HMI when you didn't / can't use the Propertyset 
	/// interface to adequately control the component.
	/// </summary>
	public class SupportsInterface
	{
		/// <summary>
		/// This string seems to supply the informal name through which this 
		/// interface is supplied.  This seems pretty analogous to a port name.  
		/// My guess it that it may be referenced by this name in the SAD 
		/// connnectsinterface section.
		/// </summary>
		private String supportsName = "";
		/// <summary>
		/// A repository ID for the interface supported.  This 
		/// (according to the SCA) references an interface in the interface section.
		/// </summary>
		private String repID = "";

		/// <summary>
		/// A descriptive name.
		/// </summary>
		public String SupportsName
		{
			get
			{
				return supportsName;
			}
			set
			{
				supportsName = value;
			}
		}
		/// <summary>
		/// This is the repository id.
		/// </summary>
		public String RepositoryID
		{
			get
			{
				return repID;
			}
			set
			{
				repID = value;
			}
		}
	}
}
