using System;
using System.Collections;
using System.Collections.Specialized;
using System.Xml;

using jvms.xmlgeneration;

namespace jvms.datamodel.spd
{
	[Serializable]
	/// <summary>
	/// Element holding author information
	/// </summary>
	public class Author : XMLObject
	{
		private String webpage = "";
		private String company = "";
		private IList names = new ArrayList();

		/// <summary>
		/// The company webpage.
		/// </summary>
		public String Webpage
		{
			get
			{
				return webpage;
			}
			set
			{
				webpage = value;
			}
		}
		/// <summary>
		/// The name of the company.
		/// </summary>
		public String Company
		{
			get
			{
				return company;
			}
			set
			{
				company = value;
			}
		}
		/// <summary>
		/// The names of the authors.
		/// </summary>
		public IList Names
		{
			get
			{
				return names;
			}
			set
			{
				names = value;
			}
		}

		/// <summary>
		/// This will return a string representation of the Author class.
		/// </summary>
		/// <returns>This author represented in a human readable form.</returns>
		public override String ToString()
		{
			String retval = "<New Author>";
			if( !Company.Equals( "" ) )
			{
				retval = Company;
			}
			return retval;
		}

		public XmlElement toXML( XmlDocument document )
		{
			XmlElement author = document.CreateElement( "author" );

			XMLHelper.writeListOfStringElements( document, author, "name", names );
			XMLHelper.writeElement( document, author, "company", company );
			XMLHelper.writeElement( document, author, "webpage", webpage );
			return author;
		}
	}
}
