using System;
using System.Xml;

using jvms.datamodel.common;

using jvms.xmlgeneration;

namespace jvms.datamodel.spd
{
	[Serializable]
	/// <summary>
	/// The code element (see Figure D-5) will be used to indicate the local filename of the code that is described by the softpkg element, for a specific implementation of the software component.  The stack size and priority are options parameters used by the CF ExecutableDevice execute() operation.  Data types for the values of these options are unsigned long.  The type attribute for the code element will also indicate the type of file being delivered to the system.  The entrypoint element provides the means for providing the name of the entry point of the component being delivered.  The valid values for the type attribute are: "Executable", "KernelModule", "SharedLibrary", and "Driver."
	/// The meaning of the code type attribute:
	/// 1.	Executable means to use CF LoadableDevice load and CF ExecutableDevice execute.  This is a "main" process.
	/// 2.	Driver and Kernel Module means load only.
	/// 3.	SharedLibrary means dynamic linking.
	/// a.	Without a code entrypoint element means load only.
	/// b.	With a code entrypoint element means load and CF Device execute.
	/// 
	/// </summary>
	public class Code : XMLObject
	{
		/// <summary>
		/// The stack size and priority are options parameters used by the CF ExecutableDevice execute() operation.  Data types for the values of these options are unsigned long. 
		/// </summary>
		private String priority = "";
		/// <summary>
		/// The stack size and priority are options parameters used by the CF ExecutableDevice execute() operation.  Data types for the values of these options are unsigned long. 
		/// </summary>
		private String stackSize = "";
		/// <summary>
		/// The stack size and priority are options parameters used by the CF ExecutableDevice execute() operation.  Data types for the values of these options are unsigned long. 
		/// </summary>
		private String entryPoint = "";
		private String localFile = "";
		/// <summary>
		/// The type attribute for the code element will also indicate the type of file being delivered to the system.  The entrypoint element provides the means for providing the name of the entry point of the component being delivered.  The valid values for the type attribute are: "Executable", "KernelModule", "SharedLibrary", and "Driver."
		/// The meaning of the code type attribute:
		/// 1.	Executable means to use CF LoadableDevice load and CF ExecutableDevice execute.  This is a "main" process.
		/// 2.	Driver and Kernel Module means load only.
		/// 3.	SharedLibrary means dynamic linking.
		/// 
		/// </summary>
		private String type = "";

		/// <summary>
		/// The priority of the code file.
		/// </summary>
		public String Priority
		{
			get
			{
				return priority;
			}
			set
			{
				priority = value;
			}
		}
		/// <summary>
		/// The stack size of the code file.
		/// </summary>
		public String StackSize
		{
			get
			{
				return stackSize;
			}
			set
			{
				stackSize = value;
			}
		}
		/// <summary>
		/// An entry point into the code file.
		/// </summary>
		public String EntryPoint
		{
			get
			{
				return entryPoint;
			}
			set
			{
				entryPoint = value;
			}
		}
		/// <summary>
		/// A reference to a file in the filesystem.
		/// </summary>
		public String LocalFile
		{
			get
			{
				return localFile;
			}
			set
			{
				localFile = value;
			}
		}

		/// <summary>
		/// This is the type of the code.
		/// </summary>
		public String Type
		{
			get
			{
				return type;
			}
			set
			{
				type = value;
			}
		}

		/// <summary>
		/// This will convert this object to an XML element.
		/// </summary>
		/// <param name="document">The document that this element will go into.</param>
		/// <returns>The element representation of the object.</returns>
		public XmlElement toXML( XmlDocument document )
		{
			XmlElement element = document.CreateElement( "code" );
			if( !type.Equals( "" ) )
			{
				element.SetAttribute( "type", type );
			}

			XmlElement localFileElement = document.CreateElement( "localfile" );
			localFileElement.SetAttribute( "name" , localFile );
			element.AppendChild( localFileElement );

			XMLHelper.writeElement( document, element, "entrypoint", entryPoint );
			XMLHelper.writeElement( document, element, "stacksize", stackSize );
			XMLHelper.writeElement( document, element, "priority", priority );

			return element;
		}
	}
}
