using System;
using System.Xml;

using jvms.datamodel.common;
using jvms.xmlgeneration;

namespace jvms.datamodel.spd
{
	[Serializable]
	/// <summary>
	/// "The softpkgref element (see Figure D-7) refers to a softpkg element contained in another Software Package Descriptor file and indicates a file-load dependency on that file. The other file is referenced by the localfile element. An optional implref element refers to a particular implementation-unique identifier, within the Software Package Descriptor of the other file"
	/// 
	/// CL paraphrase: Before this implementation is loaded the component (or possibly the implementation, if specified) indicated by this element must have been loaded.  If it is already resident, great.  Otherwise it needs to be loaded if possible.  Only if one of these two conditions is met can this dependency succeed.
	/// 
	/// The localfile element in this case refers to the .spd file for the component that must be first resident.
	/// 
	/// The implref, if present, refers to the specific implementation reference (a uuid) that must be present.  Presumably this implref corresponds to one of the implementations in the localfile.
	/// 
	/// BJL - If this points to a SoftwarePkg in the system  then we can 
	/// have an attribute of type SoftwarePackage, if it needs to be able 
	/// to reference external software packages then it will need an 
	/// attribute for a LocalFile, same for can be inferred for the implref.  
	/// I will leave these out for now until we get some clarification.
	/// </summary>
	public class SoftwarePackageReference
	{

		//this will need to be resolved to a localfile when exported to xml
		private NonSCAComponent component = null;
		private Implementation implRef = null; //optional

		public NonSCAComponent Component
		{
			get { return component; }
			set { component = value; }
		}

		public Implementation ImplRef
		{
			get { return implRef; }
			set { implRef = value; }
		}

		/// <summary>
		/// This will convert this object to an XML element.
		/// </summary>
		/// <param name="document">The document that this element will go into.</param>
		/// <returns>The element representation of the object.</returns>
		public XmlElement toXML( XmlDocument document, FileSystemObject parent )
		{			
			XmlElement element = document.CreateElement( "notreallyused" );
			XMLHelper.writeFileReference( document, element, "softpkgref", parent, component );
			element = (XmlElement)element.FirstChild;
			if( implRef != null )
			{
				XmlElement implElement = document.CreateElement( "implref" );
				element.AppendChild( implElement );
				implElement.SetAttribute( "refid", implRef.ID.DCEUUID );
			}

			return element;
		}

	}
}
