using System;
using System.Collections;
using System.Xml;

using jvms.datamodel;
using jvms.datamodel.common;
using jvms.xmlgeneration;

namespace jvms.datamodel.spd
{
	[Serializable]
	/// <summary>
	/// "The usesdevice element describes any "uses" relationships this component has with a device in the system.  The propertyref element references allocation properties, which indicate the CF Device to be used, and/or the capacity needed from the CF Device to be used."
	/// 
	/// Good question... (I don't even care what your question was).  It seems that a usesdevice is specified when the component requires use of a device in the system (presumably other than a "deployed on" type of device, which is covered in the implementation section.  My guess is that the properties can either be to find a matching device or to set capacity on the device.  
	/// 
	/// Based on my reading of the Appendix D overview, it seems that the ID field is a unique Id for this element - it gets referenced by the SAD componentusedbythiscomponentref element. 
	/// 
	/// Got it?
	/// 
	/// 
	/// </summary>
	public class UsesDevice : XMLObject
	{
//		IList thePropertyRef;
 		private TypedArrayList propertyRefs = new TypedArrayList(typeof(SimpleRef));
		/// <summary>
		/// It is unclear what goes in type.  The only value in the example waveform is "uses".
		/// </summary>
		private String type = "";
		/// <summary>
		/// Based on my reading of the Appendix D overview, it seems that the ID 
		/// field is a unique Id for this element - it gets referenced by the SAD 
		/// componentusedbythiscomponentref element. 
		/// </summary>
		private String id = ""; //should be a uniqe value

		public TypedArrayList PropertyRefs
		{
			get { return propertyRefs; }
			set { propertyRefs = value; }
		}
		public String Type
		{
			get { return type; }
			set { type = value; }
		}
		public String ID
		{
			get { return id; }
			set { id = value; }
		}

		/// <summary>
		/// This will return a string representation of this class.
		/// </summary>
		/// <returns>This class represented in a human readable form.</returns>
		public override String ToString()
		{
			String retval = "<New UsesDevice>";
			if( !id.Equals( "" ) )
			{
				retval = id;
			}
			return retval;
		}

		/// <summary>
		/// This will convert this object to an XML element.
		/// </summary>
		/// <param name="document">The document that this element will go into.</param>
		/// <returns>The element representation of the object.</returns>
		public XmlElement toXML( XmlDocument document )
		{
			XmlElement element = document.CreateElement( "usesdevice" );
			element.SetAttribute( "id", id );
			element.SetAttribute( "type", type );

			foreach( SimpleRef simple in propertyRefs )
			{
				XmlElement simpleElement = document.CreateElement( "propertyref" );
				XMLHelper.writeAttribute( simpleElement, "refid", simple.Simple.ID.DCEUUID );
				XMLHelper.writeAttribute( simpleElement, "value", simple.Value );
				element.AppendChild( simpleElement );
			}

			return element;
		}
	}
}
