using System;
using System.Reflection;

using jvms.presentation;
using jvms.system;


namespace jvms
{
	/// <summary>
	/// Main class for JVMS, starts various application components and connects
	/// them together.
	/// </summary>
	public class JVMSMain
	{
		private static JVMSForm jvmsForm;
		private static String assemblyDir;

		/// <summary>Gets the main form for JVMS.</summary>
		/// <value>Main user interface window for JVMS.</value>
		public static JVMSForm JvmsForm
		{
			get
			{
				return jvmsForm;
			}
		}

		/// <summary>The absolute path to JVMS.exe</summary>
		/// <value>A string representing a file path.</value>
		public static String AssemblyDir
		{
			get
			{
				return assemblyDir;
			}
		}

		/// <summary>The absolute path to the resuorces directory.</summary>
		/// <value>A string representing a file path.</value>
		public static String ResourcesDir
		{
			get
			{
				return assemblyDir + "resources\\";
			}
		}

		/// <summary>
		/// Execution entry point.
		/// </summary>
		/// <param name="args">A list of command line arguments.</param>
		[STAThread]
		public static void Main( String[] args )
		{
			Assembly jvmsAssembly = Assembly.GetExecutingAssembly();
			assemblyDir = jvmsAssembly.Location.Substring( 0, jvmsAssembly.Location.LastIndexOf( '\\' ) + 1 );
			jvmsForm = new JVMSForm();
			StatusBarManager sbm = StatusBarManager.Instance;
			JVMSProperties props = JVMSProperties.Instance;
			ViewManager vm = ViewManager.Instance;

			SplashScreen splashScreen = new SplashScreen();
			splashScreen.showSplashScreen();

			if ( args.Length == 1 )
			{
				jvmsForm.LoadedFile = args[0];
			}

			System.Windows.Forms.Application.Run( jvmsForm );
		}
	}
}