using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

using jvms.datamodel;
using jvms.datamodel.scd;
using jvms.datamodel.prf;
using jvms.datamodel.common;
using jvms.system;

namespace jvms.presentation
{
	/// <summary>
	/// Summary description for ComplexXMLEditorForm.
	/// </summary>
	public class AllocPropsValueEditorForm : System.Windows.Forms.Form
	{
		#region Windows Form Designer generated state
		private System.Windows.Forms.Button addButton;
		private System.Windows.Forms.PropertyGrid propertyGrid;
		private System.Windows.Forms.GroupBox itemsGroupBox;
		private System.Windows.Forms.GroupBox attributesGroupBox;
		private System.Windows.Forms.ListBox itemsListBox;
		private System.Windows.Forms.GroupBox separator;
		private System.Windows.Forms.Button cancelButton;
		private System.Windows.Forms.Button okButton;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		#endregion
		private Object selected = null;

		private ArrayList allocList = null;
		private ArrayList instanceList = null;
		private System.Windows.Forms.GroupBox groupBox1;
		private System.Windows.Forms.ListBox listBox1;
		private System.Windows.Forms.Button button1;
		private ComponentProperties instanceProperties = null;
		private Hashtable map = new Hashtable();

		public AllocPropsValueEditorForm()
		{
			InitializeComponent();

			okButton.Click += new EventHandler( okButtonClicked );
			cancelButton.Click += new EventHandler( cancelButtonClicked );
			listBox1.SelectedIndexChanged += new EventHandler( itemSelectionChanged );
			propertyGrid.PropertyValueChanged += new PropertyValueChangedEventHandler( propertyValueChanged );
		}

		
		/// <summary>
		/// Sets the list which is displayed in the editor.
		/// </summary>
		/// <param name="list">The desired list.</param>
		public void SetAllocationProperties( Properties properties )
		{
			allocList = new ArrayList();
			if( properties != null )
			{
				allocList.AddRange( properties.Simples );
				allocList.AddRange( properties.SimpleSequences );
				allocList.AddRange( properties.Structs );
				allocList.AddRange( properties.StructSequences );
			}
			itemsListBox.Items.AddRange( allocList.ToArray() );
			if( allocList.Count > 0 )
			{
				itemsListBox.SetSelected( 0, true );
			}
		}

		/// <summary>
		/// 
		/// </summary>
		/// <returns></returns>
		public void SetInstanceProperties( ComponentProperties properties )
		{
			instanceProperties = properties;
			instanceList = new ArrayList();
			foreach( SimpleRef reference in properties.Simples )
			{
				instanceList.Add( reference );
				map.Add( reference, reference.Simple );
				if( allocList.Contains( reference.Simple ) )
				{
					allocList.Remove( reference.Simple );
					itemsListBox.Items.Remove( reference.Simple );
				}
			}
			foreach( SimpleSequenceRef reference in properties.SimpleSequences )
			{
				instanceList.Add( reference );
				map[reference] = reference.SimpleSequence;
				if( allocList.Contains( reference.SimpleSequence ) )
				{
					allocList.Remove( reference.SimpleSequence );
					itemsListBox.Items.Remove( reference.SimpleSequence );
				}
			}
			foreach( StructRef reference in properties.Structs )
			{
				instanceList.Add( reference );
				map[reference] = reference.Struct;
				if( allocList.Contains( reference.Struct ) )
				{
					allocList.Remove( reference.Struct );
					itemsListBox.Items.Remove( reference.Struct );
				}
			}
			foreach( StructSequenceRef reference in properties.StructSequences )
			{
				instanceList.Add( reference );
				map[reference] = reference.StructSequence;
				if( allocList.Contains( reference.StructSequence ) )
				{
					allocList.Remove( reference.StructSequence );
					itemsListBox.Items.Remove( reference.StructSequence );
				}
			}
			listBox1.Items.AddRange( instanceList.ToArray() );
			if( instanceList.Count > 0 )
			{
				listBox1.SetSelected( 0, true );
			}
			addButton.Enabled = (itemsListBox.SelectedItem != null);
			button1.Enabled = (listBox1.SelectedItem != null);
		}

		public ComponentProperties GetProperties()
		{
			//add them back in here
			instanceProperties.Simples.Clear();
			instanceProperties.SimpleSequences.Clear();
			instanceProperties.Structs.Clear();
			instanceProperties.StructSequences.Clear();
			foreach( Object obj in listBox1.Items )
			{
				Type type = obj.GetType();
				if( type.Equals(typeof(SimpleRef)) )
				{
					instanceProperties.Simples.Add( obj );
				}
				else if( type.Equals(typeof(SimpleSequenceRef)) )
				{
					instanceProperties.SimpleSequences.Add( obj );
				}
				else if( type.Equals(typeof(StructRef)) )
				{
					instanceProperties.Structs.Add( obj );
				}
				else if( type.Equals(typeof(StructSequenceRef)) )
				{
					instanceProperties.StructSequences.Add( obj );
				}
			}
			return instanceProperties;
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		/// <summary>
		/// Called when the OK button is clicked.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void okButtonClicked( Object source, EventArgs e )
		{
			this.DialogResult = DialogResult.OK;
			this.Close();
			View view = JVMSMain.JvmsForm.CurrentView;
			view.Refresh();
		}

		/// <summary>
		/// Called when the cancel button is clicked.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void cancelButtonClicked( Object source, EventArgs e )
		{
			this.DialogResult = DialogResult.Cancel;
			this.Close();
			View view = JVMSMain.JvmsForm.CurrentView;
			view.Refresh();
		}

		/// <summary>
		/// Called when the selected in the items list box is changed.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void itemSelectionChanged( Object source, EventArgs e )
		{
			SimpleRef obj = (SimpleRef)listBox1.SelectedItem;
			button1.Enabled = (obj != null);
			if ( obj != null )
			{
				SimpleRefReadOnly copy = new SimpleRefReadOnly( obj );
				AttributeSheet sheet = AttributeSheetFactory.Instance.createAttributeSheet(copy);
				propertyGrid.SelectedObject = sheet;
				selected = obj;
			}
		}

		/// <summary>
		/// Called when the value of an attribute has been changed.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void propertyValueChanged( Object source, PropertyValueChangedEventArgs e )
		{
			/*itemsListBox.SuspendLayout();
			itemsListBox.Items.Clear();
			itemsListBox.Items.AddRange( instanceList.ToArray() );
			itemsListBox.ResumeLayout();
			*/
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.itemsListBox = new System.Windows.Forms.ListBox();
			this.addButton = new System.Windows.Forms.Button();
			this.propertyGrid = new System.Windows.Forms.PropertyGrid();
			this.itemsGroupBox = new System.Windows.Forms.GroupBox();
			this.attributesGroupBox = new System.Windows.Forms.GroupBox();
			this.separator = new System.Windows.Forms.GroupBox();
			this.cancelButton = new System.Windows.Forms.Button();
			this.okButton = new System.Windows.Forms.Button();
			this.groupBox1 = new System.Windows.Forms.GroupBox();
			this.listBox1 = new System.Windows.Forms.ListBox();
			this.button1 = new System.Windows.Forms.Button();
			this.itemsGroupBox.SuspendLayout();
			this.groupBox1.SuspendLayout();
			this.SuspendLayout();
			// 
			// itemsListBox
			// 
			this.itemsListBox.Location = new System.Drawing.Point(16, 24);
			this.itemsListBox.Name = "itemsListBox";
			this.itemsListBox.SelectionMode = System.Windows.Forms.SelectionMode.MultiExtended;
			this.itemsListBox.Size = new System.Drawing.Size(176, 251);
			this.itemsListBox.TabIndex = 0;
			this.itemsListBox.SelectedIndexChanged += new System.EventHandler(this.itemsListBox_SelectedIndexChanged);
			// 
			// addButton
			// 
			this.addButton.Enabled = false;
			this.addButton.Location = new System.Drawing.Point(48, 280);
			this.addButton.Name = "addButton";
			this.addButton.Size = new System.Drawing.Size(112, 24);
			this.addButton.TabIndex = 1;
			this.addButton.Text = "Add Property";
			this.addButton.Click += new System.EventHandler(this.addButton_Click);
			// 
			// propertyGrid
			// 
			this.propertyGrid.CommandsVisibleIfAvailable = true;
			this.propertyGrid.LargeButtons = false;
			this.propertyGrid.LineColor = System.Drawing.SystemColors.ScrollBar;
			this.propertyGrid.Location = new System.Drawing.Point(472, 32);
			this.propertyGrid.Name = "propertyGrid";
			this.propertyGrid.PropertySort = System.Windows.Forms.PropertySort.Categorized;
			this.propertyGrid.Size = new System.Drawing.Size(248, 280);
			this.propertyGrid.TabIndex = 3;
			this.propertyGrid.Text = "propertyGrid";
			this.propertyGrid.ViewBackColor = System.Drawing.SystemColors.Window;
			this.propertyGrid.ViewForeColor = System.Drawing.SystemColors.WindowText;
			// 
			// itemsGroupBox
			// 
			this.itemsGroupBox.Controls.AddRange(new System.Windows.Forms.Control[] {
																						this.itemsListBox,
																						this.addButton});
			this.itemsGroupBox.Location = new System.Drawing.Point(8, 8);
			this.itemsGroupBox.Name = "itemsGroupBox";
			this.itemsGroupBox.Size = new System.Drawing.Size(208, 320);
			this.itemsGroupBox.TabIndex = 4;
			this.itemsGroupBox.TabStop = false;
			this.itemsGroupBox.Text = "Allocation Properties";
			// 
			// attributesGroupBox
			// 
			this.attributesGroupBox.Location = new System.Drawing.Point(456, 8);
			this.attributesGroupBox.Name = "attributesGroupBox";
			this.attributesGroupBox.Size = new System.Drawing.Size(280, 320);
			this.attributesGroupBox.TabIndex = 5;
			this.attributesGroupBox.TabStop = false;
			this.attributesGroupBox.Text = "Attributes";
			// 
			// separator
			// 
			this.separator.Location = new System.Drawing.Point(8, 336);
			this.separator.Name = "separator";
			this.separator.Size = new System.Drawing.Size(736, 8);
			this.separator.TabIndex = 6;
			this.separator.TabStop = false;
			// 
			// cancelButton
			// 
			this.cancelButton.Location = new System.Drawing.Point(656, 352);
			this.cancelButton.Name = "cancelButton";
			this.cancelButton.Size = new System.Drawing.Size(75, 24);
			this.cancelButton.TabIndex = 7;
			this.cancelButton.Text = "Cancel";
			// 
			// okButton
			// 
			this.okButton.Location = new System.Drawing.Point(568, 352);
			this.okButton.Name = "okButton";
			this.okButton.Size = new System.Drawing.Size(75, 24);
			this.okButton.TabIndex = 8;
			this.okButton.Text = "OK";
			// 
			// groupBox1
			// 
			this.groupBox1.Controls.AddRange(new System.Windows.Forms.Control[] {
																					this.listBox1,
																					this.button1});
			this.groupBox1.Location = new System.Drawing.Point(232, 8);
			this.groupBox1.Name = "groupBox1";
			this.groupBox1.Size = new System.Drawing.Size(208, 320);
			this.groupBox1.TabIndex = 5;
			this.groupBox1.TabStop = false;
			this.groupBox1.Text = "Instance Properties";
			// 
			// listBox1
			// 
			this.listBox1.Location = new System.Drawing.Point(16, 24);
			this.listBox1.Name = "listBox1";
			this.listBox1.SelectionMode = System.Windows.Forms.SelectionMode.MultiExtended;
			this.listBox1.Size = new System.Drawing.Size(176, 251);
			this.listBox1.TabIndex = 0;
			// 
			// button1
			// 
			this.button1.Enabled = false;
			this.button1.Location = new System.Drawing.Point(48, 280);
			this.button1.Name = "button1";
			this.button1.Size = new System.Drawing.Size(112, 24);
			this.button1.TabIndex = 1;
			this.button1.Text = "Remove Property";
			this.button1.Click += new System.EventHandler(this.button1_Click);
			// 
			// AllocPropsValueEditorForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(746, 383);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.okButton,
																		  this.cancelButton,
																		  this.separator,
																		  this.itemsGroupBox,
																		  this.propertyGrid,
																		  this.attributesGroupBox,
																		  this.groupBox1});
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.Name = "AllocPropsValueEditorForm";
			this.ShowInTaskbar = false;
			this.Text = "Allocation Properties Dialog";
			this.itemsGroupBox.ResumeLayout(false);
			this.groupBox1.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		private void addButton_Click(object sender, System.EventArgs e)
		{
			Object obj = itemsListBox.SelectedItem;
			if ( obj != null )
			{
				//this needs to be a copy of the object
				Object newObj = null;
				Type type = obj.GetType();
				if( type.Equals(typeof(Simple)) )
				{
					newObj = new SimpleRef();
					((SimpleRef)newObj).Simple = (Simple)obj;
				}
				else if( type.Equals(typeof(SimpleSequence)) )
				{
					newObj = new SimpleSequenceRef();
					((SimpleSequenceRef)newObj).SimpleSequence = (SimpleSequence)obj;
				}
				else if( type.Equals(typeof(Struct)) )
				{
					newObj = new StructRef();
					((StructRef)newObj).Struct = (Struct)obj;
					foreach( Simple simple in ((Struct)obj).Simples )
					{
						SimpleRef simpleRef = new SimpleRef();
						simpleRef.Simple = simple;
						((StructRef)newObj).SimpleRefs.Add( simpleRef );
					}
				}
				else if( type.Equals(typeof(StructSequence)) )
				{
					newObj = new StructSequenceRef();
					((StructSequenceRef)newObj).StructSequence = (StructSequence)obj;
					foreach( Simple simple in ((StructSequence)obj).StructValues )
					{
						SimpleRef simpleRef = new SimpleRef();
						simpleRef.Simple = simple;
						((StructSequenceRef)newObj).StructValues.Add( simpleRef );
					}
				}
				listBox1.Items.Add( newObj );
				itemsListBox.Items.Remove( obj );
				map.Add( newObj, obj );
			}
		}

		private void button1_Click(object sender, System.EventArgs e)
		{
			Object obj = listBox1.SelectedItem;
			if ( obj != null )
			{
				itemsListBox.Items.Add( map[obj] );
				listBox1.Items.Remove( obj );
				propertyGrid.SelectedObject = null;
			}
		}

		private void itemsListBox_SelectedIndexChanged(object sender, System.EventArgs e)
		{
			addButton.Enabled = (itemsListBox.SelectedItem != null);
		}
	}
}
