using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;

using jvms.project;
using jvms.validation;

namespace jvms.presentation
{
	/// <summary>
	/// Represents the sheet control that allows designers to edit
	/// the attributes of a component.
	/// </summary>
	public class AttributeSheetControl : System.Windows.Forms.UserControl
	{
		#region Component Designer generated state
		private System.Windows.Forms.TabControl tabControl;
		private System.Windows.Forms.TabPage attrGridPage;
		private System.Windows.Forms.TabPage helpPage;
		private System.Windows.Forms.PropertyGrid attributeGrid;
		/// <summary> 
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		#endregion

		/// <summary>Gets or sets the collection of attributes displayed on the sheet.</summary>
		/// <value>The current attribute collection.</value>
		public AttributeSheet AttributeSheet
		{
			get
			{
				return (AttributeSheet)attributeGrid.SelectedObject;
			}
			
			set
			{
				attributeGrid.SelectedObject = value;
				resetSelection();
			}
		}

		/// <summary>
		/// Creates a new AttributeSheetControl.
		/// </summary>
		public AttributeSheetControl()
		{
			InitializeComponent();
			attributeGrid.PropertyValueChanged += new PropertyValueChangedEventHandler( propertyValueChanged );
        }

		/// <summary>
		/// Clears the attribute sheet, so it displays no attributes.
		/// </summary>
		public void clearAttributeSheet()
		{
			attributeGrid.SelectedObject = null;
		}

		/// <summary>
		/// Resets the selected item in the property grid.
		/// </summary>
		public void resetSelection()
		{
			GridItem selectedItem = attributeGrid.SelectedGridItem;
			if ( selectedItem != null )
			{
				GridItem rootItem = selectedItem;
				while ( rootItem.GridItemType != GridItemType.Root )
				{
					rootItem = rootItem.Parent;
				}

				if ( rootItem.GridItems.Count != 0 )
				{
					attributeGrid.SelectedGridItem = rootItem.GridItems[0];
				}
			}
		}

		/// <summary>
		/// Called when the value of a property is changed using the attribute sheet.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void propertyValueChanged( Object source, PropertyValueChangedEventArgs e )
		{
			JVMSMain.JvmsForm.CurrentView.Refresh();
			ProjectManager.Instance.ChangedFlag = true;
			JVMSMain.JvmsForm.updateTitle();
		}

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.tabControl = new System.Windows.Forms.TabControl();
			this.attrGridPage = new System.Windows.Forms.TabPage();
			this.attributeGrid = new System.Windows.Forms.PropertyGrid();
			this.helpPage = new System.Windows.Forms.TabPage();
			this.tabControl.SuspendLayout();
			this.attrGridPage.SuspendLayout();
			this.SuspendLayout();
			// 
			// tabControl
			// 
			this.tabControl.Alignment = System.Windows.Forms.TabAlignment.Bottom;
			this.tabControl.Controls.AddRange(new System.Windows.Forms.Control[] {
																					 this.attrGridPage,
																					 this.helpPage});
			this.tabControl.Dock = System.Windows.Forms.DockStyle.Fill;
			this.tabControl.Multiline = true;
			this.tabControl.Name = "tabControl";
			this.tabControl.SelectedIndex = 0;
			this.tabControl.Size = new System.Drawing.Size(208, 368);
			this.tabControl.TabIndex = 0;
			// 
			// attrGridPage
			// 
			this.attrGridPage.Controls.AddRange(new System.Windows.Forms.Control[] {
																					   this.attributeGrid});
			this.attrGridPage.Location = new System.Drawing.Point(4, 4);
			this.attrGridPage.Name = "attrGridPage";
			this.attrGridPage.Size = new System.Drawing.Size(200, 342);
			this.attrGridPage.TabIndex = 0;
			this.attrGridPage.Text = "Attributes";
			// 
			// attributeGrid
			// 
			this.attributeGrid.CommandsVisibleIfAvailable = true;
			this.attributeGrid.Dock = System.Windows.Forms.DockStyle.Fill;
			this.attributeGrid.LargeButtons = false;
			this.attributeGrid.LineColor = System.Drawing.SystemColors.ScrollBar;
			this.attributeGrid.Name = "attributeGrid";
			this.attributeGrid.PropertySort = System.Windows.Forms.PropertySort.Categorized;
			this.attributeGrid.Size = new System.Drawing.Size(200, 342);
			this.attributeGrid.TabIndex = 0;
			this.attributeGrid.Text = "propertyGrid1";
			this.attributeGrid.ViewBackColor = System.Drawing.SystemColors.Window;
			this.attributeGrid.ViewForeColor = System.Drawing.SystemColors.WindowText;
			// 
			// helpPage
			// 
			this.helpPage.Location = new System.Drawing.Point(4, 4);
			this.helpPage.Name = "helpPage";
			this.helpPage.Size = new System.Drawing.Size(200, 342);
			this.helpPage.TabIndex = 1;
			this.helpPage.Text = "Help";
			// 
			// AttributeSheetControl
			// 
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.tabControl});
			this.Name = "AttributeSheetControl";
			this.Size = new System.Drawing.Size(208, 368);
			this.tabControl.ResumeLayout(false);
			this.attrGridPage.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

	}
}
