using System;
using System.Collections;

using jvms.datamodel;
using jvms.validation;

namespace jvms.presentation
{
	/// <summary>
	/// Constructs AttributeSheetControl objects given a component object.
	/// </summary>
	public class AttributeSheetFactory
	{
		private AttributeService attributeService = AttributeService.Instance;
		private static AttributeSheetFactory instance = null;

		private AttributeSheetFactory()
		{
		}

		public static AttributeSheetFactory Instance
		{
			get
			{
				if( instance == null )
				{
					instance = new AttributeSheetFactory();
				}
				return instance;
			}
		}
		
		/// <summary>
		/// Returns a collection of attributes for the given object.
		/// </summary>
		/// <param name="component">The component to create the attribute list for.</param>
		/// <returns>A collection of attributes.</returns>
		public AttributeSheet createAttributeSheet( Object component )
		{
			IList attributeTypes = attributeService.QueryValues( component );
			AttributeSheet sheet = new AttributeSheet( attributeTypes );
			sheet.WorkingObject = component;
			return sheet;
		}
	}
}
