using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

using jvms.datamodel;
using jvms.project;
using jvms.system;

namespace jvms.presentation
{
	/// <summary>
	/// Summary description for ComplexXMLEditorForm.
	/// </summary>
	public class ComplexXMLEditorForm : System.Windows.Forms.Form
	{
		#region Windows Form Designer generated state
		private System.Windows.Forms.Button addButton;
		private System.Windows.Forms.Button delButton;
		private System.Windows.Forms.PropertyGrid propertyGrid;
		private System.Windows.Forms.GroupBox itemsGroupBox;
		private System.Windows.Forms.GroupBox attributesGroupBox;
		private System.Windows.Forms.ListBox itemsListBox;
		private System.Windows.Forms.GroupBox separator;
		private System.Windows.Forms.Button cancelButton;
		private System.Windows.Forms.Button okButton;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		#endregion

		private TypedArrayList list = null;
		private Object[] backupList = null;

		/// <summary>
		/// Creates a new ComplexXMLEditorForm.
		/// </summary>
		public ComplexXMLEditorForm()
		{
			InitializeComponent();

			okButton.Click += new EventHandler( okButtonClicked );
			cancelButton.Click += new EventHandler( cancelButtonClicked );
			addButton.Click += new EventHandler( addButtonClicked );
			delButton.Click += new EventHandler( deleteButtonClicked );
			itemsListBox.SelectedIndexChanged += new EventHandler( itemSelectionChanged );
			itemsListBox.KeyDown += new KeyEventHandler( listKeyDown );
			propertyGrid.PropertyValueChanged += new PropertyValueChangedEventHandler( propertyValueChanged );
		}

		
		/// <summary>
		/// Sets the list which is displayed in the editor.
		/// </summary>
		/// <param name="list">The desired list.</param>
		public void SetList( TypedArrayList list )
		{
			this.list = list;
			backupList = new Object[list.Count];
			list.CopyTo( backupList, 0 );
            itemsListBox.Items.AddRange( list.ToArray() );
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		/// <summary>
		/// Resets the selection in the attribute grid.
		/// </summary>
		private void resetAttributeSelection()
		{
			GridItem selectedItem = propertyGrid.SelectedGridItem;
			if ( selectedItem != null )
			{
				GridItem rootItem = selectedItem;
				while ( rootItem.GridItemType != GridItemType.Root )
				{
					rootItem = rootItem.Parent;
				}

				if ( rootItem.GridItems.Count != 0 )
				{
					propertyGrid.SelectedGridItem = rootItem.GridItems[0];
				}
			}
		}

		/// <summary>
		/// Called when the add button is clicked.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void addButtonClicked( Object source, EventArgs e )
		{
			Object obj = list.CreateInstanceOfType();
			itemsListBox.Items.Add( obj );
			itemsListBox.ClearSelected();
			itemsListBox.SelectedItem = obj;
			list.Add( obj );
			propertyGrid.Focus();
		}

		/// <summary>
		/// Called when the delete button is clicked.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void deleteButtonClicked( Object source, EventArgs e )
		{
			Object[] selectedItems = new Object[itemsListBox.Items.Count];
			itemsListBox.SelectedItems.CopyTo( selectedItems, 0 );
			for ( int i = 0; i < selectedItems.Length; i += 1 )
			{
				itemsListBox.Items.Remove( selectedItems[i] );
				list.Remove( selectedItems[i] );
			}
			propertyGrid.SelectedObject = null;
		}

		/// <summary>
		/// Called when the OK button is clicked.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void okButtonClicked( Object source, EventArgs e )
		{
			this.Close();
			View view = JVMSMain.JvmsForm.CurrentView;
			view.Refresh();
		}

		/// <summary>
		/// Called when the cancel button is clicked.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void cancelButtonClicked( Object source, EventArgs e )
		{
			this.Close();
			this.list.Clear();
			this.list.AddRange( backupList );
			View view = JVMSMain.JvmsForm.CurrentView;
			view.Refresh();
		}

		/// <summary>
		/// Called when the selected in the items list box is changed.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void itemSelectionChanged( Object source, EventArgs e )
		{
			Object obj = itemsListBox.SelectedItem;
			if ( obj != null )
			{
				AttributeSheet sheet = AttributeSheetFactory.Instance.createAttributeSheet(obj);
				propertyGrid.SelectedObject = sheet;
				resetAttributeSelection();
				propertyGrid.Focus();
			}
		}

		/// <summary>
		/// Called when the value of an attribute has been changed.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void propertyValueChanged( Object source, PropertyValueChangedEventArgs e )
		{
			itemsListBox.SuspendLayout();
			itemsListBox.Items.Clear();
			itemsListBox.Items.AddRange( list.ToArray() );
			itemsListBox.ResumeLayout();
			ProjectManager.Instance.ChangedFlag = true;
			JVMSMain.JvmsForm.updateTitle();
		}

		/// <summary>
		/// Called when a key is pressed on the list.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void listKeyDown( Object source, KeyEventArgs e )
		{
			if ( e.KeyCode == Keys.A && e.Control )
			{
				addButton.PerformClick();
			}
			else if ( e.KeyCode == Keys.D && e.Control )
			{
				delButton.PerformClick();
			}
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.itemsListBox = new System.Windows.Forms.ListBox();
			this.addButton = new System.Windows.Forms.Button();
			this.delButton = new System.Windows.Forms.Button();
			this.propertyGrid = new System.Windows.Forms.PropertyGrid();
			this.itemsGroupBox = new System.Windows.Forms.GroupBox();
			this.attributesGroupBox = new System.Windows.Forms.GroupBox();
			this.separator = new System.Windows.Forms.GroupBox();
			this.cancelButton = new System.Windows.Forms.Button();
			this.okButton = new System.Windows.Forms.Button();
			this.itemsGroupBox.SuspendLayout();
			this.attributesGroupBox.SuspendLayout();
			this.SuspendLayout();
			// 
			// itemsListBox
			// 
			this.itemsListBox.Location = new System.Drawing.Point(16, 24);
			this.itemsListBox.Name = "itemsListBox";
			this.itemsListBox.SelectionMode = System.Windows.Forms.SelectionMode.MultiExtended;
			this.itemsListBox.Size = new System.Drawing.Size(176, 251);
			this.itemsListBox.TabIndex = 0;
			// 
			// addButton
			// 
			this.addButton.Location = new System.Drawing.Point(16, 280);
			this.addButton.Name = "addButton";
			this.addButton.Size = new System.Drawing.Size(72, 24);
			this.addButton.TabIndex = 1;
			this.addButton.Text = "Add";
			// 
			// delButton
			// 
			this.delButton.Location = new System.Drawing.Point(120, 280);
			this.delButton.Name = "delButton";
			this.delButton.Size = new System.Drawing.Size(72, 24);
			this.delButton.TabIndex = 2;
			this.delButton.Text = "Remove";
			// 
			// propertyGrid
			// 
			this.propertyGrid.CommandsVisibleIfAvailable = true;
			this.propertyGrid.LargeButtons = false;
			this.propertyGrid.LineColor = System.Drawing.SystemColors.ScrollBar;
			this.propertyGrid.Location = new System.Drawing.Point(16, 24);
			this.propertyGrid.Name = "propertyGrid";
			this.propertyGrid.PropertySort = System.Windows.Forms.PropertySort.Categorized;
			this.propertyGrid.Size = new System.Drawing.Size(248, 280);
			this.propertyGrid.TabIndex = 3;
			this.propertyGrid.Text = "propertyGrid";
			this.propertyGrid.ViewBackColor = System.Drawing.SystemColors.Window;
			this.propertyGrid.ViewForeColor = System.Drawing.SystemColors.WindowText;
			// 
			// itemsGroupBox
			// 
			this.itemsGroupBox.Controls.AddRange(new System.Windows.Forms.Control[] {
																						this.itemsListBox,
																						this.addButton,
																						this.delButton});
			this.itemsGroupBox.Location = new System.Drawing.Point(8, 8);
			this.itemsGroupBox.Name = "itemsGroupBox";
			this.itemsGroupBox.Size = new System.Drawing.Size(208, 320);
			this.itemsGroupBox.TabIndex = 4;
			this.itemsGroupBox.TabStop = false;
			this.itemsGroupBox.Text = "Items";
			// 
			// attributesGroupBox
			// 
			this.attributesGroupBox.Controls.AddRange(new System.Windows.Forms.Control[] {
																							 this.propertyGrid});
			this.attributesGroupBox.Location = new System.Drawing.Point(232, 8);
			this.attributesGroupBox.Name = "attributesGroupBox";
			this.attributesGroupBox.Size = new System.Drawing.Size(280, 320);
			this.attributesGroupBox.TabIndex = 5;
			this.attributesGroupBox.TabStop = false;
			this.attributesGroupBox.Text = "Attributes";
			// 
			// separator
			// 
			this.separator.Location = new System.Drawing.Point(8, 336);
			this.separator.Name = "separator";
			this.separator.Size = new System.Drawing.Size(504, 8);
			this.separator.TabIndex = 6;
			this.separator.TabStop = false;
			// 
			// cancelButton
			// 
			this.cancelButton.Location = new System.Drawing.Point(440, 352);
			this.cancelButton.Name = "cancelButton";
			this.cancelButton.Size = new System.Drawing.Size(75, 24);
			this.cancelButton.TabIndex = 8;
			this.cancelButton.Text = "Cancel";
			// 
			// okButton
			// 
			this.okButton.Location = new System.Drawing.Point(352, 352);
			this.okButton.Name = "okButton";
			this.okButton.Size = new System.Drawing.Size(75, 24);
			this.okButton.TabIndex = 7;
			this.okButton.Text = "OK";
			// 
			// ComplexXMLEditorForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(522, 383);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.okButton,
																		  this.cancelButton,
																		  this.separator,
																		  this.attributesGroupBox,
																		  this.itemsGroupBox});
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.Name = "ComplexXMLEditorForm";
			this.ShowInTaskbar = false;
			this.Text = "Editor Dialog";
			this.itemsGroupBox.ResumeLayout(false);
			this.attributesGroupBox.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

	}
}
