using System;
using System.Collections;
using System.ComponentModel;
using System.Reflection;
using System.Drawing;
using System.Drawing.Design;
using System.Windows.Forms;
using System.Windows.Forms.Design;

using jvms.datamodel.scd;
using jvms.datamodel.spd;
using jvms.datamodel;
using jvms.project;
using jvms.system;
using jvms.validation;
using jvms.presentation.drawablecomponent;

namespace jvms.presentation
{
	/// <summary>
	/// Creates an editor for port types.
	/// </summary>
	public class ComponentListValueEditor : UITypeEditor
	{
		private IWindowsFormsEditorService editorService;

		/// <summary>
		/// Creates a new PortTypeValueEditor.
		/// </summary>
		public ComponentListValueEditor()
		{
		}

		/// <summary>
		/// Edit's the specified object's value.
		/// </summary>
		/// <param name="context">Provides contextual information about the object.</param>
		/// <param name="provider">Can be used by the editor to obtain services.</param>
		/// <param name="value">The object to be edited.</param>
		/// <returns>The new value of the object.</returns>
		public override object EditValue( ITypeDescriptorContext context, IServiceProvider provider, Object value )
		{
			editorService = (IWindowsFormsEditorService)provider.GetService( typeof( IWindowsFormsEditorService ) );
			ListBox listBox = new ListBox();
			listBox.BorderStyle = BorderStyle.None;
			listBox.SelectedIndexChanged += new EventHandler( itemChanged );

			ICollection itemsForList = ProjectManager.Instance.Project.Components;
			
			foreach( Object obj in itemsForList )
			{
				listBox.Items.Add( obj );
			}
			
			//remove ourselves from the list
			DrawableComponent selected = JVMSMain.JvmsForm.SelectedComponent;
			if( selected != null )
			{
				listBox.Items.Remove( selected.getWrappedObject() );
			}
			
			NonSCAComponent currentValue = (NonSCAComponent)value;
			listBox.SelectedItem = currentValue;


			editorService.DropDownControl( listBox );
			if( listBox.SelectedItem != null )
			{
				currentValue = (NonSCAComponent)listBox.SelectedItem;
			}
			return currentValue;
		}

		private void itemChanged( Object sender, EventArgs e )
		{
			editorService.CloseDropDown();
		}

		/// <summary>
		/// Gets the editor style that will be used to edit the type.
		/// </summary>
		/// <param name="context">Provides contextual information.</param>
		/// <returns>The UITypeEditorEditStyle that will be used.</returns>
		public override UITypeEditorEditStyle GetEditStyle( ITypeDescriptorContext context )
		{
			return UITypeEditorEditStyle.DropDown;
		}
	}
}
