using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;

using jvms.system;

namespace jvms.presentation
{
	/// <summary>
	/// Summary description for ControlPane.
	/// </summary>
	public class ControlPane : System.Windows.Forms.UserControl
	{
		#region Component Designer generated state
		private jvms.presentation.JVMSToolBox jvmsToolBox;
		private System.Windows.Forms.Splitter controlPaneSplitter;
		private jvms.presentation.AttributeSheetControl attributeSheetControl;
		/// <summary> 
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		#endregion	

		/// <summary>Gets or sets the attribute sheet control that is shown on the control pane.</summary>
		/// <value>The attribute sheet control.</value>
		public AttributeSheetControl CurrentAttributeSheetControl
		{
			get
			{
				return this.attributeSheetControl;
			}

			set
			{
				this.attributeSheetControl = value;
				this.attributeSheetControl.Invalidate();
			}
		}

		/// <summary>Gets the toolbox control.</summary>
		/// <value>The toolbox control.</value>
		public JVMSToolBox ToolBoxControl
		{
			get
			{
				return this.jvmsToolBox;
			}
		}

        /// <summary>
        /// Creates a new ControlPane.
        /// </summary>
		public ControlPane()
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();
			loadPreferences();
		}

		/// <summary>
		/// Called when the control should set related properties.
		/// </summary>
		public void setPreferences()
		{
			JVMSProperties props = JVMSProperties.Instance;
			props.setProperty( "controlpane.splitposition", this.controlPaneSplitter.SplitPosition );
		}

		/// <summary>
		/// Applies user preferences to the control.
		/// </summary>
		private void loadPreferences()
		{
			JVMSProperties props = JVMSProperties.Instance;
			
			this.controlPaneSplitter.SplitPosition = props.getProperty( "controlpane.splitposition", 300 );
		}

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if( components != null )
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.jvmsToolBox = new jvms.presentation.JVMSToolBox();
			this.controlPaneSplitter = new System.Windows.Forms.Splitter();
			this.attributeSheetControl = new jvms.presentation.AttributeSheetControl();
			this.SuspendLayout();
			// 
			// jvmsToolBox
			// 
			this.jvmsToolBox.Dock = System.Windows.Forms.DockStyle.Top;
			this.jvmsToolBox.Name = "jvmsToolBox";
			this.jvmsToolBox.Size = new System.Drawing.Size(224, 216);
			this.jvmsToolBox.TabIndex = 0;
			// 
			// controlPaneSplitter
			// 
			this.controlPaneSplitter.Dock = System.Windows.Forms.DockStyle.Top;
			this.controlPaneSplitter.Location = new System.Drawing.Point(0, 216);
			this.controlPaneSplitter.Name = "controlPaneSplitter";
			this.controlPaneSplitter.Size = new System.Drawing.Size(224, 3);
			this.controlPaneSplitter.TabIndex = 1;
			this.controlPaneSplitter.TabStop = false;
			// 
			// attributeSheetControl
			// 
			this.attributeSheetControl.AttributeSheet = null;
			this.attributeSheetControl.Dock = System.Windows.Forms.DockStyle.Fill;
			this.attributeSheetControl.Location = new System.Drawing.Point(0, 219);
			this.attributeSheetControl.Name = "attributeSheetControl";
			this.attributeSheetControl.Size = new System.Drawing.Size(224, 309);
			this.attributeSheetControl.TabIndex = 2;
			// 
			// ControlPane
			// 
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.attributeSheetControl,
																		  this.controlPaneSplitter,
																		  this.jvmsToolBox});
			this.Name = "ControlPane";
			this.Size = new System.Drawing.Size(224, 528);
			this.ResumeLayout(false);

		}
		#endregion

	}
}
