using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

using jvms.datamodel;
using jvms.datamodel.common;
using jvms.datamodel.spd;
using jvms.project;
using jvms.system;

namespace jvms.presentation
{
	/// <summary>
	/// Summary description for ComplexXMLEditorForm.
	/// </summary>
	public class DependencyEditorForm : System.Windows.Forms.Form
	{
		#region Windows Form Designer generated state
		private System.Windows.Forms.Button addButton;
		private System.Windows.Forms.Button delButton;
		private System.Windows.Forms.PropertyGrid propertyGrid;
		private System.Windows.Forms.GroupBox itemsGroupBox;
		private System.Windows.Forms.GroupBox attributesGroupBox;
		private System.Windows.Forms.ListBox itemsListBox;
		private System.Windows.Forms.GroupBox separator;
		private System.Windows.Forms.Button cancelButton;
		private System.Windows.Forms.Button okButton;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		#endregion

		private TypedArrayList list = null;
		private System.Windows.Forms.RadioButton radioButton1;
		private System.Windows.Forms.RadioButton radioButton2;
		private System.Windows.Forms.GroupBox groupBox1;
		private Object[] backupList = null;
		private System.Windows.Forms.TextBox textBox1;
		private System.Windows.Forms.Label label1;
		private Object currentSelectedObject = null;

		public DependencyEditorForm()
		{
			InitializeComponent();

			okButton.Click += new EventHandler( okButtonClicked );
			cancelButton.Click += new EventHandler( cancelButtonClicked );
			addButton.Click += new EventHandler( addButtonClicked );
			delButton.Click += new EventHandler( deleteButtonClicked );
			itemsListBox.SelectedIndexChanged += new EventHandler( itemSelectionChanged );
			propertyGrid.PropertyValueChanged += new PropertyValueChangedEventHandler( propertyValueChanged );
		}

		
		/// <summary>
		/// Sets the list which is displayed in the editor.
		/// </summary>
		/// <param name="list">The desired list.</param>
		public void SetList( TypedArrayList list )
		{
			this.list = list;
			backupList = new Object[list.Count];
			list.CopyTo( backupList, 0 );
            itemsListBox.Items.AddRange( list.ToArray() );
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		/// <summary>
		/// Called when the add button is clicked.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void addButtonClicked( Object source, EventArgs e )
		{
			Object obj = list.CreateInstanceOfType();
			itemsListBox.Items.Add( obj );
			itemsListBox.ClearSelected();
			itemsListBox.SelectedItem = obj;
			list.Add( obj );
		}

		/// <summary>
		/// Called when the delete button is clicked.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void deleteButtonClicked( Object source, EventArgs e )
		{
			Object[] selectedItems = new Object[itemsListBox.Items.Count];
			itemsListBox.SelectedItems.CopyTo( selectedItems, 0 );
			for ( int i = 0; i < selectedItems.Length; i += 1 )
			{
				itemsListBox.Items.Remove( selectedItems[i] );
				list.Remove( selectedItems[i] );
			}
			propertyGrid.SelectedObject = null;
		}

		/// <summary>
		/// Called when the OK button is clicked.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void okButtonClicked( Object source, EventArgs e )
		{
			this.Close();
			View view = JVMSMain.JvmsForm.CurrentView;
			view.Refresh();
		}

		/// <summary>
		/// Called when the cancel button is clicked.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void cancelButtonClicked( Object source, EventArgs e )
		{
			this.Close();
			this.list.Clear();
			this.list.AddRange( backupList );
			View view = JVMSMain.JvmsForm.CurrentView;
			view.Refresh();
		}

		/// <summary>
		/// Called when the selected in the items list box is changed.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void itemSelectionChanged( Object source, EventArgs e )
		{
			Dependency dep = (Dependency)itemsListBox.SelectedItem;
			currentSelectedObject = dep;
			if ( dep != null )
			{
				textBox1.Text = dep.Type;
				AttributeSheet sheet;
				if( dep.PropertyRef != null )
				{
					if( dep.PropertyRef == null )
					{
						dep.PropertyRef = new SimpleRef();
					}
					sheet = AttributeSheetFactory.Instance.createAttributeSheet(dep.PropertyRef);
					dep.SoftwareDependency = null;
					radioButton2.Checked = true;
				}
				else
				{
					if( dep.SoftwareDependency == null )
					{
						dep.SoftwareDependency = new SoftwarePackageReference();
					}
					sheet = AttributeSheetFactory.Instance.createAttributeSheet(dep.SoftwareDependency);
					dep.PropertyRef = null;
					radioButton1.Checked = true;
				}
				propertyGrid.SelectedObject = sheet;
			}
		}

		/// <summary>
		/// Called when the value of an attribute has been changed.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void propertyValueChanged( Object source, PropertyValueChangedEventArgs e )
		{
			itemsListBox.SuspendLayout();
			itemsListBox.Items.Clear();
			itemsListBox.Items.AddRange( list.ToArray() );
			itemsListBox.ResumeLayout();
			ProjectManager.Instance.ChangedFlag = true;
			JVMSMain.JvmsForm.updateTitle();
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.itemsListBox = new System.Windows.Forms.ListBox();
			this.addButton = new System.Windows.Forms.Button();
			this.delButton = new System.Windows.Forms.Button();
			this.propertyGrid = new System.Windows.Forms.PropertyGrid();
			this.itemsGroupBox = new System.Windows.Forms.GroupBox();
			this.attributesGroupBox = new System.Windows.Forms.GroupBox();
			this.groupBox1 = new System.Windows.Forms.GroupBox();
			this.radioButton2 = new System.Windows.Forms.RadioButton();
			this.radioButton1 = new System.Windows.Forms.RadioButton();
			this.separator = new System.Windows.Forms.GroupBox();
			this.cancelButton = new System.Windows.Forms.Button();
			this.okButton = new System.Windows.Forms.Button();
			this.textBox1 = new System.Windows.Forms.TextBox();
			this.label1 = new System.Windows.Forms.Label();
			this.itemsGroupBox.SuspendLayout();
			this.attributesGroupBox.SuspendLayout();
			this.groupBox1.SuspendLayout();
			this.SuspendLayout();
			// 
			// itemsListBox
			// 
			this.itemsListBox.Location = new System.Drawing.Point(8, 24);
			this.itemsListBox.Name = "itemsListBox";
			this.itemsListBox.SelectionMode = System.Windows.Forms.SelectionMode.MultiExtended;
			this.itemsListBox.Size = new System.Drawing.Size(176, 303);
			this.itemsListBox.TabIndex = 0;
			// 
			// addButton
			// 
			this.addButton.Location = new System.Drawing.Point(16, 336);
			this.addButton.Name = "addButton";
			this.addButton.Size = new System.Drawing.Size(72, 24);
			this.addButton.TabIndex = 1;
			this.addButton.Text = "Add";
			// 
			// delButton
			// 
			this.delButton.Location = new System.Drawing.Point(104, 336);
			this.delButton.Name = "delButton";
			this.delButton.Size = new System.Drawing.Size(72, 24);
			this.delButton.TabIndex = 2;
			this.delButton.Text = "Remove";
			// 
			// propertyGrid
			// 
			this.propertyGrid.CommandsVisibleIfAvailable = true;
			this.propertyGrid.LargeButtons = false;
			this.propertyGrid.LineColor = System.Drawing.SystemColors.ScrollBar;
			this.propertyGrid.Location = new System.Drawing.Point(16, 24);
			this.propertyGrid.Name = "propertyGrid";
			this.propertyGrid.PropertySort = System.Windows.Forms.PropertySort.Categorized;
			this.propertyGrid.Size = new System.Drawing.Size(248, 280);
			this.propertyGrid.TabIndex = 3;
			this.propertyGrid.Text = "propertyGrid";
			this.propertyGrid.ViewBackColor = System.Drawing.SystemColors.Window;
			this.propertyGrid.ViewForeColor = System.Drawing.SystemColors.WindowText;
			// 
			// itemsGroupBox
			// 
			this.itemsGroupBox.Controls.AddRange(new System.Windows.Forms.Control[] {
																						this.itemsListBox,
																						this.addButton,
																						this.delButton});
			this.itemsGroupBox.Location = new System.Drawing.Point(8, 8);
			this.itemsGroupBox.Name = "itemsGroupBox";
			this.itemsGroupBox.Size = new System.Drawing.Size(192, 376);
			this.itemsGroupBox.TabIndex = 4;
			this.itemsGroupBox.TabStop = false;
			this.itemsGroupBox.Text = "Dependencies";
			// 
			// attributesGroupBox
			// 
			this.attributesGroupBox.Controls.AddRange(new System.Windows.Forms.Control[] {
																							 this.label1,
																							 this.textBox1,
																							 this.propertyGrid,
																							 this.groupBox1});
			this.attributesGroupBox.Location = new System.Drawing.Point(216, 8);
			this.attributesGroupBox.Name = "attributesGroupBox";
			this.attributesGroupBox.Size = new System.Drawing.Size(280, 408);
			this.attributesGroupBox.TabIndex = 5;
			this.attributesGroupBox.TabStop = false;
			this.attributesGroupBox.Text = "Attributes";
			// 
			// groupBox1
			// 
			this.groupBox1.Controls.AddRange(new System.Windows.Forms.Control[] {
																					this.radioButton2,
																					this.radioButton1});
			this.groupBox1.Location = new System.Drawing.Point(16, 312);
			this.groupBox1.Name = "groupBox1";
			this.groupBox1.Size = new System.Drawing.Size(248, 56);
			this.groupBox1.TabIndex = 11;
			this.groupBox1.TabStop = false;
			this.groupBox1.Text = "Dependency Type";
			// 
			// radioButton2
			// 
			this.radioButton2.Location = new System.Drawing.Point(144, 24);
			this.radioButton2.Name = "radioButton2";
			this.radioButton2.Size = new System.Drawing.Size(80, 24);
			this.radioButton2.TabIndex = 10;
			this.radioButton2.Text = "Property";
			this.radioButton2.CheckedChanged += new System.EventHandler(this.radioButton2_CheckedChanged);
			// 
			// radioButton1
			// 
			this.radioButton1.Location = new System.Drawing.Point(8, 24);
			this.radioButton1.Name = "radioButton1";
			this.radioButton1.Size = new System.Drawing.Size(120, 24);
			this.radioButton1.TabIndex = 9;
			this.radioButton1.Text = "Software Package";
			this.radioButton1.CheckedChanged += new System.EventHandler(this.radioButton1_CheckedChanged);
			// 
			// separator
			// 
			this.separator.Location = new System.Drawing.Point(0, 424);
			this.separator.Name = "separator";
			this.separator.Size = new System.Drawing.Size(648, 8);
			this.separator.TabIndex = 6;
			this.separator.TabStop = false;
			// 
			// cancelButton
			// 
			this.cancelButton.Location = new System.Drawing.Point(424, 448);
			this.cancelButton.Name = "cancelButton";
			this.cancelButton.Size = new System.Drawing.Size(75, 24);
			this.cancelButton.TabIndex = 7;
			this.cancelButton.Text = "Cancel";
			// 
			// okButton
			// 
			this.okButton.Location = new System.Drawing.Point(328, 448);
			this.okButton.Name = "okButton";
			this.okButton.Size = new System.Drawing.Size(75, 24);
			this.okButton.TabIndex = 8;
			this.okButton.Text = "OK";
			// 
			// textBox1
			// 
			this.textBox1.Location = new System.Drawing.Point(56, 376);
			this.textBox1.Name = "textBox1";
			this.textBox1.Size = new System.Drawing.Size(200, 20);
			this.textBox1.TabIndex = 12;
			this.textBox1.Text = "";
			this.textBox1.TextChanged += new System.EventHandler(this.textBox1_TextChanged);
			// 
			// label1
			// 
			this.label1.Location = new System.Drawing.Point(16, 376);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(32, 23);
			this.label1.TabIndex = 13;
			this.label1.Text = "Type";
			// 
			// DependencyEditorForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(506, 488);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.okButton,
																		  this.cancelButton,
																		  this.separator,
																		  this.itemsGroupBox,
																		  this.attributesGroupBox});
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.Name = "DependencyEditorForm";
			this.ShowInTaskbar = false;
			this.Text = "Dependency Editor Dialog";
			this.itemsGroupBox.ResumeLayout(false);
			this.attributesGroupBox.ResumeLayout(false);
			this.groupBox1.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		private void radioButton1_CheckedChanged(object sender, System.EventArgs e)
		{
			radioButtonChanged();
		}

		private void radioButton2_CheckedChanged(object sender, System.EventArgs e)
		{
			//radioButtonChanged();
		}

		private void radioButtonChanged()
		{
			Dependency dep = (Dependency)currentSelectedObject;
			if ( dep != null )
			{
				
				AttributeSheet sheet;
				if( radioButton1.Checked )
				{
					if( dep.SoftwareDependency == null )
					{
						dep.SoftwareDependency = new SoftwarePackageReference();
					}
					sheet = AttributeSheetFactory.Instance.createAttributeSheet(dep.SoftwareDependency);
					dep.PropertyRef = null;
				}
				else
				{
					if( dep.PropertyRef == null )
					{
						dep.PropertyRef = new SimpleRef();
					}
					sheet = AttributeSheetFactory.Instance.createAttributeSheet(dep.PropertyRef);
					dep.SoftwareDependency = null;
				}
				propertyGrid.SelectedObject = sheet;
			}
		}

		private void textBox1_TextChanged(object sender, System.EventArgs e)
		{
			if( currentSelectedObject != null )
			{
				((Dependency)currentSelectedObject).Type = textBox1.Text;
				itemsListBox.SuspendLayout();
				itemsListBox.Items.Clear();
				itemsListBox.Items.AddRange( list.ToArray() );
				itemsListBox.ResumeLayout();
				ProjectManager.Instance.ChangedFlag = true;
			}
		}

	}
}
