using System;
using System.ComponentModel;
using System.Drawing.Design;
using System.IO;
using System.Windows.Forms;
using System.Windows.Forms.Design;

using jvms.project;
using jvms.system;


namespace jvms.presentation
{
	/// <summary>
	/// Represents a value type editor for properties of type DirectoryInfo.
	/// </summary>
	public class DirectoryInfoValueEditor : UITypeEditor
	{
		private IWindowsFormsEditorService editorService;

		/// <summary>
		/// Creates a new ComplexXMLValueEditor.
		/// </summary>
		public DirectoryInfoValueEditor()
		{
		}

		/// <summary>
		/// Edit's the specified object's value.
		/// </summary>
		/// <param name="context">Provides contextual information about the object.</param>
		/// <param name="provider">Can be used by the editor to obtain services.</param>
		/// <param name="value">The object to be edited.</param>
		/// <returns>The new value of the object.</returns>
		public override object EditValue( ITypeDescriptorContext context, IServiceProvider provider, Object value )
		{
			editorService = (IWindowsFormsEditorService)provider.GetService( typeof( IWindowsFormsEditorService ) );
			FolderBrowser browser = new FolderBrowser();	
			String projectDir = ProjectManager.Instance.Project.ProjectDirectory;
			browser.RootFolder = projectDir;
			DirectoryInfo directory = (DirectoryInfo)value;
			if ( browser.ShowDialog() == DialogResult.OK )
			{
				String path = browser.DirectoryPath;
				path = path.Substring( projectDir.Length );
				if( !path.Equals( "" ) )
				{
					directory = new DirectoryInfo( path );
				}
				else
				{
					directory = new DirectoryInfo( "\\" );
				}
			}
			
			return directory;
		}

		/// <summary>
		/// Gets the editor style that will be used to edit the type.
		/// </summary>
		/// <param name="context">Provides contextual information.</param>
		/// <returns>The UITypeEditorEditStyle that will be used.</returns>
		public override UITypeEditorEditStyle GetEditStyle( ITypeDescriptorContext context )
		{
			return UITypeEditorEditStyle.Modal;
		}
	}
}
