using System;
using System.Collections;
using System.ComponentModel;
using System.Reflection;
using System.Drawing;
using System.Drawing.Design;
using System.Windows.Forms;
using System.Windows.Forms.Design;

using jvms.datamodel.scd;
using jvms.validation;

namespace jvms.presentation
{
	/// <summary>
	/// Creates an editor for port types.
	/// </summary>
	public class DynamicListValueEditor : UITypeEditor
	{
		private IWindowsFormsEditorService editorService;

		/// <summary>
		/// Creates a new PortTypeValueEditor.
		/// </summary>
		public DynamicListValueEditor()
		{
		}

		/// <summary>
		/// Edit's the specified object's value.
		/// </summary>
		/// <param name="context">Provides contextual information about the object.</param>
		/// <param name="provider">Can be used by the editor to obtain services.</param>
		/// <param name="value">The object to be edited.</param>
		/// <returns>The new value of the object.</returns>
		public override object EditValue( ITypeDescriptorContext context, IServiceProvider provider, Object value )
		{
			editorService = (IWindowsFormsEditorService)provider.GetService( typeof( IWindowsFormsEditorService ) );
			//IList selectedPortTypes = (IList)value;
			ListBox listBox = new ListBox();
			listBox.BorderStyle = BorderStyle.None;
			listBox.SelectedIndexChanged += new EventHandler( itemChanged );

			Flobbster.Windows.Forms.PropertyBag.PropertySpecDescriptor desc = (Flobbster.Windows.Forms.PropertyBag.PropertySpecDescriptor)context.PropertyDescriptor;
			BoundPropertySpec spec = (BoundPropertySpec)desc.Item;
			String propertyName = spec.PropertyName;
			
			AttributeSheet attSheet = (AttributeSheet)context.Instance;
			AttributeType currentType = null;
			
			String[] propertyPath = propertyName.Split( new char[] {'.'} );
			propertyName = propertyPath[ propertyPath.Length-1];
			foreach( AttributeType attributeType in attSheet.AttributeTypes )
			{
				if( propertyName.Equals( attributeType.BoundSpec.BoundProperty.Name ) )
				{					
					currentType = attributeType;
				}
			}		

			IList itemsForList = null;
			if( currentType.DataProviderMethod != null )
			{
				DataProvider dataProvider = DataProvider.Instance;
				MethodInfo method = dataProvider.GetType().GetMethod( currentType.DataProviderMethod );
				itemsForList = (IList)method.Invoke( dataProvider, new Object[] {} );
			}
			else if( currentType.ValidValues != null )
			{
				itemsForList = currentType.ValidValues;
			}

			foreach( Object obj in itemsForList )
			{
				listBox.Items.Add( obj );
			}

			editorService.DropDownControl( listBox );
			if( listBox.SelectedItem == null )
			{
				return value;
			}
			return listBox.SelectedItem;
		}

		private void itemChanged( Object sender, EventArgs e )
		{
			editorService.CloseDropDown();
		}

		/// <summary>
		/// Gets the editor style that will be used to edit the type.
		/// </summary>
		/// <param name="context">Provides contextual information.</param>
		/// <returns>The UITypeEditorEditStyle that will be used.</returns>
		public override UITypeEditorEditStyle GetEditStyle( ITypeDescriptorContext context )
		{
			return UITypeEditorEditStyle.DropDown;
		}
	}
}
