using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

using jvms.datamodel.common;
using jvms.system;

namespace jvms.presentation
{
	/// <summary>
	/// Summary description for FindByXMLValueEditorForm.
	/// </summary>
	public class FindByXMLValueEditorForm : System.Windows.Forms.Form
	{
		#region Windows Form Designer generated state
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		private System.Windows.Forms.GroupBox groupBox1;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.GroupBox groupBox2;
		private System.Windows.Forms.Label label2;
		private System.Windows.Forms.Label label3;
		private System.Windows.Forms.Button okButton;
		private System.Windows.Forms.Button cancelButton;
		private System.Windows.Forms.RadioButton namingRadioButton;
		private System.Windows.Forms.RadioButton domainRadioButton;
		private System.Windows.Forms.TextBox namingTextfield;
		private System.Windows.Forms.ComboBox domainType;
		private System.Windows.Forms.TextBox domainTextfield;
		private System.Windows.Forms.GroupBox groupBox3;
		#endregion

		private FindBy findBy;

		/// <summary>
		/// This will get the findby for the editor.
		/// </summary>
		public FindBy FindBy
		{
			get
			{
				return findBy;
			}

			set
			{
				if ( value is NamingService )
				{
					namingTextfield.Text = ( (NamingService)value ).Name;
					namingRadioButton.Checked = true;
				}
				else if ( value is DomainFinder )
				{
					domainTextfield.Text = ((DomainFinder)value).Name;
					for( int i = 0; i < domainType.Items.Count; i++ )
					{
						if( domainType.Items[i].Equals( ( (DomainFinder)value ).Type ) )
						{
							domainType.SelectedIndex = i;
						}
					}
					domainRadioButton.Checked = true;
				}
				findBy = value;
			}
		}

		public FindByXMLValueEditorForm()
		{
			InitializeComponent();

			domainType.Items.Add( "filemanger" );
			domainType.Items.Add( "log" );
			domainType.Items.Add( "eventchannel" );
			domainType.Items.Add( "namingservice" );

			okButton.Click += new EventHandler( okButtonClicked );
			cancelButton.Click += new EventHandler( cancelButtonClicked );
			domainRadioButton.CheckedChanged += new EventHandler( typeCheckedChanged );
			namingRadioButton.CheckedChanged += new EventHandler( typeCheckedChanged );
		}

		/// <summary>
		/// Called when the OK button has been clicked.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void okButtonClicked( Object source, EventArgs e )
		{
			this.Close();

			if ( namingRadioButton.Checked )
			{
				NamingService naming = new NamingService();
				naming.Name = namingTextfield.Text;
				findBy = naming;
			}
			else if ( domainRadioButton.Checked )
			{
				DomainFinder finder = new DomainFinder();
				finder.Name = domainTextfield.Text;
				if ( domainType.SelectedItem != null )
				{
					finder.Type = domainType.SelectedItem.ToString();
				}
				findBy = finder;
			}

			View view = JVMSMain.JvmsForm.CurrentView;
			view.Refresh();
		}

		/// <summary>
		/// Called when the cancel button is clicked.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void cancelButtonClicked( Object source, EventArgs e )
		{
			this.Close();
			View view = JVMSMain.JvmsForm.CurrentView;
			view.Refresh();
		}

		/// <summary>
		/// Called when one of the type radio button's checked state has changed.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void typeCheckedChanged( Object source, EventArgs e )
		{
			if ( domainRadioButton.Checked )
			{
				domainTextfield.Enabled = true;
				domainType.Enabled = true;
				namingTextfield.Enabled = false;
			}
			else if ( namingRadioButton.Checked )
			{
				domainTextfield.Enabled = false;
				domainType.Enabled = false;
				namingTextfield.Enabled = true;
			}
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.namingRadioButton = new System.Windows.Forms.RadioButton();
			this.domainRadioButton = new System.Windows.Forms.RadioButton();
			this.namingTextfield = new System.Windows.Forms.TextBox();
			this.groupBox1 = new System.Windows.Forms.GroupBox();
			this.label1 = new System.Windows.Forms.Label();
			this.groupBox2 = new System.Windows.Forms.GroupBox();
			this.domainType = new System.Windows.Forms.ComboBox();
			this.label3 = new System.Windows.Forms.Label();
			this.label2 = new System.Windows.Forms.Label();
			this.domainTextfield = new System.Windows.Forms.TextBox();
			this.okButton = new System.Windows.Forms.Button();
			this.cancelButton = new System.Windows.Forms.Button();
			this.groupBox3 = new System.Windows.Forms.GroupBox();
			this.groupBox1.SuspendLayout();
			this.groupBox2.SuspendLayout();
			this.groupBox3.SuspendLayout();
			this.SuspendLayout();
			// 
			// namingRadioButton
			// 
			this.namingRadioButton.Location = new System.Drawing.Point(112, 24);
			this.namingRadioButton.Name = "namingRadioButton";
			this.namingRadioButton.TabIndex = 0;
			this.namingRadioButton.Text = "Naming Service";
			// 
			// domainRadioButton
			// 
			this.domainRadioButton.Location = new System.Drawing.Point(8, 24);
			this.domainRadioButton.Name = "domainRadioButton";
			this.domainRadioButton.Size = new System.Drawing.Size(96, 24);
			this.domainRadioButton.TabIndex = 1;
			this.domainRadioButton.Text = "Domain Finder";
			// 
			// namingTextfield
			// 
			this.namingTextfield.Enabled = false;
			this.namingTextfield.Location = new System.Drawing.Point(48, 20);
			this.namingTextfield.Name = "namingTextfield";
			this.namingTextfield.Size = new System.Drawing.Size(120, 20);
			this.namingTextfield.TabIndex = 2;
			this.namingTextfield.Text = "";
			// 
			// groupBox1
			// 
			this.groupBox1.Controls.AddRange(new System.Windows.Forms.Control[] {
																					this.label1,
																					this.namingTextfield});
			this.groupBox1.Location = new System.Drawing.Point(208, 72);
			this.groupBox1.Name = "groupBox1";
			this.groupBox1.Size = new System.Drawing.Size(184, 96);
			this.groupBox1.TabIndex = 3;
			this.groupBox1.TabStop = false;
			this.groupBox1.Text = "Naming Service";
			// 
			// label1
			// 
			this.label1.Location = new System.Drawing.Point(8, 20);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(36, 20);
			this.label1.TabIndex = 3;
			this.label1.Text = "Name";
			this.label1.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
			// 
			// groupBox2
			// 
			this.groupBox2.Controls.AddRange(new System.Windows.Forms.Control[] {
																					this.domainType,
																					this.label3,
																					this.label2,
																					this.domainTextfield});
			this.groupBox2.Location = new System.Drawing.Point(8, 72);
			this.groupBox2.Name = "groupBox2";
			this.groupBox2.Size = new System.Drawing.Size(192, 96);
			this.groupBox2.TabIndex = 4;
			this.groupBox2.TabStop = false;
			this.groupBox2.Text = "Domain Finder";
			// 
			// domainType
			// 
			this.domainType.Enabled = false;
			this.domainType.Location = new System.Drawing.Point(56, 57);
			this.domainType.Name = "domainType";
			this.domainType.Size = new System.Drawing.Size(120, 21);
			this.domainType.TabIndex = 7;
			// 
			// label3
			// 
			this.label3.Location = new System.Drawing.Point(16, 58);
			this.label3.Name = "label3";
			this.label3.Size = new System.Drawing.Size(32, 20);
			this.label3.TabIndex = 6;
			this.label3.Text = "Type";
			this.label3.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
			// 
			// label2
			// 
			this.label2.Location = new System.Drawing.Point(8, 24);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(40, 20);
			this.label2.TabIndex = 5;
			this.label2.Text = "Name";
			this.label2.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
			// 
			// domainTextfield
			// 
			this.domainTextfield.Enabled = false;
			this.domainTextfield.Location = new System.Drawing.Point(56, 24);
			this.domainTextfield.Name = "domainTextfield";
			this.domainTextfield.Size = new System.Drawing.Size(120, 20);
			this.domainTextfield.TabIndex = 4;
			this.domainTextfield.Text = "";
			// 
			// okButton
			// 
			this.okButton.Location = new System.Drawing.Point(240, 176);
			this.okButton.Name = "okButton";
			this.okButton.Size = new System.Drawing.Size(72, 24);
			this.okButton.TabIndex = 5;
			this.okButton.Text = "Ok";
			// 
			// cancelButton
			// 
			this.cancelButton.Location = new System.Drawing.Point(320, 176);
			this.cancelButton.Name = "cancelButton";
			this.cancelButton.Size = new System.Drawing.Size(76, 24);
			this.cancelButton.TabIndex = 6;
			this.cancelButton.Text = "Cancel";
			// 
			// groupBox3
			// 
			this.groupBox3.Controls.AddRange(new System.Windows.Forms.Control[] {
																					this.domainRadioButton,
																					this.namingRadioButton});
			this.groupBox3.Location = new System.Drawing.Point(8, 8);
			this.groupBox3.Name = "groupBox3";
			this.groupBox3.Size = new System.Drawing.Size(384, 56);
			this.groupBox3.TabIndex = 7;
			this.groupBox3.TabStop = false;
			this.groupBox3.Text = "Find By Type";
			// 
			// FindByXMLValueEditorForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(402, 207);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.groupBox3,
																		  this.cancelButton,
																		  this.okButton,
																		  this.groupBox2,
																		  this.groupBox1});
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.Name = "FindByXMLValueEditorForm";
			this.ShowInTaskbar = false;
			this.Text = "FindBy Editor";
			this.groupBox1.ResumeLayout(false);
			this.groupBox2.ResumeLayout(false);
			this.groupBox3.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

	}
}
