using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

using jvms.datamodel;
using jvms.datamodel.common;
using jvms.datamodel.sad;
using jvms.system;
using jvms.validation;
using jvms.presentation.drawablecomponent;

namespace jvms.presentation
{
	/// <summary>
	/// Summary description for FindByXMLValueEditorForm.
	/// </summary>
	public class FindComponentXMLValueEditorForm : System.Windows.Forms.Form
	{
		#region Windows Form Designer generated state
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		private System.Windows.Forms.GroupBox groupBox1;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.GroupBox groupBox2;
		private System.Windows.Forms.Label label3;
		private System.Windows.Forms.Button okButton;
		private System.Windows.Forms.Button cancelButton;
		private System.Windows.Forms.RadioButton namingRadioButton;
		private System.Windows.Forms.TextBox namingTextfield;
		private System.Windows.Forms.GroupBox groupBox3;
		#endregion
		private System.Windows.Forms.ComboBox factory;
		private System.Windows.Forms.Button editPropertiesButton;
		private System.Windows.Forms.RadioButton factoryRadioButton;

		private FindComponent findComponent;

		/// <summary>
		/// This will get the findby for the editor.
		/// </summary>
		public FindComponent FindComponent
		{
			get
			{
				return findComponent;
			}

			set
			{
				findComponent = value;
				factoryRadioButton.Checked = true;
				if( value.NamingService != null )
				{
					namingTextfield.Text = value.NamingService.Name;
					namingRadioButton.Checked = true;
				}
				else if( value.ResourceFactory != null )
				{		
					try
					{
						if( value.ResourceFactory.Instance != null )
						{
							factory.SelectedItem = value.ResourceFactory.Instance;
						}
						factoryRadioButton.Checked = true;
					}
					catch( Exception e )
					{
						Console.WriteLine( e );
					}
				}				
				if( findComponent.ResourceFactory == null )
				{
					findComponent.ResourceFactory = new ComponentResourceFactoryRef();
				}
			}
		}

		public FindComponentXMLValueEditorForm()
		{
			InitializeComponent();

			IList allFactories = DataProvider.Instance.GetResourceFactoryInstances();

			foreach( Object obj in allFactories )
			{
				factory.Items.Add( obj );
			}


			okButton.Click += new EventHandler( okButtonClicked );
			cancelButton.Click += new EventHandler( cancelButtonClicked );
			factoryRadioButton.CheckedChanged += new EventHandler( typeCheckedChanged );
			namingRadioButton.CheckedChanged += new EventHandler( typeCheckedChanged );
		}

		/// <summary>
		/// Called when the OK button has been clicked.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void okButtonClicked( Object source, EventArgs e )
		{
			this.Close();

			if ( namingRadioButton.Checked )
			{
				NamingService naming = new NamingService();
				naming.Name = namingTextfield.Text;
				findComponent.NamingService = naming;
				findComponent.ResourceFactory = null;
			}
			else if ( factoryRadioButton.Checked )
			{				
				findComponent.NamingService = null;
			}

			View view = JVMSMain.JvmsForm.CurrentView;
			view.Refresh();
		}

		/// <summary>
		/// Called when the cancel button is clicked.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void cancelButtonClicked( Object source, EventArgs e )
		{
			this.Close();
			View view = JVMSMain.JvmsForm.CurrentView;
			view.Refresh();
		}

		/// <summary>
		/// Called when one of the type radio button's checked state has changed.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void typeCheckedChanged( Object source, EventArgs e )
		{
			factory.Enabled = false;
			editPropertiesButton.Enabled = false;
			namingTextfield.Enabled = false;
			if ( factoryRadioButton.Checked )
			{
				factory.Enabled = true;
				editPropertiesButton.Enabled = true;
			}
			else if ( namingRadioButton.Checked )
			{
				namingTextfield.Enabled = true;
			}
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.namingRadioButton = new System.Windows.Forms.RadioButton();
			this.factoryRadioButton = new System.Windows.Forms.RadioButton();
			this.namingTextfield = new System.Windows.Forms.TextBox();
			this.groupBox1 = new System.Windows.Forms.GroupBox();
			this.label1 = new System.Windows.Forms.Label();
			this.groupBox2 = new System.Windows.Forms.GroupBox();
			this.editPropertiesButton = new System.Windows.Forms.Button();
			this.factory = new System.Windows.Forms.ComboBox();
			this.label3 = new System.Windows.Forms.Label();
			this.okButton = new System.Windows.Forms.Button();
			this.cancelButton = new System.Windows.Forms.Button();
			this.groupBox3 = new System.Windows.Forms.GroupBox();
			this.groupBox1.SuspendLayout();
			this.groupBox2.SuspendLayout();
			this.groupBox3.SuspendLayout();
			this.SuspendLayout();
			// 
			// namingRadioButton
			// 
			this.namingRadioButton.Location = new System.Drawing.Point(128, 24);
			this.namingRadioButton.Name = "namingRadioButton";
			this.namingRadioButton.TabIndex = 0;
			this.namingRadioButton.Text = "Naming Service";
			// 
			// factoryRadioButton
			// 
			this.factoryRadioButton.Location = new System.Drawing.Point(8, 24);
			this.factoryRadioButton.Name = "factoryRadioButton";
			this.factoryRadioButton.Size = new System.Drawing.Size(112, 24);
			this.factoryRadioButton.TabIndex = 1;
			this.factoryRadioButton.Text = "Resource Factory";
			// 
			// namingTextfield
			// 
			this.namingTextfield.Enabled = false;
			this.namingTextfield.Location = new System.Drawing.Point(48, 20);
			this.namingTextfield.Name = "namingTextfield";
			this.namingTextfield.Size = new System.Drawing.Size(120, 20);
			this.namingTextfield.TabIndex = 2;
			this.namingTextfield.Text = "";
			// 
			// groupBox1
			// 
			this.groupBox1.Controls.AddRange(new System.Windows.Forms.Control[] {
																					this.label1,
																					this.namingTextfield});
			this.groupBox1.Location = new System.Drawing.Point(208, 72);
			this.groupBox1.Name = "groupBox1";
			this.groupBox1.Size = new System.Drawing.Size(184, 96);
			this.groupBox1.TabIndex = 3;
			this.groupBox1.TabStop = false;
			this.groupBox1.Text = "Naming Service";
			// 
			// label1
			// 
			this.label1.Location = new System.Drawing.Point(8, 20);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(36, 20);
			this.label1.TabIndex = 3;
			this.label1.Text = "Name";
			this.label1.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
			// 
			// groupBox2
			// 
			this.groupBox2.Controls.AddRange(new System.Windows.Forms.Control[] {
																					this.editPropertiesButton,
																					this.factory,
																					this.label3});
			this.groupBox2.Location = new System.Drawing.Point(8, 72);
			this.groupBox2.Name = "groupBox2";
			this.groupBox2.Size = new System.Drawing.Size(192, 96);
			this.groupBox2.TabIndex = 4;
			this.groupBox2.TabStop = false;
			this.groupBox2.Text = "Resource Factory";
			// 
			// editPropertiesButton
			// 
			this.editPropertiesButton.Location = new System.Drawing.Point(56, 64);
			this.editPropertiesButton.Name = "editPropertiesButton";
			this.editPropertiesButton.Size = new System.Drawing.Size(88, 23);
			this.editPropertiesButton.TabIndex = 8;
			this.editPropertiesButton.Text = "Edit Properties";
			this.editPropertiesButton.Click += new System.EventHandler(this.editProperties_Click);
			// 
			// factory
			// 
			this.factory.Enabled = false;
			this.factory.Location = new System.Drawing.Point(64, 24);
			this.factory.Name = "factory";
			this.factory.Size = new System.Drawing.Size(120, 21);
			this.factory.TabIndex = 7;
			this.factory.SelectedIndexChanged += new System.EventHandler(this.factory_SelectedIndexChanged);
			// 
			// label3
			// 
			this.label3.Location = new System.Drawing.Point(8, 24);
			this.label3.Name = "label3";
			this.label3.Size = new System.Drawing.Size(48, 20);
			this.label3.TabIndex = 6;
			this.label3.Text = "Factory";
			this.label3.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
			// 
			// okButton
			// 
			this.okButton.Location = new System.Drawing.Point(240, 176);
			this.okButton.Name = "okButton";
			this.okButton.Size = new System.Drawing.Size(72, 24);
			this.okButton.TabIndex = 5;
			this.okButton.Text = "Ok";
			// 
			// cancelButton
			// 
			this.cancelButton.Location = new System.Drawing.Point(320, 176);
			this.cancelButton.Name = "cancelButton";
			this.cancelButton.Size = new System.Drawing.Size(76, 24);
			this.cancelButton.TabIndex = 6;
			this.cancelButton.Text = "Cancel";
			// 
			// groupBox3
			// 
			this.groupBox3.Controls.AddRange(new System.Windows.Forms.Control[] {
																					this.factoryRadioButton,
																					this.namingRadioButton});
			this.groupBox3.Location = new System.Drawing.Point(8, 8);
			this.groupBox3.Name = "groupBox3";
			this.groupBox3.Size = new System.Drawing.Size(384, 56);
			this.groupBox3.TabIndex = 7;
			this.groupBox3.TabStop = false;
			this.groupBox3.Text = "Find By Type";
			// 
			// FindComponentXMLValueEditorForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(402, 207);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.groupBox3,
																		  this.cancelButton,
																		  this.okButton,
																		  this.groupBox2,
																		  this.groupBox1});
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.Name = "FindComponentXMLValueEditorForm";
			this.ShowInTaskbar = false;
			this.Text = "FindComponent Editor";
			this.groupBox1.ResumeLayout(false);
			this.groupBox2.ResumeLayout(false);
			this.groupBox3.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		private void editProperties_Click(object sender, System.EventArgs e)
		{
			AllocPropsValueEditorForm form = new AllocPropsValueEditorForm();						
			ComponentProperties props = new ComponentProperties();
			DrawableComponent thisComponent = JVMSMain.JvmsForm.SelectedComponent;
			ApplicationComponentInstantiation appInstance = 
				(ApplicationComponentInstantiation)thisComponent.getWrappedObject();

			props.BaseComponent = appInstance.Component;
			if( findComponent.ResourceFactory.ResourceFactoryProperties != null )
			{
				ComponentProperties properties = findComponent.ResourceFactory.ResourceFactoryProperties;				
				props.Simples = new TypedArrayList( properties.Simples );
				props.SimpleSequences = new TypedArrayList( properties.SimpleSequences );
				props.Structs = new TypedArrayList( properties.Structs );
				props.StructSequences = new TypedArrayList( properties.StructSequences );
			}

			form.SetAllocationProperties( props.BaseProperties );
			form.SetInstanceProperties( props );
			DialogResult result = form.ShowDialog( this );
			if( result == DialogResult.OK )
			{
				findComponent.ResourceFactory.ResourceFactoryProperties = 
					form.GetProperties();
			}
		}

		private void factory_SelectedIndexChanged(object sender, System.EventArgs e)
		{
			findComponent.ResourceFactory.Instance = (ComponentInstantiation)factory.SelectedItem;
		}
	}
}
