using System;
using System.Collections;
using System.ComponentModel;
using System.Reflection;
using System.Drawing;
using System.Drawing.Design;
using System.Windows.Forms;
using System.Windows.Forms.Design;

using jvms.datamodel.scd;
using jvms.datamodel.spd;
using jvms.datamodel;
using jvms.datamodel.common;
using jvms.system;
using jvms.validation;
using jvms.presentation.drawablecomponent;

namespace jvms.presentation
{
	/// <summary>
	/// Creates an editor for port types.
	/// </summary>
	public class ImplementationListValueEditor : UITypeEditor
	{
		private IWindowsFormsEditorService editorService;

		/// <summary>
		/// Creates a new PortTypeValueEditor.
		/// </summary>
		public ImplementationListValueEditor()
		{
		}

		/// <summary>
		/// Edit's the specified object's value.
		/// </summary>
		/// <param name="context">Provides contextual information about the object.</param>
		/// <param name="provider">Can be used by the editor to obtain services.</param>
		/// <param name="value">The object to be edited.</param>
		/// <returns>The new value of the object.</returns>
		public override object EditValue( ITypeDescriptorContext context, IServiceProvider provider, Object value )
		{
			editorService = (IWindowsFormsEditorService)provider.GetService( typeof( IWindowsFormsEditorService ) );
			SoftwarePackageReference reference = (SoftwarePackageReference)((AttributeSheet)context.Instance).WorkingObject;
			
			ListBox listBox = new ListBox();
			listBox.BorderStyle = BorderStyle.None;
			listBox.SelectedIndexChanged += new EventHandler( itemChanged );

			NonSCAComponent component = reference.Component;
			ICollection itemsForList;
			
			if( component == null )
			{
				itemsForList = new ArrayList();
			}
			else
			{
				itemsForList = component.Implementations;
			}
			
			Implementation currentValue = (Implementation)value;
			foreach( Implementation impl in itemsForList )
			{
				listBox.Items.Add( impl );
				if( currentValue == impl )
				{
					listBox.SelectedItem = impl;
				}
			}
			Implementation dummy = new Implementation();
			dummy.Description = "<none>";
			listBox.Items.Add( dummy );

			if( listBox.SelectedItem == null )
			{
				listBox.SelectedItem = dummy;
			}


			editorService.DropDownControl( listBox );
			if( listBox.SelectedItem != null )
			{
				currentValue = ((Implementation)listBox.SelectedItem);
				if( listBox.SelectedItem == dummy )
				{
					currentValue = null;
				}
			}
			return currentValue;
		}

		private void itemChanged( Object sender, EventArgs e )
		{
			editorService.CloseDropDown();
		}

		/// <summary>
		/// Gets the editor style that will be used to edit the type.
		/// </summary>
		/// <param name="context">Provides contextual information.</param>
		/// <returns>The UITypeEditorEditStyle that will be used.</returns>
		public override UITypeEditorEditStyle GetEditStyle( ITypeDescriptorContext context )
		{
			return UITypeEditorEditStyle.DropDown;
		}
	}
}
