using System;
using System.ComponentModel;
using System.Drawing.Design;
using System.Windows.Forms.Design;

using jvms.datamodel;
using jvms.validation;


namespace jvms.presentation
{
	/// <summary>
	/// Represents a value type editor for complex XML types.
	/// </summary>
	public class InterfacesValueEditor : UITypeEditor
	{
		private IWindowsFormsEditorService editorService;

		/// <summary>
		/// Creates a new ComplexXMLValueEditor.
		/// </summary>
		public InterfacesValueEditor()
		{
		}

		/// <summary>
		/// Edit's the specified object's value.
		/// </summary>
		/// <param name="context">Provides contextual information about the object.</param>
		/// <param name="provider">Can be used by the editor to obtain services.</param>
		/// <param name="value">The object to be edited.</param>
		/// <returns>The new value of the object.</returns>
		public override object EditValue( ITypeDescriptorContext context, IServiceProvider provider, Object value )
		{
			InterfacesValueEditorForm form = new InterfacesValueEditorForm();
			editorService = (IWindowsFormsEditorService)provider.GetService( typeof( IWindowsFormsEditorService ) );
			TypedArrayList list = DataProvider.Instance.GetInterfaces();
			if( value != null )
			{
				list.Remove( value );
				list.Insert( 0, value );
			}
			form.SetList( list );
			editorService.ShowDialog( form );
			return form.GetSelected();
		}

		/// <summary>
		/// Gets the editor style that will be used to edit the type.
		/// </summary>
		/// <param name="context">Provides contextual information.</param>
		/// <returns>The UITypeEditorEditStyle that will be used.</returns>
		public override UITypeEditorEditStyle GetEditStyle( ITypeDescriptorContext context )
		{
			return UITypeEditorEditStyle.Modal;
		}
	}
}
