using System;
using System.Windows.Forms;

using jvms.project;
using jvms.system;

namespace jvms.presentation
{
	/// <summary>
	/// Listens for events from the tool bar and takes appropriate action.
	/// </summary>
	public class JVMSToolBarListener
	{
		/// <summary>
		/// Creates a new JVMSToolBarListener.
		/// </summary>
		public JVMSToolBarListener()
		{
		}

		/// <summary>
		/// Called when a button on the toolbar is clicked.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		public void buttonClicked( Object source, ToolBarButtonClickEventArgs e )
		{
			String action = (String)e.Button.Tag;
			ProjectManager pm = ProjectManager.Instance;
			
			if ( action.Equals( "view" ) )
			{
				JVMSMain.JvmsForm.CurrentView.Refresh();
			}
			else if ( action.Equals( "save" ) )
			{
				pm.saveProject();
			}
			else if ( action.Equals( "saveas" ) )
			{
				pm.saveProjectAs();
			}
			else if ( action.Equals( "open" ) )
			{
				pm.openProject();
			}
			else if ( action.Equals( "tasklist" ) )
			{
				JVMSMain.JvmsForm.setTaskListVisible( e.Button.Pushed );
			}
		}

		/// <summary>
		/// Called when the designer selects the platform component view.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		public void platformComponentViewSelected( Object source, EventArgs e )
		{
			JVMSForm jvmsForm = JVMSMain.JvmsForm;
			jvmsForm.CurrentView = ViewManager.Instance.PlatCompView;
			jvmsForm.CurrentAttributeSheetControl.clearAttributeSheet();
		}

		/// <summary>
		/// Called when the designer selects the platform assembly view.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		public void platformAssemblyViewSelected( Object source, EventArgs e )
		{
			JVMSForm jvmsForm = JVMSMain.JvmsForm;
			jvmsForm.CurrentView = ViewManager.Instance.PlatAssmView;
			jvmsForm.CurrentAttributeSheetControl.clearAttributeSheet();
		}

		/// <summary>
		/// Called when the designer selects the application component view.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		public void applicationComponentViewSelected( Object source, EventArgs e )
		{
			JVMSForm jvmsForm = JVMSMain.JvmsForm;
			jvmsForm.CurrentView = ViewManager.Instance.AppCompView;
			jvmsForm.CurrentAttributeSheetControl.clearAttributeSheet();
		}

		/// <summary>
		/// Called when the designer selects the application assembly view.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		public void applicationAssemblyViewSelected( Object source, EventArgs e )
		{
			JVMSForm jvmsForm = JVMSMain.JvmsForm;
			jvmsForm.CurrentView = ViewManager.Instance.AppAssmView;
			jvmsForm.CurrentAttributeSheetControl.clearAttributeSheet();
		}
	}
}
