using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;

using jvms.presentation.relationships;
using jvms.system;

namespace jvms.presentation
{
	/// <summary>
	/// Provides controls that allow a designer to add components to a project.
	/// </summary>
	public class JVMSToolBox : System.Windows.Forms.UserControl
	{
		#region Component Designer generated state

		private System.Windows.Forms.ImageList appCompImageList;
		private System.Windows.Forms.ImageList platCompImageList;
		private System.Windows.Forms.ImageList appAssmImageList;
		private System.Windows.Forms.ImageList platAssmImageList;
		private System.Windows.Forms.TabControl tabbedPane;
		private System.Windows.Forms.TabPage compPage;
		private System.Windows.Forms.TabPage relatPage;
		private System.Windows.Forms.ListView toolListView;
		private System.Windows.Forms.Label compInfoLabel;
		private System.Windows.Forms.ListView relatListView;
		private System.Windows.Forms.Label relatInfoLabel;
		private System.Windows.Forms.ImageList relationshipsImageList;
		private System.ComponentModel.IContainer components;
		#endregion

		/// <summary>Sets or gets the information text displayed in the toolbox.</summary>
		/// <value>The text displayed in the tool box.</value>
		public String InfoText
		{
			get
			{
				return compInfoLabel.Text;
			}

			set
			{
				compInfoLabel.Text = value;
				relatInfoLabel.Text = value;
			}
		}

		/// <summary>
		/// Creates a new JVMSToolBox.
		/// </summary>
		public JVMSToolBox()
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();

			this.toolListView.ItemActivate += new EventHandler( toolItemActivated );
			this.toolListView.ItemDrag += new ItemDragEventHandler( toolItemDragged );
			this.relatListView.ItemActivate += new EventHandler( relationshipItemActivated );
		}

		/// <summary>
		/// Called when the view has been changed, displays the appropriate toolset.
		/// </summary>
		/// <param name="viewName">The new view.</param>
		public void viewChanged( View view )
		{
			this.toolListView.SuspendLayout();
			this.relatListView.SuspendLayout();
			this.toolListView.Items.Clear();
			this.relatListView.Items.Clear();

			String viewName = view.ToString();

			if ( viewName.Equals( "Platform Component" ) )
			{
				ListViewItem deviceListItem = new ListViewItem( "CF Device", 0 );
				ListViewItem nonScaListItem = new ListViewItem( "Non-SCA Component", 1 );
				ListViewItem filesysListItem = new ListViewItem( "CF Filesystem", 2 );
				ListViewItem loggerListItem = new ListViewItem( "CF Logger", 3 );
				ListViewItem devmgrListItem = new ListViewItem( "Device Manager", 4 );
				ListViewItem propsListItem = new ListViewItem( "Properties", 5 );

				ListViewItem propRelatListItem = new ListViewItem( "Properties", 5 );

				deviceListItem.Tag = "cfdevice";
				nonScaListItem.Tag = "nonscacomponent";
				filesysListItem.Tag = "cffilesystem";
				devmgrListItem.Tag = "devicemanager";
				loggerListItem.Tag = "cflogger";
				propsListItem.Tag = "properties";

				propRelatListItem.Tag = typeof( PropertiesRelationship ).FullName;

				this.toolListView.LargeImageList = this.platCompImageList;
				this.relatListView.LargeImageList = this.relationshipsImageList;
				this.toolListView.Items.AddRange( new ListViewItem[] { deviceListItem, nonScaListItem, filesysListItem, devmgrListItem, loggerListItem, propsListItem } );
				this.relatListView.Items.AddRange( new ListViewItem[] { propRelatListItem } );

				InfoText = "Platform Component View";
			}
			else if ( viewName.Equals( "Platform Assembly" ) )
			{
				ListViewItem devcListItem = new ListViewItem( "Device Configuration", 0 );
				ListViewItem instListItem = new ListViewItem( "Component Instantiation", 1 );

				ListViewItem devConfRelatListItem = new ListViewItem( "Add Instance to Device Configuration", 1 );
				ListViewItem instRefRelatListItem = new ListViewItem( "Standard Component Connection", 4 );
				ListViewItem devUsedRelatListItem = new ListViewItem( "Device Used By Component", 2 );
				ListViewItem findByRelatListItem = new ListViewItem( "Find By", 3 );

				devcListItem.Tag = "deviceconfig";
				instListItem.Tag = "instance";

				devConfRelatListItem.Tag = typeof( DeviceConfigRelationship ).FullName;
				instRefRelatListItem.Tag = typeof( InstanceRefRelationship ).FullName;
				devUsedRelatListItem.Tag = typeof( DeviceUsedByRelationship ).FullName;
				findByRelatListItem.Tag = typeof( FindByRelationship ).FullName;

				this.toolListView.LargeImageList = this.platAssmImageList;
				this.relatListView.LargeImageList = this.relationshipsImageList;
				this.toolListView.Items.AddRange( new ListViewItem[] { devcListItem, instListItem } );
				this.relatListView.Items.AddRange( new ListViewItem[] { devConfRelatListItem, instRefRelatListItem, devUsedRelatListItem, findByRelatListItem } );
				
				InfoText = "Platform Assembly View";
			}
			else if ( viewName.Equals( "Application Component" ) )
			{
				ListViewItem resListItem = new ListViewItem( "CF Resource", 0 );
				ListViewItem nonScaListItem = new ListViewItem( "Non-SCA Component", 1 );
				ListViewItem resourceFactoryListItem = new ListViewItem( "Resource Factory", 3 );
				ListViewItem propsListItem = new ListViewItem( "Properties", 2 );

				ListViewItem propRelatListItem = new ListViewItem( "Properties", 5 );

				resListItem.Tag = "cfresource";
				nonScaListItem.Tag = "nonscacomponent";
				resourceFactoryListItem.Tag = "resourcefactory";
				propsListItem.Tag = "properties";

				propRelatListItem.Tag = typeof( PropertiesRelationship ).FullName;

				this.toolListView.LargeImageList = this.appCompImageList;
				this.relatListView.LargeImageList = this.relationshipsImageList;
				this.toolListView.Items.AddRange( new ListViewItem[] { resListItem, nonScaListItem, resourceFactoryListItem, propsListItem } );
				this.relatListView.Items.AddRange( new ListViewItem[] { propRelatListItem } );

				InfoText = "Application Component View";
			}
			else if ( viewName.Equals( "Application Assembly" ) )
			{
				ListViewItem appListItem = new ListViewItem( "Application", 0 );
				ListViewItem instListItem = new ListViewItem( "Component Instantiation", 1 );

				ListViewItem appRelatListItem = new ListViewItem( "Add Instance to Application", 0 );
				ListViewItem instRefRelatListItem = new ListViewItem( "Standard Component Connection", 4 );
				ListViewItem devUsedRelatListItem = new ListViewItem( "Device Used By Component", 2 );
				ListViewItem findByRelatListItem = new ListViewItem( "Find By", 3 );

				appListItem.Tag = "application";
				instListItem.Tag = "instance";

				appRelatListItem.Tag = typeof( ApplicationRelationship ).FullName;
				instRefRelatListItem.Tag = typeof( InstanceRefRelationship ).FullName;
				devUsedRelatListItem.Tag = typeof( DeviceUsedByRelationship ).FullName;
				findByRelatListItem.Tag = typeof( FindByRelationship ).FullName;

				this.toolListView.LargeImageList = this.appAssmImageList;
				this.relatListView.LargeImageList = this.relationshipsImageList;
				this.toolListView.Items.AddRange( new ListViewItem[] { appListItem, instListItem } );
				this.relatListView.Items.AddRange( new ListViewItem[] { appRelatListItem, instRefRelatListItem, devUsedRelatListItem, findByRelatListItem } );

				InfoText = "Application Assembly View";
			}
            else
			{
				throw new ArgumentException(
					"Value must be one of 'platAssm', 'appAssm', 'platComp', or 'appComp'", "viewName" );
			}
			this.toolListView.ResumeLayout();
			this.relatListView.ResumeLayout();
		}

		/// <summary>
		/// Called when an item in the tool list view is activated.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void toolItemActivated( Object source, EventArgs e )
		{
			ListViewItem selectedItem = this.toolListView.SelectedItems[0];

			JVMSForm jvmsForm = JVMSMain.JvmsForm;
			jvmsForm.ActiveControl = jvmsForm.CurrentView;
			View view = jvmsForm.CurrentView;
			view.beginAddComponent( (String)selectedItem.Tag );
		}

		/// <summary>
		/// Called when an item in the relationship list view is activated.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void relationshipItemActivated( Object source, EventArgs e )
		{
			ListViewItem selectedItem = this.relatListView.SelectedItems[0];

			JVMSMain.JvmsForm.ActiveControl = JVMSMain.JvmsForm.CurrentView;
			RelationshipManager.Instance.beginCreateRelationship( (String)selectedItem.Tag );
		}

		/// <summary>
		/// Called when an item in the tool list view is dragged.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void toolItemDragged( Object source, ItemDragEventArgs e )
		{
			ListViewItem selectedItem = (ListViewItem)e.Item;
			DoDragDrop( selectedItem.Tag, DragDropEffects.Copy );
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if( components != null )
					components.Dispose();
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(JVMSToolBox));
			this.appCompImageList = new System.Windows.Forms.ImageList(this.components);
			this.platCompImageList = new System.Windows.Forms.ImageList(this.components);
			this.appAssmImageList = new System.Windows.Forms.ImageList(this.components);
			this.platAssmImageList = new System.Windows.Forms.ImageList(this.components);
			this.tabbedPane = new System.Windows.Forms.TabControl();
			this.compPage = new System.Windows.Forms.TabPage();
			this.toolListView = new System.Windows.Forms.ListView();
			this.compInfoLabel = new System.Windows.Forms.Label();
			this.relatPage = new System.Windows.Forms.TabPage();
			this.relatListView = new System.Windows.Forms.ListView();
			this.relatInfoLabel = new System.Windows.Forms.Label();
			this.relationshipsImageList = new System.Windows.Forms.ImageList(this.components);
			this.tabbedPane.SuspendLayout();
			this.compPage.SuspendLayout();
			this.relatPage.SuspendLayout();
			this.SuspendLayout();
			// 
			// appCompImageList
			// 
			this.appCompImageList.ColorDepth = System.Windows.Forms.ColorDepth.Depth8Bit;
			this.appCompImageList.ImageSize = new System.Drawing.Size(32, 32);
			this.appCompImageList.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("appCompImageList.ImageStream")));
			this.appCompImageList.TransparentColor = System.Drawing.Color.Transparent;
			// 
			// platCompImageList
			// 
			this.platCompImageList.ColorDepth = System.Windows.Forms.ColorDepth.Depth8Bit;
			this.platCompImageList.ImageSize = new System.Drawing.Size(32, 32);
			this.platCompImageList.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("platCompImageList.ImageStream")));
			this.platCompImageList.TransparentColor = System.Drawing.Color.Transparent;
			// 
			// appAssmImageList
			// 
			this.appAssmImageList.ColorDepth = System.Windows.Forms.ColorDepth.Depth8Bit;
			this.appAssmImageList.ImageSize = new System.Drawing.Size(32, 32);
			this.appAssmImageList.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("appAssmImageList.ImageStream")));
			this.appAssmImageList.TransparentColor = System.Drawing.Color.Transparent;
			// 
			// platAssmImageList
			// 
			this.platAssmImageList.ColorDepth = System.Windows.Forms.ColorDepth.Depth8Bit;
			this.platAssmImageList.ImageSize = new System.Drawing.Size(32, 32);
			this.platAssmImageList.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("platAssmImageList.ImageStream")));
			this.platAssmImageList.TransparentColor = System.Drawing.Color.Transparent;
			// 
			// tabbedPane
			// 
			this.tabbedPane.Alignment = System.Windows.Forms.TabAlignment.Bottom;
			this.tabbedPane.Controls.AddRange(new System.Windows.Forms.Control[] {
																					 this.compPage,
																					 this.relatPage});
			this.tabbedPane.Dock = System.Windows.Forms.DockStyle.Fill;
			this.tabbedPane.Name = "tabbedPane";
			this.tabbedPane.SelectedIndex = 0;
			this.tabbedPane.Size = new System.Drawing.Size(224, 216);
			this.tabbedPane.TabIndex = 0;
			// 
			// compPage
			// 
			this.compPage.Controls.AddRange(new System.Windows.Forms.Control[] {
																				   this.toolListView,
																				   this.compInfoLabel});
			this.compPage.Location = new System.Drawing.Point(4, 4);
			this.compPage.Name = "compPage";
			this.compPage.Size = new System.Drawing.Size(216, 190);
			this.compPage.TabIndex = 0;
			this.compPage.Text = "Components";
			// 
			// toolListView
			// 
			this.toolListView.Dock = System.Windows.Forms.DockStyle.Fill;
			this.toolListView.HideSelection = false;
			this.toolListView.Location = new System.Drawing.Point(0, 16);
			this.toolListView.MultiSelect = false;
			this.toolListView.Name = "toolListView";
			this.toolListView.Size = new System.Drawing.Size(216, 174);
			this.toolListView.TabIndex = 0;
			// 
			// compInfoLabel
			// 
			this.compInfoLabel.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.compInfoLabel.Dock = System.Windows.Forms.DockStyle.Top;
			this.compInfoLabel.Font = new System.Drawing.Font("Arial", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.compInfoLabel.Name = "compInfoLabel";
			this.compInfoLabel.Size = new System.Drawing.Size(216, 16);
			this.compInfoLabel.TabIndex = 1;
			this.compInfoLabel.Text = "Tool Box";
			// 
			// relatPage
			// 
			this.relatPage.Controls.AddRange(new System.Windows.Forms.Control[] {
																					this.relatListView,
																					this.relatInfoLabel});
			this.relatPage.Location = new System.Drawing.Point(4, 4);
			this.relatPage.Name = "relatPage";
			this.relatPage.Size = new System.Drawing.Size(216, 190);
			this.relatPage.TabIndex = 1;
			this.relatPage.Text = "Relationships";
			// 
			// relatListView
			// 
			this.relatListView.Dock = System.Windows.Forms.DockStyle.Fill;
			this.relatListView.HideSelection = false;
			this.relatListView.Location = new System.Drawing.Point(0, 16);
			this.relatListView.MultiSelect = false;
			this.relatListView.Name = "relatListView";
			this.relatListView.Size = new System.Drawing.Size(216, 174);
			this.relatListView.TabIndex = 2;
			// 
			// relatInfoLabel
			// 
			this.relatInfoLabel.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.relatInfoLabel.Dock = System.Windows.Forms.DockStyle.Top;
			this.relatInfoLabel.Font = new System.Drawing.Font("Arial", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.relatInfoLabel.Name = "relatInfoLabel";
			this.relatInfoLabel.Size = new System.Drawing.Size(216, 16);
			this.relatInfoLabel.TabIndex = 3;
			this.relatInfoLabel.Text = "Tool Box";
			// 
			// relationshipsImageList
			// 
			this.relationshipsImageList.ColorDepth = System.Windows.Forms.ColorDepth.Depth8Bit;
			this.relationshipsImageList.ImageSize = new System.Drawing.Size(32, 32);
			this.relationshipsImageList.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("relationshipsImageList.ImageStream")));
			this.relationshipsImageList.TransparentColor = System.Drawing.Color.Transparent;
			// 
			// JVMSToolBox
			// 
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.tabbedPane});
			this.Name = "JVMSToolBox";
			this.Size = new System.Drawing.Size(224, 216);
			this.tabbedPane.ResumeLayout(false);
			this.compPage.ResumeLayout(false);
			this.relatPage.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

	}
}
