using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Design;
using System.Windows.Forms;
using System.Windows.Forms.Design;

using jvms.datamodel.prf;
using jvms.validation;

namespace jvms.presentation
{
	/// <summary>
	/// Creates an editor for port types.
	/// </summary>
	public class KindsValueEditor : UITypeEditor
	{
		private IWindowsFormsEditorService editorService;

		/// <summary>
		/// Creates a new KindsValueEditor.
		/// </summary>
		public KindsValueEditor()
		{
		}

		/// <summary>
		/// Edit's the specified object's value.
		/// </summary>
		/// <param name="context">Provides contextual information about the object.</param>
		/// <param name="provider">Can be used by the editor to obtain services.</param>
		/// <param name="value">The object to be edited.</param>
		/// <returns>The new value of the object.</returns>
		public override object EditValue( ITypeDescriptorContext context, IServiceProvider provider, Object value )
		{
			editorService = (IWindowsFormsEditorService)provider.GetService( typeof( IWindowsFormsEditorService ) );
			ListBox listBox = new ListBox();
			listBox.BorderStyle = BorderStyle.None;
			listBox.SelectedIndexChanged += new EventHandler( itemChanged );

			ICollection itemsForList = getKinds();
			
			foreach( Object obj in itemsForList )
			{
				listBox.Items.Add( obj );
			}
			
			KindArrayList currentValues = (KindArrayList)value;
			String currentValue = null;
			if( currentValues.Count > 0 )
			{
				currentValue = (string)currentValues[0];
			}
			if( currentValues.Count > 1 )
			{
				currentValue = "configuration/allocation";
			}
			listBox.SelectedItem = currentValue;


			editorService.DropDownControl( listBox );
			if( listBox.SelectedItem != null )
			{
				currentValues.Clear();
				if( listBox.SelectedItem.Equals("configuration/allocation") )
				{
					currentValues.Add( "configuration" );
					currentValues.Add( "allocation" );
				}
				else
				{
					currentValues.Add( listBox.SelectedItem );
				}
			}
			return currentValues;
		}

		private void itemChanged( Object sender, EventArgs e )
		{
			editorService.CloseDropDown();
		}

		/// <summary>
		/// Gets the editor style that will be used to edit the type.
		/// </summary>
		/// <param name="context">Provides contextual information.</param>
		/// <returns>The UITypeEditorEditStyle that will be used.</returns>
		public override UITypeEditorEditStyle GetEditStyle( ITypeDescriptorContext context )
		{
			return UITypeEditorEditStyle.DropDown;
		}

		/// <summary>
		/// Returns a list of acceptable port types.
		/// </summary>
		/// <returns>A list of port types</returns>
		private ICollection getKinds()
		{
			IList attributeTypes = AttributeService.Instance.QueryValues( new Simple() );
			AttributeType kindsAttrType = null;
			foreach ( AttributeType attrType in attributeTypes )
			{
				if ( attrType.BoundSpec.Name.Equals( "Kinds" ) )
				{
					kindsAttrType = attrType;
					break;
				}
			}

			return kindsAttrType.ValidValues;
		}
	}
}
