using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

using jvms.datamodel;
using jvms.datamodel.scd;
using jvms.ruleframework;
using jvms.system;
using jvms.validation;

namespace jvms.presentation
{
	/// <summary>
	/// Represents a dialog which the designer can use to view options.
	/// </summary>
	public class OptionsForm : System.Windows.Forms.Form
	{
		#region Windows Form Designer generated state
		private System.Windows.Forms.ListBox categoriesListBox;
		private System.Windows.Forms.GroupBox separator;
		private System.Windows.Forms.Button okButton;
		private System.Windows.Forms.Button cancelButton;
		private System.Windows.Forms.GroupBox optionsPanel;
		private System.Windows.Forms.ImageList rulesImageList;
		private System.ComponentModel.IContainer components;
		#endregion

		private Hashtable optionsPanels = new Hashtable();
		private Hashtable colors = new Hashtable();
		private Hashtable defaultColors = new Hashtable();
		private Hashtable ruleCategoryNodes = new Hashtable();
		private Hashtable checkedOptions = new Hashtable();
		private Hashtable checkedRules = new Hashtable();

		private ListBox interfacesListBox;
		private TextBox nameTextBox;
		private TextBox repidTextBox;
		private TextBox inheritTextBox;

		private TextBox ruleDescriptionTextBox;

		/// <summary>
		/// Creates a new OptionsForm.
		/// </summary>
		public OptionsForm()
		{
			InitializeComponent();
			initGeneralPanel();
			initDefaultColors();
			initInterfacePanel();
			initRulesPanel();
			initColorPanel();

			categoriesListBox.SelectedValueChanged += new EventHandler( categorySelected );
			okButton.Click += new EventHandler( okButtonClicked );
			cancelButton.Click += new EventHandler( cancelButtonClicked );
		}

		/// <summary>
		/// Initializes the general panel.
		/// </summary>
		private void initGeneralPanel()
		{
			Panel generalPanel = new Panel();
			JVMSProperties props = JVMSProperties.Instance;

			CheckBox loadLastCheckBox = new CheckBox();
			loadLastCheckBox.Text = "Load last project when starting JVMS.";
			loadLastCheckBox.Checked = (bool)props.getProperty( "loadlast", false );
			loadLastCheckBox.Tag = "loadlast";
			loadLastCheckBox.Size = new Size( 250, 20 );
			loadLastCheckBox.Location = new Point( 10, 5 );
			loadLastCheckBox.CheckStateChanged += new EventHandler( optionChecked );

			generalPanel.Controls.AddRange( new Control[] { loadLastCheckBox } );

			optionsPanels["general"] = generalPanel;
		}

		/// <summary>
		/// Initializes the interfaces panel.
		/// </summary>
		private void initInterfacePanel()
		{
			Panel interfacesPanel = new Panel();
			JVMSProperties props = JVMSProperties.Instance;

			interfacesListBox = new ListBox();
			interfacesListBox.Size = new Size( 160, 260 );
			interfacesListBox.Location = new Point( 5, 8 );
			interfacesListBox.Sorted = true;
			interfacesListBox.ScrollAlwaysVisible = true;
			interfacesListBox.SelectionMode = SelectionMode.One;
			interfacesListBox.SelectedValueChanged += new EventHandler( interfaceSelected );

			ArrayList interfaceList = (ArrayList)props.getProperty( "interfaces", new ArrayList() );
			interfacesListBox.Items.AddRange( interfaceList.ToArray() );

			Label nameLabel = new Label();
			nameLabel.Text = "Interface Name";
			nameLabel.Size = new Size( 90, 20 );
			nameLabel.Location = new Point( interfacesListBox.Right + 10, interfacesListBox.Top + 5 );
			nameLabel.TextAlign = ContentAlignment.MiddleLeft;

			nameTextBox = new TextBox();
			nameTextBox.Size = new Size( 170, 20 );
			nameTextBox.Location = new Point( nameLabel.Right + 5, nameLabel.Top );

			Label repidLabel = new Label();
			repidLabel.Text = "Repository Id";
			repidLabel.Size = new Size( 90, 20 );
			repidLabel.Location = new Point( nameLabel.Left, nameLabel.Bottom + 5 );
			repidLabel.TextAlign = ContentAlignment.MiddleLeft;

			repidTextBox = new TextBox();
			repidTextBox.Size = new Size( 170, 20 );
			repidTextBox.Location = new Point( repidLabel.Right + 5, repidLabel.Top );

			Label inheritLabel = new Label();
			inheritLabel.Text = "Inherits Interfaces (type repository ids, one per line)";
			inheritLabel.Size = new Size( 270, 20 );
			inheritLabel.Location = new Point( repidLabel.Left, repidLabel.Bottom + 30 );
			inheritLabel.TextAlign = ContentAlignment.MiddleLeft;

			inheritTextBox = new TextBox();
			inheritTextBox.Size = new Size( 255, 100 );
			inheritTextBox.Location = new Point( inheritLabel.Left, inheritLabel.Bottom + 2 );
			inheritTextBox.Multiline = true;

			Button delButton = new Button();
			delButton.Text = "Remove";
			delButton.Width = 70;
			delButton.Location = new Point( inheritTextBox.Right - delButton.Width, inheritTextBox.Bottom + 10 );
			delButton.Click += new EventHandler( delInterfaceButtonClicked );

			Button addButton = new Button();
			addButton.Text = "Add";
			addButton.Width = 70;
			addButton.Location = new Point( delButton.Left - addButton.Width - 5, delButton.Top );
			addButton.Click += new EventHandler( addInterfaceButtonClicked );

			interfacesPanel.Controls.AddRange(
				new Control[] { interfacesListBox, nameLabel, repidLabel, inheritLabel,
								nameTextBox, repidTextBox, inheritTextBox, addButton, delButton } );

			optionsPanels["interfaces"] = interfacesPanel;
		}

		/// <summary>
		/// Initializses the rules options panel.
		/// </summary>
		private void initRulesPanel()
		{
			Panel rulesPanel = new Panel();

			TreeView rulesTreeView = new TreeView();
			rulesTreeView.CheckBoxes = true;
			rulesTreeView.HideSelection = false;
			rulesTreeView.ImageList = rulesImageList;
			rulesTreeView.Size = new Size( 200, 250 );
			rulesTreeView.Location = new Point( 5, 8 );
			rulesTreeView.AfterCheck += new TreeViewEventHandler( rulesTreeViewChecked );
			rulesTreeView.AfterSelect += new TreeViewEventHandler( rulesTreeViewSelected );
			rulesTreeView.AfterExpand += new TreeViewEventHandler( rulesTreeViewExpanded );
			rulesTreeView.AfterCollapse += new TreeViewEventHandler( rulesTreeViewCollapsed );

			initRulesTreeView( rulesTreeView );

			GroupBox descriptionBox = new GroupBox();
			descriptionBox.Text = "Description";
			descriptionBox.Size = new Size( 235, 125 );
			descriptionBox.Location = new Point( rulesTreeView.Right + 5, rulesTreeView.Top );

			ruleDescriptionTextBox = new TextBox();
			ruleDescriptionTextBox.ReadOnly = true;
			ruleDescriptionTextBox.Multiline = true;
			ruleDescriptionTextBox.ScrollBars = ScrollBars.Vertical;
			ruleDescriptionTextBox.Font = new Font( "Arial", 9.0f );
			ruleDescriptionTextBox.Dock = DockStyle.Fill;

			GroupBox ruleOptionsBox = new GroupBox();
			ruleOptionsBox.Text = "Rule Options";
			ruleOptionsBox.Size = new Size( 235, 120 );
			ruleOptionsBox.Location = new Point( descriptionBox.Left, descriptionBox.Bottom + 5 );

			descriptionBox.Controls.Add( ruleDescriptionTextBox );
			rulesPanel.Controls.AddRange( new Control[] { rulesTreeView, descriptionBox, ruleOptionsBox } );

			optionsPanels["rules"] = rulesPanel;
		}

		/// <summary>
		/// Initializes the colors options panel.
		/// </summary>
		private void initColorPanel()
		{
			Panel colorsPanel = new Panel();
			JVMSProperties props = JVMSProperties.Instance;
			int labelWidth = 120;
			int labelHeight = 30;
			
			int buttonWidth = 70;
			int buttonHeight = 20;

			Label scaHeaderLabel = new Label();
			scaHeaderLabel.Text = "Component Header";
			scaHeaderLabel.Size = new Size( labelWidth, labelHeight );
			scaHeaderLabel.Location = new Point( 5, 10 );
			scaHeaderLabel.TextAlign = ContentAlignment.MiddleLeft;

			Button scaHeaderButton = new Button();
			scaHeaderButton.Tag = "color.scaheader";
			scaHeaderButton.BackColor = getColor( (String)scaHeaderButton.Tag );
			scaHeaderButton.FlatStyle = FlatStyle.Flat;
			scaHeaderButton.Size = new Size( buttonWidth, buttonHeight );
			scaHeaderButton.Location = new Point( scaHeaderLabel.Right + 20, scaHeaderLabel.Top + 5 );
			scaHeaderButton.Click += new EventHandler( colorButtonClicked );

			Label portLabel = new Label();
			portLabel.Text = "Ports";
			portLabel.Size = new Size( labelWidth, labelHeight );
			portLabel.Location = new Point( scaHeaderLabel.Left, scaHeaderLabel.Bottom );
			portLabel.TextAlign = ContentAlignment.MiddleLeft;

			Button portButton = new Button();
			portButton.Tag = "color.port";
			portButton.BackColor = getColor( (String)portButton.Tag );
			portButton.FlatStyle = FlatStyle.Flat;
			portButton.Size = new Size( buttonWidth, buttonHeight );
			portButton.Location = new Point( portLabel.Right + 20, portLabel.Top + 5 );
			portButton.Click += new EventHandler( colorButtonClicked );

			Label nonScaHeaderLabel = new Label();
			nonScaHeaderLabel.Text = "Non-SCA Header";
			nonScaHeaderLabel.Size = new Size( labelWidth, labelHeight );
			nonScaHeaderLabel.Location = new Point( portLabel.Left, portLabel.Bottom );
			nonScaHeaderLabel.TextAlign = ContentAlignment.MiddleLeft;

			Button nonScaHeaderButton = new Button();
			nonScaHeaderButton.Tag = "color.nonscaheader";
			nonScaHeaderButton.BackColor = getColor( (String)nonScaHeaderButton.Tag );
			nonScaHeaderButton.FlatStyle = FlatStyle.Flat;
			nonScaHeaderButton.Size = new Size( buttonWidth, buttonHeight );
			nonScaHeaderButton.Location = new Point( nonScaHeaderLabel.Right + 20, nonScaHeaderLabel.Top + 5 );
			nonScaHeaderButton.Click += new EventHandler( colorButtonClicked );

			Label propsHeaderLabel = new Label();
			propsHeaderLabel.Text = "Properties Header";
			propsHeaderLabel.Size = new Size( labelWidth, labelHeight );
			propsHeaderLabel.Location = new Point( nonScaHeaderLabel.Left, nonScaHeaderLabel.Bottom );
			propsHeaderLabel.TextAlign = ContentAlignment.MiddleLeft;

			Button propsHeaderButton = new Button();
			propsHeaderButton.Tag = "color.propsheader";
			propsHeaderButton.BackColor = getColor( (String)propsHeaderButton.Tag );
			propsHeaderButton.FlatStyle = FlatStyle.Flat;
			propsHeaderButton.Size = new Size( buttonWidth, buttonHeight );
			propsHeaderButton.Location = new Point( propsHeaderLabel.Right + 20, propsHeaderLabel.Top + 5 );
			propsHeaderButton.Click += new EventHandler( colorButtonClicked );

			Label applicationHeaderLabel = new Label();
			applicationHeaderLabel.Text = "Application Header";
			applicationHeaderLabel.Size = new Size( labelWidth, labelHeight );
			applicationHeaderLabel.Location = new Point( propsHeaderLabel.Left, propsHeaderLabel.Bottom );
			applicationHeaderLabel.TextAlign = ContentAlignment.MiddleLeft;

			Button applicationHeaderButton = new Button();
			applicationHeaderButton.Tag = "color.appheader";
			applicationHeaderButton.BackColor = getColor( (String)applicationHeaderButton.Tag );
			applicationHeaderButton.FlatStyle = FlatStyle.Flat;
			applicationHeaderButton.Size = new Size( buttonWidth, buttonHeight );
			applicationHeaderButton.Location = new Point( applicationHeaderLabel.Right + 20, applicationHeaderLabel.Top + 5 );			
			applicationHeaderButton.Click += new EventHandler( colorButtonClicked );

			Label deviceConfigHeaderLabel = new Label();
			deviceConfigHeaderLabel.Text = "Device Config Header";
			deviceConfigHeaderLabel.Size = new Size( labelWidth, labelHeight );
			deviceConfigHeaderLabel.Location = new Point( applicationHeaderLabel.Left, applicationHeaderLabel.Bottom );
			deviceConfigHeaderLabel.TextAlign = ContentAlignment.MiddleLeft;

			Button deviceConfigHeaderButton = new Button();
			deviceConfigHeaderButton.Tag = "color.deviceconfigheader";
			deviceConfigHeaderButton.BackColor = getColor( (String)deviceConfigHeaderButton.Tag );
			deviceConfigHeaderButton.FlatStyle = FlatStyle.Flat;
			deviceConfigHeaderButton.Size = new Size( buttonWidth, buttonHeight );
			deviceConfigHeaderButton.Location = new Point( deviceConfigHeaderLabel.Right + 20, deviceConfigHeaderLabel.Top + 5 );
			deviceConfigHeaderButton.Click += new EventHandler( colorButtonClicked );

			Label instanceHeaderLabel = new Label();
			instanceHeaderLabel.Text = "Instance Header";
			instanceHeaderLabel.Size = new Size( labelWidth, labelHeight );
			instanceHeaderLabel.Location = new Point( deviceConfigHeaderLabel.Left, deviceConfigHeaderLabel.Bottom );
			instanceHeaderLabel.TextAlign = ContentAlignment.MiddleLeft;

			Button instanceHeaderButton = new Button();
			instanceHeaderButton.Tag = "color.instanceheader";
			instanceHeaderButton.BackColor = getColor( (String)instanceHeaderButton.Tag );
			instanceHeaderButton.FlatStyle = FlatStyle.Flat;
			instanceHeaderButton.Size = new Size( buttonWidth, buttonHeight );
			instanceHeaderButton.Location = new Point( instanceHeaderLabel.Right + 20, instanceHeaderLabel.Top + 5 );
			instanceHeaderButton.Click += new EventHandler( colorButtonClicked );

			Label contentLabel = new Label();
			contentLabel.Text = "Component Content";
			contentLabel.Size = new Size( labelWidth, labelHeight );
			contentLabel.Location = new Point( instanceHeaderLabel.Left, instanceHeaderLabel.Bottom );
			contentLabel.TextAlign = ContentAlignment.MiddleLeft;

			Button contentButton = new Button();
			contentButton.Tag = "color.content";
			contentButton.BackColor = getColor( (String)contentButton.Tag );
			contentButton.FlatStyle = FlatStyle.Flat;
			contentButton.Size = new Size( buttonWidth, buttonHeight );
			contentButton.Location = new Point( contentLabel.Right + 20, contentLabel.Top + 5 );
			contentButton.Click += new EventHandler( colorButtonClicked );

			colorsPanel.Controls.AddRange( 
				new Control[] { scaHeaderLabel, portLabel, nonScaHeaderLabel, propsHeaderLabel,
								applicationHeaderLabel, deviceConfigHeaderLabel, instanceHeaderLabel,
								contentLabel } );

			colorsPanel.Controls.AddRange(
				new Control[] { scaHeaderButton, portButton, nonScaHeaderButton, propsHeaderButton,
								applicationHeaderButton, deviceConfigHeaderButton, instanceHeaderButton,
								contentButton } );

			optionsPanels["colors"] = colorsPanel;
		}

		/// <summary>
		/// Initializes a list of default colors.
		/// </summary>
		private void initDefaultColors()
		{
			defaultColors["color.scaheader"] = Color.Aqua;
			defaultColors["color.port"] = Color.LightBlue;
			defaultColors["color.nonscaheader"] = Color.LemonChiffon;
			defaultColors["color.propsheader"] = Color.PowderBlue;
			defaultColors["color.appheader"] = Color.PowderBlue;
			defaultColors["color.deviceconfigheader"] = Color.PowderBlue;
			defaultColors["color.instanceheader"] = Color.CornflowerBlue;
			defaultColors["color.content"] = Color.Beige;
		}

		/// <summary>
		/// Returns the Color object for the given property.
		/// </summary>
		/// <param name="propName">The name of the desired color property.</param>
		/// <returns>The corresponding color.</returns>
		private Color getColor( String propName )
		{
			return (Color)JVMSProperties.Instance.getProperty( propName, ( (Color)defaultColors[propName] ) );
		}

		/// <summary>
		/// Initializes the rules tree view control.
		/// </summary>
		/// <param name="treeView">The tree view control.</param>
		private void initRulesTreeView( TreeView treeView )
		{
			IList rules = RuleEngine.Instance.Rules;
			IList categories = new ArrayList();
			IList selectedCategories = (IList)JVMSProperties.Instance.getProperty( "selectedrulecategories", new ArrayList() );
			IList selectedRules = (IList)JVMSProperties.Instance.getProperty( "selectedrules", new ArrayList() );


			foreach ( Rule r in rules )
			{
				if ( !categories.Contains( r.Category ) )
				{
					categories.Add( r.Category );
				}
			}

			foreach ( String category in categories )
			{
				TreeNode categoryNode = new TreeNode( category, 0, 0 );

				if ( selectedCategories.Contains( category ) )
				{
					categoryNode.Checked = true;
				}

				ruleCategoryNodes.Add( category, categoryNode );
				treeView.Nodes.Add( categoryNode );
			}

			foreach ( Rule r in rules )
			{
				TreeNode categoryNode = (TreeNode)ruleCategoryNodes[r.Category];
				if ( categoryNode != null )
				{
					TreeNode ruleNode = new TreeNode( r.Name, 2, 2 );
					ruleNode.Tag = r;

					if ( categoryNode.Checked || selectedRules.Contains( r.Name ) )
					{
						ruleNode.Checked = true;
					}

					categoryNode.Nodes.Add( ruleNode );
				}
			}
		}

		/// <summary>
		/// Saves the options that have been selected.
		/// </summary>
		private void setOptions()
		{
			JVMSProperties props = JVMSProperties.Instance;

			foreach ( String propName in checkedOptions.Keys )
			{
				props.setProperty( propName, checkedOptions[propName] );
			}

			foreach ( String propName in colors.Keys )
			{
				props.setProperty( propName, colors[propName] );
			}

			IList selectedRuleCategories = new ArrayList();
			IList selectedRules = new ArrayList();
			foreach ( TreeNode categoryNode in ruleCategoryNodes.Values )
			{
				if ( categoryNode.Checked )
				{
					selectedRuleCategories.Add( categoryNode.Text );
				}

				foreach ( TreeNode ruleNode in categoryNode.Nodes )
				{
					if ( ruleNode.Checked )
					{
						selectedRules.Add( ruleNode.Text );
					}
				}
			}
			props.setProperty( "selectedrulecategories", selectedRuleCategories );
			props.setProperty( "selectedrules", selectedRules );

			ArrayList interfaceList = new ArrayList();
			foreach ( Interface i in interfacesListBox.Items )
			{
				interfaceList.Add( i );
			}
			props.setProperty( "interfaces", interfaceList );
		}

		/// <summary>
		/// Called when an options category is selected.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void categorySelected( Object source, EventArgs e )
		{
			String category = categoriesListBox.Text;

			optionsPanel.SuspendLayout();
			optionsPanel.Controls.Clear();
			optionsPanel.Text = category;
			Panel p = (Panel)optionsPanels[category.ToLower()];
			if ( p != null )
			{
				p.Dock = DockStyle.Fill;
				optionsPanel.Controls.Add( p );
			}
			optionsPanel.ResumeLayout();
		}

		/// <summary>
		/// Called when an option check box has been checked.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void optionChecked( Object source, EventArgs e )
		{
			CheckBox cb = (CheckBox)source;
			checkedOptions[cb.Tag] = cb.Checked;
		}

		/// <summary>
		/// Called when a color button has been selected.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void colorButtonClicked( Object source, EventArgs e )
		{
			Button b = (Button)source;
			ColorDialog dialog = new ColorDialog();
			dialog.Color = b.BackColor;
			dialog.ShowDialog( this );
			b.BackColor = dialog.Color;
			colors[b.Tag] = dialog.Color;
		}

		/// <summary>
		/// Called when the selected interface has changed.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void interfaceSelected( Object source, EventArgs e )
		{
			Interface i = (Interface)interfacesListBox.SelectedItem;

			if ( i != null )
			{
				nameTextBox.Text = i.Name;
				repidTextBox.Text = i.RepositoryID;
				inheritTextBox.Clear();

				IList inheritList = i.Inherits;
				foreach ( Interface ii in inheritList )
				{
					inheritTextBox.AppendText( ii.RepositoryID + "\r\n" );
				}
			}
		}

		/// <summary>
		/// Called when the add interface button has been selected.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void addInterfaceButtonClicked( Object source, EventArgs e )
		{
			if ( !nameTextBox.Text.Equals( "" ) )
			{
				Interface i = new Interface();
				i.Name = nameTextBox.Text;
				i.RepositoryID = repidTextBox.Text;

				JVMSProperties props = JVMSProperties.Instance;
				TypedArrayList inheritsList = new TypedArrayList( typeof(Interface) );
				foreach ( String s in inheritTextBox.Lines )
				{
					if ( !s.Equals( "" ) )
					{
						Interface parentInterface = null;
						foreach( Interface item in interfacesListBox.Items )
						{
							if( item.RepositoryID.Equals(s) )
							{
								parentInterface = item;
								break;
							}
						}
						if( parentInterface == null )
						{
							MessageBox.Show( "Interface with id = " + s + 
								" could not be found. Please create it first." );
							return;
						}
						else
						{
							inheritsList.Add( parentInterface );
						}
					}
				}

				i.Inherits = inheritsList;

				interfacesListBox.SuspendLayout();
				Interface selectedInterface = (Interface)interfacesListBox.SelectedItem;
				if ( selectedInterface == null || !selectedInterface.Name.Equals( i.Name ) )
				{
					interfacesListBox.Items.Add( i );
				}
				else
				{
					interfacesListBox.Items[interfacesListBox.SelectedIndex] = i;
				}

				nameTextBox.Text = "";
				repidTextBox.Text = "";
				inheritTextBox.Text = "";
				interfacesListBox.ClearSelected();
				interfacesListBox.ResumeLayout();
			}
		}

		/// <summary>
		/// Called when the remove interface button has been selected.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void delInterfaceButtonClicked( Object source, EventArgs e )
		{
			Interface selectedInterface = (Interface)interfacesListBox.SelectedItem;

			if ( selectedInterface != null )
			{
				interfacesListBox.SuspendLayout();
				interfacesListBox.Items.Remove( selectedInterface );
				nameTextBox.Text = "";
				repidTextBox.Text = "";
				inheritTextBox.Text = "";
				interfacesListBox.ClearSelected();
				interfacesListBox.ResumeLayout();
			}
		}

		/// <summary>
		/// Called when the rules tree view has been expanded.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void rulesTreeViewExpanded( Object source, TreeViewEventArgs e )
		{
			if ( e.Node.ImageIndex == 0 )
			{
				e.Node.ImageIndex = 1;
				e.Node.SelectedImageIndex = 1;
			}
		}

		/// <summary>
		/// Called when a node in the rules tree view has been checked.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void rulesTreeViewChecked( Object source, TreeViewEventArgs e )
		{
			if ( ruleCategoryNodes.ContainsKey( e.Node.Text ) )
			{
				foreach ( TreeNode node in e.Node.Nodes )
				{
					node.Checked = e.Node.Checked;
				}
			}
			else
			{
				if ( !e.Node.Checked && e.Node.Parent.Checked )
				{
					e.Node.Parent.Checked = false;
				}
			}
		}

		/// <summary>
		/// Called when a node in the rules tree view has been selected.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void rulesTreeViewSelected( Object source, TreeViewEventArgs e )
		{
			if ( !ruleCategoryNodes.ContainsKey( e.Node.Text ) )
			{
				Rule r = (Rule)e.Node.Tag;
				ruleDescriptionTextBox.Text = r.Description;
			}
			else
			{
				ruleDescriptionTextBox.Text = "";
			}
		}

		/// <summary>
		/// Called when the rules tree view has been collapsed.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void rulesTreeViewCollapsed( Object source, TreeViewEventArgs e )
		{
			if ( e.Node.ImageIndex == 1 )
			{
				e.Node.ImageIndex = 0;
				e.Node.SelectedImageIndex = 0;
			}
		}

		/// <summary>
		/// Called when the ok button has been selected.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void okButtonClicked( Object source, EventArgs e )
		{
			setOptions();
			this.Close();
			JVMSMain.JvmsForm.CurrentView.Refresh();
		}

		/// <summary>
		/// Called when the cancel button has been selected.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void cancelButtonClicked( Object source, EventArgs e )
		{
			this.Close();
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(OptionsForm));
			this.categoriesListBox = new System.Windows.Forms.ListBox();
			this.optionsPanel = new System.Windows.Forms.GroupBox();
			this.separator = new System.Windows.Forms.GroupBox();
			this.okButton = new System.Windows.Forms.Button();
			this.cancelButton = new System.Windows.Forms.Button();
			this.rulesImageList = new System.Windows.Forms.ImageList(this.components);
			this.SuspendLayout();
			// 
			// categoriesListBox
			// 
			this.categoriesListBox.Font = new System.Drawing.Font("Arial", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.categoriesListBox.ItemHeight = 15;
			this.categoriesListBox.Items.AddRange(new object[] {
																   "General",
																   "Interfaces",
																   "Rules",
																   "Colors"});
			this.categoriesListBox.Location = new System.Drawing.Point(8, 8);
			this.categoriesListBox.Name = "categoriesListBox";
			this.categoriesListBox.ScrollAlwaysVisible = true;
			this.categoriesListBox.Size = new System.Drawing.Size(144, 289);
			this.categoriesListBox.TabIndex = 0;
			// 
			// optionsPanel
			// 
			this.optionsPanel.Location = new System.Drawing.Point(160, 8);
			this.optionsPanel.Name = "optionsPanel";
			this.optionsPanel.Size = new System.Drawing.Size(464, 288);
			this.optionsPanel.TabIndex = 1;
			this.optionsPanel.TabStop = false;
			this.optionsPanel.Text = "Options";
			// 
			// separator
			// 
			this.separator.Location = new System.Drawing.Point(8, 304);
			this.separator.Name = "separator";
			this.separator.Size = new System.Drawing.Size(616, 8);
			this.separator.TabIndex = 2;
			this.separator.TabStop = false;
			// 
			// okButton
			// 
			this.okButton.Location = new System.Drawing.Point(472, 320);
			this.okButton.Name = "okButton";
			this.okButton.TabIndex = 3;
			this.okButton.Text = "OK";
			// 
			// cancelButton
			// 
			this.cancelButton.Location = new System.Drawing.Point(552, 320);
			this.cancelButton.Name = "cancelButton";
			this.cancelButton.TabIndex = 4;
			this.cancelButton.Text = "Cancel";
			// 
			// rulesImageList
			// 
			this.rulesImageList.ColorDepth = System.Windows.Forms.ColorDepth.Depth8Bit;
			this.rulesImageList.ImageSize = new System.Drawing.Size(16, 16);
			this.rulesImageList.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("rulesImageList.ImageStream")));
			this.rulesImageList.TransparentColor = System.Drawing.Color.Transparent;
			// 
			// OptionsForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(634, 351);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.cancelButton,
																		  this.okButton,
																		  this.optionsPanel,
																		  this.categoriesListBox,
																		  this.separator});
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.Name = "OptionsForm";
			this.ShowInTaskbar = false;
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterParent;
			this.Text = "JVMS Options";
			this.ResumeLayout(false);

		}
		#endregion

	}
}
