using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

using jvms.presentation.relationships;

namespace jvms.presentation
{
	/// <summary>
	/// Represents a dialog used for selecting a relationship from a list.
	/// </summary>
	public class SelectRelationshipDialog : System.Windows.Forms.Form
	{
		#region Windows Form Designer generated state
		private System.Windows.Forms.ListBox relationshipsListBox;
		private System.ComponentModel.Container components = null;
		#endregion

		private ArrayList relationships;

		/// <summary>Gets or sets the list of relationships.</summary>
		/// <value>The list of relationships.</value>
		public IList Relationships
		{
			get
			{
				return relationships;
			}

			set
			{
				relationships = new ArrayList( value );

				relationshipsListBox.Items.Clear();
				relationshipsListBox.Items.AddRange( relationships.ToArray() );
			}
		}

		/// <summary>
		/// Creates a new SelectRelationshipDialog.
		/// </summary>
		public SelectRelationshipDialog()
		{
			InitializeComponent();

			Location = Cursor.Position;
			relationshipsListBox.SelectedValueChanged += new EventHandler( itemSelected );
		}

		/// <summary>
		/// Shows the selection dialog and returns the selected relationship, or null if none was selected.
		/// </summary>
		/// <returns>The selected relationship, or null if there was none.</returns>
		public IRelationship showSelectionDialog()
		{
			ShowDialog( JVMSMain.JvmsForm );

			return (IRelationship)relationshipsListBox.SelectedItem;
		}

		/// <summary>
		/// Called when an item in the list box has been selected.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		public void itemSelected( Object source, EventArgs e )
		{
			Close();
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.relationshipsListBox = new System.Windows.Forms.ListBox();
			this.SuspendLayout();
			// 
			// relationshipsListBox
			// 
			this.relationshipsListBox.Dock = System.Windows.Forms.DockStyle.Fill;
			this.relationshipsListBox.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.relationshipsListBox.IntegralHeight = false;
			this.relationshipsListBox.Name = "relationshipsListBox";
			this.relationshipsListBox.ScrollAlwaysVisible = true;
			this.relationshipsListBox.Size = new System.Drawing.Size(226, 58);
			this.relationshipsListBox.TabIndex = 0;
			// 
			// SelectRelationshipDialog
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(226, 58);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.relationshipsListBox});
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedToolWindow;
			this.Name = "SelectRelationshipDialog";
			this.ShowInTaskbar = false;
			this.StartPosition = System.Windows.Forms.FormStartPosition.Manual;
			this.Text = "Select Relationship";
			this.ResumeLayout(false);

		}
		#endregion
	}
}
