using System;
using System.ComponentModel;
using System.Drawing.Design;
using System.Windows.Forms.Design;

using jvms.datamodel;
using jvms.datamodel.prf;


namespace jvms.presentation
{
	/// <summary>
	/// Represents a value type editor for complex XML types.
	/// </summary>
	public class SimpleRefEditor : UITypeEditor
	{
		private IWindowsFormsEditorService editorService;

		/// <summary>
		/// Creates a new ComplexXMLValueEditor.
		/// </summary>
		public SimpleRefEditor()
		{
		}

		/// <summary>
		/// Edit's the specified object's value.
		/// </summary>
		/// <param name="context">Provides contextual information about the object.</param>
		/// <param name="provider">Can be used by the editor to obtain services.</param>
		/// <param name="value">The object to be edited.</param>
		/// <returns>The new value of the object.</returns>
		public override object EditValue( ITypeDescriptorContext context, IServiceProvider provider, Object value )
		{
			SimpleRefEditorForm form = new SimpleRefEditorForm();
			editorService = (IWindowsFormsEditorService)provider.GetService( typeof( IWindowsFormsEditorService ) );
			form.SetSimpleProperty( (Simple)value );
			editorService.ShowDialog( form );
			return form.GetSimpleProperty();
		}

		/// <summary>
		/// Gets the editor style that will be used to edit the type.
		/// </summary>
		/// <param name="context">Provides contextual information.</param>
		/// <returns>The UITypeEditorEditStyle that will be used.</returns>
		public override UITypeEditorEditStyle GetEditStyle( ITypeDescriptorContext context )
		{
			return UITypeEditorEditStyle.Modal;
		}
	}
}
