using System;
using System.Drawing;
using System.Windows.Forms;

using jvms.system;

namespace jvms.presentation
{
	/// <summary>
	/// Controls access to the main form's status bar, and provides convenient
	/// methods for showing the status of the current operation.
	/// </summary>
	public class StatusBarManager
	{
		private static StatusBarManager instance = null;
		
		private StatusBar statusBar;
		private StatusBarPanel statusTextPanel;
		private StatusBarPanel progressBarPanel;
		private StatusBarPanel helpTextPanel;

		private int progressMin;
		private int progressMax;
		private int progressVal;

		/// <summary>Gets the working instance of StatusBarManager.</summary>
		/// <value>The singleton instance of StatusBarManager.</value>
		public static StatusBarManager Instance
		{
			get
			{
				if ( instance == null )
				{
					instance = new StatusBarManager();
				}
				return instance;
			}
		}

		/// <summary>Sets or gets the text on the main status bar panel.</summary>
		/// <value>The text which appears in the main status bar panel.</value>
		public String StatusText
		{
			get
			{
				return statusTextPanel.Text;
			}

			set
			{
				if ( value.Equals( "" ) )
				{
					statusTextPanel.Text = "JTRS Visual Modeling Studio";
				}
				else
				{
					statusTextPanel.Text = value;
				}
			}
		}

		/// <summary>Sets or gets the text on the help status bar panel.</summary>
		/// <value>The text which appears in the help status bar panel.</value>
		public String HelpText
		{
			get
			{
				return helpTextPanel.Text;
			}

			set
			{
				statusTextPanel.Text = value;
			}
		}

		/// <summary>Sets or gets the minimum value of the progress bar.</summary>
		/// <value>The minimum, or starting, value of the progress bar.</value>
		public int ProgressMin
		{
			get
			{
				return progressMin;
			}

			set
			{
				progressMin = value;
			}
		}

		/// <summary>Sets or gets the maximum progress bar value.</summary>
		/// <value>The maximum value of the progress bar.</value>
		public int ProgressMax
		{
			get
			{
				return progressMax;
			}

			set
			{
				progressMax = value;
			}
		}
		
		/// <summary>Sets or gets the progress bar value.</summary>
		/// <value>The current value of the progress bar.</value>
		public int ProgressValue
		{
			get
			{
				return progressVal;
			}

			set
			{
				if ( value >= progressMin && value <= progressMax )
				{
					progressVal = value;
				}
			}
		}

		/// <summary>
		/// Creates a new StatusBarManager.
		/// </summary>
		private StatusBarManager()
		{
			statusBar = JVMSMain.JvmsForm.FormStatusBar;
			statusTextPanel = statusBar.Panels[0];
			progressBarPanel = statusBar.Panels[1];
			helpTextPanel = statusBar.Panels[2];
		}
	}
}
