using System;
using System.ComponentModel;
using System.Drawing.Design;
using System.Windows.Forms.Design;
using System.Collections;

namespace jvms.presentation
{
	/// <summary>
	/// Represents a value editor for arrays of strings.
	/// </summary>
	public class StringArrayValueEditor : UITypeEditor
	{
		private IWindowsFormsEditorService editorService;

		/// <summary>
		/// Creates a new StringArrayValueEditor.
		/// </summary>
		public StringArrayValueEditor()
		{
		}

		/// <summary>
		/// Edit's the specified object's value.
		/// </summary>
		/// <param name="context">Provides contextual information about the object.</param>
		/// <param name="provider">Can be used by the editor to obtain services.</param>
		/// <param name="value">The object to be edited.</param>
		/// <returns>The new value of the object.</returns>
		public override object EditValue( ITypeDescriptorContext context, IServiceProvider provider, Object value )
		{
			String categoryName = context.PropertyDescriptor.Category;
			String attributeName = context.PropertyDescriptor.DisplayName;
			if ( attributeName.StartsWith( "*" ) )
			{
				attributeName = attributeName.Substring( 2 );
			}

			StringArrayEditorForm form = new StringArrayEditorForm();
			form.Text = "Editor Dialog (" + categoryName + "::" + attributeName + ")";
			editorService = (IWindowsFormsEditorService)provider.GetService( typeof( IWindowsFormsEditorService ) );
			form.SetList( (IList)value );
			editorService.ShowDialog( form );
			return value;
		}
		
		/// <summary>
		/// Gets the editor style that will be used to edit the type.
		/// </summary>
		/// <param name="context">Provides contextual information.</param>
		/// <returns>The UITypeEditorEditStyle that will be used.</returns>
		public override UITypeEditorEditStyle GetEditStyle( ITypeDescriptorContext context )
		{
			return UITypeEditorEditStyle.Modal;
		}
	}
}
