using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;

using jvms.presentation.drawablecomponent;
using jvms.validation;

namespace jvms.presentation
{
	/// <summary>
	/// Represents a control that can display a list of tasks to the designer.
	/// </summary>
	public class TaskList : System.Windows.Forms.UserControl
	{
		#region Component Designer generated state
		private System.Windows.Forms.ListView taskListView;
		private System.Windows.Forms.ColumnHeader imageHeader;
		private System.Windows.Forms.ColumnHeader msgTypeHeader;
		private System.Windows.Forms.ColumnHeader messageHeader;
		private System.Windows.Forms.ColumnHeader componentHeader;
		private System.Windows.Forms.ImageList iconImageList;
		private System.ComponentModel.IContainer components;
		#endregion

		/// <summary>
		/// Creates a new TaskList.
		/// </summary>
		public TaskList()
		{
			InitializeComponent();

			Layout += new LayoutEventHandler( doLayout );

			taskListView.ItemActivate += new EventHandler( taskActivated );
		}

		/// <summary>
		/// Clears the list of tasks.
		/// </summary>
		public void clearTasks()
		{
			taskListView.Items.Clear();
		}

		public void clearItem( DrawableComponent component )
		{
			foreach( ListViewItem item in taskListView.Items )
			{
				if( ((RuleResult)item.Tag).Component == component )
				{
					taskListView.Items.Remove( item );
				}
			}
		}

		/// <summary>
		/// Adds a task specified by a given RuleResult.
		/// </summary>
		/// <param name="result">The task data.</param>
		public void addTask( RuleResult result )
		{
			ListViewItem taskItem = new ListViewItem();
			taskItem.Tag = result;
			taskItem.ImageIndex = 0;

			taskItem.SubItems.Add( "Error" );
			taskItem.SubItems.Add( result.ErrorMessage );
			taskItem.SubItems.Add( result.Component.getWrappedObject().ToString() );

			taskListView.Items.Add( taskItem );
		}

		/// <summary>
		/// Called when the control should perform layout.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void doLayout( Object source, LayoutEventArgs e )
		{
			messageHeader.Width = (int)( Width * 0.65f );
			componentHeader.Width = Width - ( imageHeader.Width + msgTypeHeader.Width + messageHeader.Width + 4 );
		}

		/// <summary>
		/// Called when a task has been activated.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void taskActivated( Object source, EventArgs e )
		{
			ListViewItem selectedTask = taskListView.SelectedItems[0];
			RuleResult result = (RuleResult)selectedTask.Tag;

			JVMSForm jvmsForm = JVMSMain.JvmsForm;
			jvmsForm.CurrentView = result.View;
			jvmsForm.CurrentView.makeVisible( result.Component );
			jvmsForm.SelectedComponent = result.Component;
			
		}

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(TaskList));
			this.taskListView = new System.Windows.Forms.ListView();
			this.imageHeader = new System.Windows.Forms.ColumnHeader();
			this.msgTypeHeader = new System.Windows.Forms.ColumnHeader();
			this.messageHeader = new System.Windows.Forms.ColumnHeader();
			this.componentHeader = new System.Windows.Forms.ColumnHeader();
			this.iconImageList = new System.Windows.Forms.ImageList(this.components);
			this.SuspendLayout();
			// 
			// taskListView
			// 
			this.taskListView.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
																						   this.imageHeader,
																						   this.msgTypeHeader,
																						   this.messageHeader,
																						   this.componentHeader});
			this.taskListView.Dock = System.Windows.Forms.DockStyle.Fill;
			this.taskListView.FullRowSelect = true;
			this.taskListView.GridLines = true;
			this.taskListView.HeaderStyle = System.Windows.Forms.ColumnHeaderStyle.Nonclickable;
			this.taskListView.HideSelection = false;
			this.taskListView.LabelWrap = false;
			this.taskListView.MultiSelect = false;
			this.taskListView.Name = "taskListView";
			this.taskListView.Size = new System.Drawing.Size(560, 200);
			this.taskListView.SmallImageList = this.iconImageList;
			this.taskListView.TabIndex = 0;
			this.taskListView.View = System.Windows.Forms.View.Details;
			// 
			// imageHeader
			// 
			this.imageHeader.Text = "";
			this.imageHeader.Width = 16;
			// 
			// msgTypeHeader
			// 
			this.msgTypeHeader.Text = "Message Type";
			this.msgTypeHeader.Width = 90;
			// 
			// messageHeader
			// 
			this.messageHeader.Text = "Message";
			this.messageHeader.Width = 350;
			// 
			// componentHeader
			// 
			this.componentHeader.Text = "Component";
			this.componentHeader.Width = 96;
			// 
			// iconImageList
			// 
			this.iconImageList.ColorDepth = System.Windows.Forms.ColorDepth.Depth8Bit;
			this.iconImageList.ImageSize = new System.Drawing.Size(16, 16);
			this.iconImageList.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("iconImageList.ImageStream")));
			this.iconImageList.TransparentColor = System.Drawing.Color.Transparent;
			// 
			// TaskList
			// 
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.taskListView});
			this.Name = "TaskList";
			this.Size = new System.Drawing.Size(560, 200);
			this.ResumeLayout(false);

		}
		#endregion
	}
}
