using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

using jvms.datamodel;
using jvms.datamodel.spd;
using jvms.datamodel.prf;
using jvms.system;
using jvms.validation;

namespace jvms.presentation
{
	/// <summary>
	/// Summary description for ComplexXMLEditorForm.
	/// </summary>
	public class UsesDevicePropertyRefEditorForm : System.Windows.Forms.Form
	{
		#region Windows Form Designer generated state

		private System.Windows.Forms.GroupBox itemsGroupBox;
		private System.Windows.Forms.ListBox itemsListBox;
		private System.Windows.Forms.GroupBox separator;
		private System.Windows.Forms.Button cancelButton;
		private System.Windows.Forms.Button okButton;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		#endregion

		private TypedArrayList list = null;
		private System.Windows.Forms.GroupBox groupBox1;
		private System.Windows.Forms.ListBox listBox1;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.Label label2;
		private Object[] backupList = null;

		public UsesDevicePropertyRefEditorForm()
		{
			InitializeComponent();

			okButton.Click += new EventHandler( okButtonClicked );
			cancelButton.Click += new EventHandler( cancelButtonClicked );
			itemsListBox.SelectedIndexChanged += new EventHandler( itemSelectionChanged );
		}

		public void SetSimpleProperty( Simple simple )
		{
			TypedArrayList cfDevices = DataProvider.Instance.GetCFDevices();
			itemsListBox.Items.AddRange( cfDevices.ToArray() );
			if( simple != null )
			{
				//see if we can find the simple
				SCAComponent result = null;
				int i = 0;
				foreach( SCAComponent sca in cfDevices )
				{
					i++;
					TypedArrayList list = DataProvider.Instance.GetAllocationProperties( sca );
					if( list.Contains( simple ) )
					{
						result = sca;
						itemsListBox.SelectedItem = sca;
						break;
					}
				}

				if( result != null )
				{
					//itemsListBox.SetSelected( i, true );
				}
			}
		}

		public Simple GetSimpleProperty()
		{
			return (Simple)listBox1.SelectedItem;
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}


		/// <summary>
		/// Called when the OK button is clicked.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void okButtonClicked( Object source, EventArgs e )
		{
			this.Close();
			View view = JVMSMain.JvmsForm.CurrentView;
			view.Refresh();
		}

		/// <summary>
		/// Called when the cancel button is clicked.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void cancelButtonClicked( Object source, EventArgs e )
		{
			this.Close();
			//this.list.Clear();
			//this.list.AddRange( backupList );
			View view = JVMSMain.JvmsForm.CurrentView;
			view.Refresh();
		}

		/// <summary>
		/// Called when the selected in the items list box is changed.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void itemSelectionChanged( Object source, EventArgs e )
		{
			SCAComponent sca = (SCAComponent)itemsListBox.SelectedItem;
			if ( sca != null )
			{
				TypedArrayList properties = DataProvider.Instance.GetAllocationProperties( sca );
				listBox1.Items.AddRange( properties.ToArray() );
			}
		}

		/// <summary>
		/// Called when the value of an attribute has been changed.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void propertyValueChanged( Object source, PropertyValueChangedEventArgs e )
		{
			itemsListBox.SuspendLayout();
			itemsListBox.Items.Clear();
			itemsListBox.Items.AddRange( list.ToArray() );
			itemsListBox.ResumeLayout();
			ProjectManager.Instance.ChangedFlag = true;
			JVMSMain.JvmsForm.updateTitle();
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.itemsListBox = new System.Windows.Forms.ListBox();
			this.itemsGroupBox = new System.Windows.Forms.GroupBox();
			this.separator = new System.Windows.Forms.GroupBox();
			this.cancelButton = new System.Windows.Forms.Button();
			this.okButton = new System.Windows.Forms.Button();
			this.groupBox1 = new System.Windows.Forms.GroupBox();
			this.listBox1 = new System.Windows.Forms.ListBox();
			this.label1 = new System.Windows.Forms.Label();
			this.label2 = new System.Windows.Forms.Label();
			this.itemsGroupBox.SuspendLayout();
			this.groupBox1.SuspendLayout();
			this.SuspendLayout();
			// 
			// itemsListBox
			// 
			this.itemsListBox.Location = new System.Drawing.Point(16, 24);
			this.itemsListBox.Name = "itemsListBox";
			this.itemsListBox.SelectionMode = System.Windows.Forms.SelectionMode.MultiExtended;
			this.itemsListBox.Size = new System.Drawing.Size(176, 251);
			this.itemsListBox.TabIndex = 0;
			// 
			// itemsGroupBox
			// 
			this.itemsGroupBox.Controls.AddRange(new System.Windows.Forms.Control[] {
																						this.label1,
																						this.itemsListBox});
			this.itemsGroupBox.Location = new System.Drawing.Point(8, 8);
			this.itemsGroupBox.Name = "itemsGroupBox";
			this.itemsGroupBox.Size = new System.Drawing.Size(208, 320);
			this.itemsGroupBox.TabIndex = 4;
			this.itemsGroupBox.TabStop = false;
			this.itemsGroupBox.Text = "CFDevices";
			// 
			// separator
			// 
			this.separator.Location = new System.Drawing.Point(8, 336);
			this.separator.Name = "separator";
			this.separator.Size = new System.Drawing.Size(504, 8);
			this.separator.TabIndex = 6;
			this.separator.TabStop = false;
			// 
			// cancelButton
			// 
			this.cancelButton.Location = new System.Drawing.Point(368, 352);
			this.cancelButton.Name = "cancelButton";
			this.cancelButton.Size = new System.Drawing.Size(75, 24);
			this.cancelButton.TabIndex = 7;
			this.cancelButton.Text = "Cancel";
			// 
			// okButton
			// 
			this.okButton.Location = new System.Drawing.Point(280, 352);
			this.okButton.Name = "okButton";
			this.okButton.Size = new System.Drawing.Size(75, 24);
			this.okButton.TabIndex = 8;
			this.okButton.Text = "OK";
			// 
			// groupBox1
			// 
			this.groupBox1.Controls.AddRange(new System.Windows.Forms.Control[] {
																					this.label2,
																					this.listBox1});
			this.groupBox1.Location = new System.Drawing.Point(232, 8);
			this.groupBox1.Name = "groupBox1";
			this.groupBox1.Size = new System.Drawing.Size(208, 320);
			this.groupBox1.TabIndex = 9;
			this.groupBox1.TabStop = false;
			this.groupBox1.Text = "Properties";
			// 
			// listBox1
			// 
			this.listBox1.Location = new System.Drawing.Point(16, 24);
			this.listBox1.Name = "listBox1";
			this.listBox1.SelectionMode = System.Windows.Forms.SelectionMode.MultiExtended;
			this.listBox1.Size = new System.Drawing.Size(176, 251);
			this.listBox1.TabIndex = 0;
			// 
			// label1
			// 
			this.label1.Location = new System.Drawing.Point(24, 288);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(160, 23);
			this.label1.TabIndex = 1;
			this.label1.Text = "Select the desired CFDevice.";
			// 
			// label2
			// 
			this.label2.Location = new System.Drawing.Point(24, 288);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(152, 23);
			this.label2.TabIndex = 1;
			this.label2.Text = "Select the Desired Property.";
			// 
			// UsesDevicePropertyRefEditorForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(450, 383);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.groupBox1,
																		  this.okButton,
																		  this.cancelButton,
																		  this.separator,
																		  this.itemsGroupBox});
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.Name = "UsesDevicePropertyRefEditorForm";
			this.ShowInTaskbar = false;
			this.Text = "Editor Dialog";
			this.itemsGroupBox.ResumeLayout(false);
			this.groupBox1.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

	}
}
