using System;

using jvms.system;

namespace jvms.presentation
{
	/// <summary>
	/// Controls access to the views.
	/// </summary>
	public class ViewManager
	{
		private static ViewManager instance = new ViewManager();

		private PlatformComponentView platCompView = new PlatformComponentView();
		private PlatformAssemblyView platAssmView = new PlatformAssemblyView();
		private ApplicationComponentView appCompView = new ApplicationComponentView();
		private ApplicationAssemblyView appAssmView = new ApplicationAssemblyView();

		/// <summary>Gets the singleton instance of ViewManager.</summary>
		/// <value>The ViewManager instance.</value>
		public static ViewManager Instance
		{
			get
			{
				return instance;
			}
		}

		/// <summary>Gets the platform component view.</summary>
		/// <value>The platform component view.</value>
		public PlatformComponentView PlatCompView
		{
			get
			{
				return platCompView;
			}
			set
			{
				platCompView = value;
			}
		}

		/// <summary>Gets the platform assembly view.</summary>
		/// <value>The platform assembly view.</value>
		public PlatformAssemblyView PlatAssmView
		{
			get
			{
				return platAssmView;
			}
			set
			{
				platAssmView = value;
			}
		}

		/// <summary>Gets the application component view.</summary>
		/// <value>The application component view.</value>
		public ApplicationComponentView AppCompView
		{
			get
			{
				return appCompView;
			}
			set
			{
				appCompView = value;
			}
		}

		/// <summary>Gets the application assembly view.</summary>
		/// <value>The application assembly view.</value>
		public ApplicationAssemblyView AppAssmView
		{
			get
			{
				return appAssmView;
			}
			set
			{
				appAssmView = value;
			}
		}

		/// <summary>
		/// Creates a new ViewManager.
		/// </summary>
		private ViewManager()
		{
		}

		/// <summary>
		/// Resets all views, only call this when you want to start over from scratch.
		/// </summary>
		public void resetViews()
		{
			//platCompView.clearView();
			//platAssmView.clearView();
			//appCompView.clearView();
			//appAssmView.clearView();

			platCompView = new PlatformComponentView();
			platAssmView = new PlatformAssemblyView();
			appCompView = new ApplicationComponentView();
			appAssmView = new ApplicationAssemblyView();
			JVMSMain.JvmsForm.CurrentView = platCompView;
		}
	}
}
