using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.IO;
using System.Windows.Forms;
using System.Runtime.Serialization;

using jvms.datamodel.dcd;
using jvms.project;
using jvms.system;

namespace jvms.presentation.drawablecomponent
{
	[Serializable]
	/// <summary>
	/// Summary description for NonSCAComponentDrawableComponent.
	/// </summary>
	public class DeviceConfigurationDrawableComponent : DrawableComponent
	{
		private const int TITLE_HEIGHT = 25;

		private DeviceConfiguration deviceConfig = new DeviceConfiguration();

		private static Icon ICON;

		/// <summary>
		/// This will get the DCD that is being drawn.
		/// </summary>
		public DeviceConfiguration DeviceConfiguration
		{
			get
			{
				return deviceConfig;
			}
			set
			{
				deviceConfig = value;
			}
		}

		/// <summary>
		/// This is a static constructor which means that it is only called once when the 
		/// class is loaded.
		/// </summary>
		static DeviceConfigurationDrawableComponent()
		{
			System.Reflection.Assembly myApp = System.Reflection.Assembly.GetExecutingAssembly();
			Stream stream = null;

			stream = myApp.GetManifestResourceStream("jvms.resources.DeviceConfigIcon_Small.ico");
			ICON = new Icon( stream );
			stream.Close();
		}

		public DeviceConfigurationDrawableComponent()
		{
			ProjectManager.Instance.Project.addDeviceConfiguration( deviceConfig );
			this.Size = new Size(250, 100 );
			initComponent();
		}

		/// <summary>
		/// Creates a new ApplicationAssemblyView.
		/// </summary>
		protected DeviceConfigurationDrawableComponent(SerializationInfo info, StreamingContext context) :
			base( info, context )
		{
			deviceConfig = (DeviceConfiguration)info.GetValue( "deviceConfig", typeof(DeviceConfiguration) );
			StatusBarManager.Instance.StatusText = "Loading " + getWrappedObject().ToString();
			initComponent();
		}

		/// <summary>
		/// Serialization Function
		/// </summary>
		public override void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			base.GetObjectData( info, context );
			info.AddValue( "deviceConfig", deviceConfig, deviceConfig.GetType() );
		}

		private void initComponent()
		{
		}

		public override void mouseUp( object sender, MouseEventArgs e )
		{
			base.mouseUp( sender, e );
			if( isOnComponent( e.X, e.Y ) )
			{
				RelationshipManager.Instance.deviceConfigSelected( this );
			}
		}

		/// <summary>
		/// Thiw will perform the action of deleting this component.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		public override void deleteComponent(object sender, System.EventArgs e)
		{
			ViewManager.Instance.PlatAssmView.removeDrawableComponent( this );
			ProjectManager.Instance.Project.DeviceConfigurations.Remove( getWrappedObject() );
			RelationshipManager.Instance.removeRelationshipsToDrawableComponent( this );
		}

		public override Object getWrappedObject()
		{
			return deviceConfig;
		}

		/// <summary>
		/// Called when the view should paint itself.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		public override void paintComponent( Object source, PaintEventArgs e )
		{
			JVMSProperties props = JVMSProperties.Instance;
			Brush headerBrush = new SolidBrush( (Color)props.getProperty("color.deviceconfigheader", Color.PowderBlue ) );
			Brush bodyBrush = new SolidBrush( (Color)props.getProperty("color.content", Color.Beige ) );
			//fill background
			e.Graphics.FillRectangle( bodyBrush, 0 ,0,Size.Width,Size.Height );
			//show title
			e.Graphics.FillRectangle( headerBrush, 0, 0,Size.Width,TITLE_HEIGHT );

			//draw border
			Pen smallPen = new Pen( Color.Black, 1 );
			Pen borderPen = getBorderPen();
			e.Graphics.DrawRectangle( borderPen, borderPen.Width-1, borderPen.Width-1, Size.Width-borderPen.Width, Size.Height-borderPen.Width );

			Font titleFont = new Font( "Arial", 12 );
			Brush titleBrush = Brushes.Black;		
			float zoomFactor = JVMSMain.JvmsForm.CurrentView.ZoomFactor;
			Rectangle targetRect = new Rectangle( 
				(int)(5*zoomFactor), 
				(int)(5 * zoomFactor), 
				(int)(ICON.Width*zoomFactor),
				(int)(ICON.Height*zoomFactor) );
			e.Graphics.DrawIcon( ICON, targetRect );

			StringFormat titleFormat = new StringFormat();
			titleFormat.Alignment = StringAlignment.Near;

			//draw component name and the line
			Rectangle titleRectangle = new Rectangle( 19, 5, Size.Width-19, 20 );
			drawTruncatedString( deviceConfig.Name, titleFont, titleRectangle, titleFormat, titleBrush, e.Graphics );
			e.Graphics.DrawLine( smallPen, new Point( 0, TITLE_HEIGHT ), 
				                           new Point( Size.Width-1, TITLE_HEIGHT ) );
		}		

		/// <summary>
		/// This will return a string representation of this drawable component.
		/// </summary>
		/// <returns></returns>
		public override String ToString()
		{
			return deviceConfig.Name;
		}
	}
}
