using System;
using System.Collections;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;
using System.Runtime.Serialization;

using jvms.datamodel.dcd;
using jvms.datamodel.sad;
using jvms.datamodel.scd;
using jvms.datamodel.spd;
using jvms.project;
using jvms.system;

namespace jvms.presentation.drawablecomponent
{
	[Serializable]
	/// <summary>
	/// Summary description for SCAComponentDrawableComponent.
	/// </summary>
	public class SCAComponentDrawableComponent : BaseSCAComponentDrawableComponent
	{
		public SCAComponentDrawableComponent( String type )
		{
			Component = new SCAComponent();
			Component.Descriptor.ComponentType = type;
			ProjectManager.Instance.Project.addComponent( Component );
			this.Size = new Size(250, 0 );
			//
			// TODO: Add constructor logic here
			//
			initComponent();
		}

		/// <summary>
		/// Creates a new ApplicationAssemblyView.
		/// </summary>
		protected SCAComponentDrawableComponent(SerializationInfo info, StreamingContext context) :
			base( info, context )
		{
			initComponent();
		}

		/// <summary>
		/// Serialization Function
		/// </summary>
		public override void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			base.GetObjectData( info, context );
		}

		private void initComponent()
		{
			Component.Type = "sca_compliant";
		}

		public override void mouseUp( object sender, MouseEventArgs e )
		{
			base.mouseUp( sender, e );
			if( isOnComponent( e.X, e.Y ) )
			{
				RelationshipManager.Instance.scaComponentSelected( this );
			}
		}

		/// <summary>
		/// Thiw will perform the action of deleting this component.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		public override void deleteComponent(object sender, System.EventArgs e)
		{
			//first delete myself
			ViewManager.Instance.AppCompView.removeDrawableComponent( this );
			ViewManager.Instance.PlatCompView.removeDrawableComponent( this );
			//then delete all references to instances of this
			deleteInstancesInList( ViewManager.Instance.PlatAssmView.DrawableComponents, sender, e );
			deleteInstancesInList( ViewManager.Instance.AppAssmView.DrawableComponents, sender, e );
			ProjectManager.Instance.Project.Components.Remove( getWrappedObject() );
			RelationshipManager.Instance.removeRelationshipsToDrawableComponent( this );
		}

		private void deleteInstancesInList( IList controls, Object sender, EventArgs e )
		{
			IList controlsCopy = new ArrayList( controls );
			foreach( DrawableComponent dc in controlsCopy )
			{
				if( dc.getWrappedObject() is ApplicationComponentInstantiation )
				{
					ApplicationComponentInstantiation instance = 
						(ApplicationComponentInstantiation)dc.getWrappedObject();
					if( instance.Component == getWrappedObject() )
					{
						dc.deleteComponent( sender, e );
					}
				}
				else if( dc.getWrappedObject() is ComponentPlacement )
				{
					ComponentPlacement instance = 
						(ComponentPlacement)dc.getWrappedObject();
					if( instance.ComponentFile == getWrappedObject() )
					{
						dc.deleteComponent( sender, e );
					}
				}
			}
		}

		public override Object getWrappedObject()
		{
			return Component;
		}

		/// <summary>
		/// Called when the view should paint itself.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		public override void paintComponent( Object source, PaintEventArgs e )
		{
			paintSCAComponent( e );
		}

		/// <summary>
		/// This will return a string representation of this drawable component.
		/// </summary>
		/// <returns></returns>
		public override String ToString()
		{
			return Component.Name;
		}
	}
}
