using System;
using System.Collections;
using System.Drawing;
using System.Windows.Forms;

namespace jvms.presentation.relationships
{
	/// <summary>
	/// Defines methods common to classes representing relationships.
	/// </summary>
	public interface IRelationship
	{
		/// <summary>
		/// Gets or sets a data object associated with the relationship.
		/// </summary>
		Object Tag
		{
			get;
			set;
		}

		/// <summary>
		/// Gets or sets a flag indicating if the relationship is selected.
		/// </summary>
		bool Selected
		{
			get;
			set;
		}

		/// <summary>
		/// Gets or sets a list of points in the relationship's line.
		/// </summary>
		IList Verticies
		{
			get;
			set;
		}

		/// <summary>
		/// Called when the relationship should draw itself.
		/// </summary>
		/// <param name="clientRect">The area in which to draw the relationship.</param>
		/// <param name="g">The graphics context to use.</param>
		void drawRelationship( Rectangle clientRect, Graphics g );

		/// <summary>
		/// Called when the component should redraw itself.
		/// </summary>
		void update();

		/// <summary>
		/// Returns whether the specified point is on a draggable part of the relationship.
		/// </summary>
		/// <param name="p">The desired point.</param>
		/// <returns>True if a draggable point, false otherwise.</returns>
		bool isDragging( Point p );

		/// <summary>
		/// Notifies the relationship that it has been dragged.
		/// </summary>
		/// <param name="p">The point to which the relationship has been dragged.</param>
		void dragNotify( Point p );

		/// <summary>
		/// Notifies the relationship that it has been scrolled.
		/// </summary>
		/// <param name="direction">The direction of the scrolling.</param>
		/// <param name="change">The amount of scrolling.</param>
		void scrollNotify( Orientation direction, int change );

		/// <summary>
		/// Returns true if the given point is on the relationship (roughly).
		/// </summary>
		/// <param name="p">The desired point.</param>
		/// <returns>True if point is on line, false otherwise.</returns>
		bool isIntersection( Point p );

		/// <summary>
		/// This will perform the operation of deleting this relationship and
		/// removing all references to other components.
		/// </summary>
		void delete();
	}
}
