using System;
using System.Collections;
using System.Drawing;

using jvms.datamodel.common;
using jvms.datamodel.scd;

using jvms.presentation.drawablecomponent;

namespace jvms.presentation.relationships
{
	/// <summary>
	/// Describes a port connection relationship.
	/// </summary>
	[Serializable]
	public abstract class PortConnectionRelationship : IPortRelationship
	{
		private ComponentPortEndPoint providesPort;
		private ComponentPortEndPoint usesPort;

		private BaseSCAComponentDrawableComponent providesComponent;
		private BaseSCAComponentDrawableComponent usesComponent;

		/// <summary>Gets or sets the provides end point,</summary>
		/// <value>The provides port data model object.</value>
		public ComponentPortEndPoint ProvidesPort
		{
			get
			{
				return providesPort;
			}
			
			set
			{
				providesPort = value;
			}
		}

		/// <summary>Gets or sets the uses end point.</summary>
		/// <value>The uses port data model object.</value>
		public ComponentPortEndPoint UsesPort
		{
			get
			{
				return usesPort;
			}

			set
			{
				usesPort = value;
			}
		}

		/// <summary>Gets or sets the provides component.</summary>
		/// <value>The provides drawable component.</value>
		public BaseSCAComponentDrawableComponent ProvidesComponent
		{
			get
			{
				return providesComponent;
			}

			set
			{
				providesComponent = value;
			}
		}

		/// <summary>Gets or sets the uses component.</summary>
		/// <value>The uses drawable component.</value>
		public BaseSCAComponentDrawableComponent UsesComponent
		{
			get
			{
				return usesComponent;
			}

			set
			{
				usesComponent = value;
			}
		}

		/// <summary>
		/// Creates a new PortConnectionRelationship.
		/// Cannot be called directly, subclass PortConnectionRelationship to obtain this functionality.
		/// </summary>
		public PortConnectionRelationship()
		{
		}

		/// <summary>
		/// Called after the relationship has been completely set up.
		/// </summary>
		public abstract void initRelationship();

		/// <summary>
		/// This will get the connectInterface that this relationship represents.
		/// </summary>
		/// <returns>The connection for this relationship.</returns>
		public abstract ConnectInterface getConnectInterface();
	}
}
