using System;

using jvms.datamodel.common;
using jvms.datamodel.scd;
using jvms.presentation.drawablecomponent;

namespace jvms.presentation.relationships
{
	/// <summary>
	/// Describes a relationship that needs a single port.
	/// </summary>
	[Serializable]
	public abstract class SinglePortConnectionRelationship : IPortRelationship
	{
		private BaseSCAComponentDrawableComponent component;
		private ComponentPortEndPoint port;

		/// <summary>The component the relationship connects to.</summary>
		/// <value>The drawable component.</value>
		public BaseSCAComponentDrawableComponent Component
		{
			get
			{
				return component;
			}

			set
			{
				component = value;
			}
		}

		/// <summary>The port to which the relationship connects.</summary>
		/// <value>The data model object.</value>
		public ComponentPortEndPoint Port
		{
			get
			{
				return port;
			}

			set
			{
				port = value;
			}
		}
		/// <summary>
		/// Creates a new SinglePortConnectionRelationship.
		/// </summary>
		public SinglePortConnectionRelationship()
		{
		}

		/// <summary>
		/// Called when the relationship has been set up
		/// </summary>
		public abstract void initRelationship();

		/// <summary>
		/// This will get the connectInterface that this relationship represents.
		/// </summary>
		/// <returns>The connection for this relationship.</returns>
		public abstract ConnectInterface getConnectInterface();
	}
}
