using System;
using System.Collections;
using System.IO;
using System.Windows.Forms;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters;
using System.Runtime.Serialization.Formatters.Soap;

using jvms.datamodel;
using jvms.datamodel.common;
using jvms.datamodel.spd;
using jvms.datamodel.sad;
using jvms.datamodel.dcd;
using jvms.presentation;
using jvms.validation;

namespace jvms.project
{
	/// <summary>
	/// Responsible for saving and loading projects.
	/// </summary>
	public class JVMSSerializer
	{
		private static JVMSSerializer serializer = new JVMSSerializer();

		/// <summary>
		/// Creates a new JVMSSerializer
		/// </summary>
		public JVMSSerializer()
		{
		}

		/// <summary>Gets the single instance of JVMSSerializer.</summary>
		/// <value>The instance of JVMSSerializer.</value>
		public static JVMSSerializer Instance
		{
			get
			{
				return serializer;
			}
		}
		
		/// <summary>
		/// Writes the given object's state as XML.
		/// </summary>
		public void writeXml( String path )
		{
			IFormatter formatter = new SoapFormatter();
			((SoapFormatter)formatter).AssemblyFormat = FormatterAssemblyStyle.Simple;

			ProjectState state = new ProjectState();
			state.Project = ProjectManager.Instance.Project;
			state.Relationships = RelationshipManager.Instance.Relationships;
			state.ApplicationAssemblyView = ViewManager.Instance.AppAssmView;
			state.ApplicationComponentView = ViewManager.Instance.AppCompView;
			state.PlatformAssemblyView = ViewManager.Instance.PlatAssmView;
			state.PlatformComponentView = ViewManager.Instance.PlatCompView;
			
			Stream projectStream = null;
			try
			{
				projectStream = new FileStream(
					path,
					FileMode.Create,
					FileAccess.Write,
					FileShare.None
					);			
				formatter.Serialize( projectStream, state );

				JVMSMain.JvmsForm.updateTitle();
			}
			finally
			{
				projectStream.Close();
			}
		}

		/// <summary>
		/// Reads the given XML file and returns the deserialized object.
		/// </summary>
		/// <param name="path">The XML file to read.</param>
		/// <returns>The deserialized object.</returns>
		public void readXml( String path )
		{
			IFormatter formatter = new SoapFormatter();
			((SoapFormatter)formatter).AssemblyFormat = FormatterAssemblyStyle.Simple;

			Stream projectStream = null;
			try
			{
				StatusBarManager.Instance.StatusText = "Loading " + path;
				projectStream = new FileStream(
					path,
					FileMode.Open,
					FileAccess.Read,
					FileShare.None
					);
	
				ProjectState state = (ProjectState)formatter.Deserialize( projectStream );				

				ProjectManager.Instance.Project = state.Project;
				RelationshipManager.Instance.Relationships = state.Relationships;
				ViewManager.Instance.AppAssmView = state.ApplicationAssemblyView ;
				ViewManager.Instance.AppCompView = state.ApplicationComponentView;
				ViewManager.Instance.PlatAssmView = state.PlatformAssemblyView;
				ViewManager.Instance.PlatCompView = state.PlatformComponentView;

				JVMSMain.JvmsForm.updateTitle();
				JVMSMain.JvmsForm.CurrentView = ViewManager.Instance.PlatCompView;
				StatusBarManager.Instance.StatusText = "JTRS Visual Modeling Studio";
			}
			finally
			{
				projectStream.Close();
			}			
		}
	}
}
