using System;
using System.ComponentModel;
using System.Windows.Forms;

using jvms.datamodel;
using jvms.presentation;

namespace jvms.project
{
	/// <summary>
	/// Summary description for Class1.
	/// </summary>
	public class ProjectManager
	{
		private static ProjectManager manager = new ProjectManager();

		private Project project = new Project();
		private String projectFile = "";
		private bool changedFlag = false;

		/// <summary>
		/// Gets the single instance of the project manager.
		/// </summary>
		/// <value>The single ProjectManager instance.</value>
		public static ProjectManager Instance
		{
			get
			{
				return manager;
			}
		}

		/// <summary>
		/// This will get/set the current project that is being worked on.
		/// </summary>
		/// <value>The current project.</value>
		public Project Project
		{
			get
			{
				return project;
			}

			set
			{
				project = value;
				JVMSMain.JvmsForm.initProject();
			}
		}

		/// <summary>Gets or sets the project filename.</summary>
		/// <value>The project file.</value>
		public String ProjectFile
		{
			get
			{
				return projectFile;
			}

			set
			{
				projectFile = value;
			}
		}

		/// <summary>Sets or gets a value indicating if the current project has been changed.</summary>
		/// <value>A bool flag.</value>
		public bool ChangedFlag
		{
			get
			{
				return changedFlag;
			}

			set
			{
				changedFlag = value;
				JVMSMain.JvmsForm.updateTitle();
			}
		}

		/// <summary>
		/// Creates a new ProjectManager.
		/// </summary>
		private ProjectManager()
		{
		}

		/// <summary>
		/// Initiates saving the project.
		/// </summary>
		public void saveProject()
		{
			if ( projectFile.Equals( "" ) )
			{
				SaveFileDialog saveDialog = new SaveFileDialog();
				saveDialog.Title = "Save Project";
				saveDialog.RestoreDirectory = true;
				saveDialog.AddExtension = true;
				saveDialog.DefaultExt = "jvproj";
				saveDialog.Filter = "JVMS Project Files (*.jvproj;*.jvp)|*.jvproj;*.jvp";
				saveDialog.InitialDirectory = project.ProjectDirectory;
			
				if ( saveDialog.ShowDialog( JVMSMain.JvmsForm ) == DialogResult.OK )
				{
					JVMSMain.JvmsForm.Cursor = Cursors.WaitCursor;
					projectFile = saveDialog.FileName;
				}
			}

			if ( !projectFile.Equals( "" ) )
			{
				try
				{
					JVMSMain.JvmsForm.Cursor = Cursors.WaitCursor;
					JVMSSerializer.Instance.writeXml( projectFile );
					changedFlag = false;
					JVMSMain.JvmsForm.updateTitle();
				}
				catch ( Exception e )
				{
					MessageBox.Show( "Cannot save project.\n" + e.Message, "Error Saving Project", MessageBoxButtons.OK, MessageBoxIcon.Error );
					Console.WriteLine( e );
				}
				finally
				{
					JVMSMain.JvmsForm.Cursor = Cursors.Default;
				}
			}
		}

		/// <summary>
		/// Initiates saving a project as.
		/// </summary>
		public void saveProjectAs()
		{
			SaveFileDialog saveDialog = new SaveFileDialog();
			saveDialog.Title = "Save Project";
			saveDialog.RestoreDirectory = true;
			saveDialog.AddExtension = true;
			saveDialog.DefaultExt = "jvproj";
			saveDialog.Filter = "JVMS Project Files (*.jvproj;*.jvp)|*.jvproj;*.jvp";
			saveDialog.InitialDirectory = project.ProjectDirectory;
			
			if ( saveDialog.ShowDialog( JVMSMain.JvmsForm ) == DialogResult.OK )
			{
				try
				{
					JVMSMain.JvmsForm.Cursor = Cursors.WaitCursor;
					projectFile = saveDialog.FileName;
					JVMSSerializer.Instance.writeXml( saveDialog.FileName );
					changedFlag = false;
					JVMSMain.JvmsForm.updateTitle();
				}
				catch ( Exception e )
				{
					MessageBox.Show( "Cannot save project.\n" + e.Message, "Error Saving Project", MessageBoxButtons.OK, MessageBoxIcon.Error );
				}
				finally
				{
					JVMSMain.JvmsForm.Cursor = Cursors.Default;
				}
			}
		}

		/// <summary>
		/// Initiates opening a project.
		/// </summary>
		public void openProject()
		{
			OpenFileDialog openDialog = new OpenFileDialog();
			openDialog.Title = "Open Project";
			openDialog.RestoreDirectory = true;
			openDialog.AddExtension = true;
			openDialog.DefaultExt = "jvproj";
			openDialog.Filter = "JVMS Project Files (*.jvproj;*.jvp)|*.jvproj;*.jvp";
			openDialog.InitialDirectory = project.ProjectDirectory;

			if ( openDialog.ShowDialog( JVMSMain.JvmsForm ) == DialogResult.OK )
			{
				try
				{
					JVMSForm jvmsForm = JVMSMain.JvmsForm;
					jvmsForm.Cursor = Cursors.WaitCursor;
					projectFile = openDialog.FileName;
					JVMSSerializer.Instance.readXml( openDialog.FileName );

					jvmsForm.CurrentAttributeSheetControl.clearAttributeSheet();
					jvmsForm.SelectedComponent = null;
					jvmsForm.CurrentView.Refresh();
					jvmsForm.TaskListControl.clearTasks();
					changedFlag = false;
					JVMSMain.JvmsForm.updateTitle();
				}
				catch ( Exception e )
				{
					MessageBox.Show( "Cannot open project.\n" + e.Message, "Error Opening Project", MessageBoxButtons.OK, MessageBoxIcon.Error );
					Console.WriteLine( e.Message );
					Console.WriteLine( e.StackTrace );
					if( e.InnerException != null )
					{
						Console.WriteLine( e.InnerException.StackTrace );
					}
				}
				finally
				{
					JVMSMain.JvmsForm.Cursor = Cursors.Default;
				}
			}
		}
	}
}
