using System;

using System.Collections;
using System.IO;
using System.Xml;
using System.Reflection;
using System.CodeDom.Compiler;
using System.Windows.Forms;
using System.ComponentModel;
using System.Drawing.Design;

using jvms.datamodel;
using jvms.datamodel.spd;
using jvms.system;
using jvms.presentation;

using Microsoft.CSharp;

namespace jvms.validation
{
	public class AttributeService
	{
		private static AttributeService instance = null;
		private XmlDocument config = null;
		private Hashtable typeConverters = new Hashtable();
		private static Hashtable typeConverterAssemblies = new Hashtable();
		private String assemblyLocation = JVMSMain.AssemblyDir;

		private AttributeService()
		{
			JVMSProperties props = JVMSProperties.Instance;
			string configFile = assemblyLocation + props.getProperty( "AttributeService.config", "AttributeService.xml" );
			config = new XmlDocument();
			config.Load( configFile );
			AppDomain.CurrentDomain.AssemblyResolve += new ResolveEventHandler(MyResolver);
		}

		public static AttributeService Instance
		{
			get
			{
				if( instance == null )
				{
					instance = new AttributeService();
				}
				return instance;
			}
		}
		
		public IList QueryValues(Object obj)
		{
			IList attributeTypes = new ArrayList();
			String id = obj.GetType().Name;
			XmlElement element = config.GetElementById(id);
			XmlNode n = config.FirstChild;
			foreach( XmlNode n2 in n.ChildNodes )
			{
				if( n2.NodeType == XmlNodeType.Comment )
				{
					continue;
				}
				if( n2.Attributes["id"].Value.Equals(id) )
				{
					
					foreach( XmlNode node in n2.ChildNodes )
					{
						string categoryName;
						XmlAttributeCollection attributes;
						AttributeType attrType;
						BoundPropertySpec spec;
						string name;
						string description = "";
						bool required;
						string path;
						string complexEditor;
						ArrayList values;
						bool exclusive;
						string defaultValue = null;
						string typeConverterName;
						string property;
						string dataprovider;
						bool readonlyAttr = false;
						XmlAttribute xmlAttr;
						if( node.NodeType == XmlNodeType.Comment )
						{
							continue;
						}
						categoryName = node.Attributes["name"].Value;
						foreach( XmlNode attribute in node.ChildNodes )
						{
							if( attribute.NodeType == XmlNodeType.Comment )
							{
								continue;
							}
							attrType = new AttributeType();
							attributes = attribute.Attributes;
							if( attributes == null ) continue;
							xmlAttr = attributes["name"];
							name = (xmlAttr == null) ? null : xmlAttr.Value;

							xmlAttr = attributes["required"];
							required = (xmlAttr == null) ? false : Boolean.Parse(xmlAttr.Value);
							if( required )
							{
								name = "* " + name;
							}

							xmlAttr = attributes["readonly"];
							readonlyAttr = (xmlAttr == null) ? false : Boolean.Parse(xmlAttr.Value);

							xmlAttr = attributes["exclusive"];
							exclusive = (xmlAttr == null) ? false : Boolean.Parse(xmlAttr.Value);

							xmlAttr = attributes["complexEditor"];							
							complexEditor = (xmlAttr == null) ? null : xmlAttr.Value;

							xmlAttr = attributes["path"];
							path = (xmlAttr == null) ? null : xmlAttr.Value;

							xmlAttr = attributes["property"];
							property = (xmlAttr == null) ? null : xmlAttr.Value;

							xmlAttr = attributes["dataprovider"];
							dataprovider = (xmlAttr == null) ? null : xmlAttr.Value;

							defaultValue = null;
							typeConverterName = null;
							description = null;
							if( attribute.HasChildNodes )
							{
								values = new ArrayList();
							}
							else
							{
								values = null;
							}
							
							bool def;
							foreach( XmlNode val in attribute.ChildNodes )
							{
								if( val.NodeType == XmlNodeType.Comment )
								{
									continue;
								}
								if( val.LocalName.Equals( "value" ) )
								{
									values.Add( val.InnerText );
									xmlAttr = val.Attributes["default"];
									def = (xmlAttr == null) ? false : Boolean.Parse(xmlAttr.Value); 
									if( def )
									{
										defaultValue = val.InnerText;
									}
								}
								else if( val.LocalName.Equals( "description" ) )
								{
									description = val.InnerText;
								}
							}
							if( values != null && values.Count > 0 )
							{
								attrType.ValidValues = values;
								//create our converter
								string[] temp = new String[values.Count];
								values.CopyTo(temp);
								String typeName = obj.GetType().Name + "_"
										+ categoryName + "_" + name;
								typeName = typeName.Replace( ' ', '_' );
								//typeConverterName = GetNameForConverter( 
								//	temp,
								//	exclusive, typeName );
							}
							Object obj2 = null;
							PropertyInfo pInfo = getPropertyInfo( obj, property, out obj2 );
							Type t = pInfo.PropertyType;
							//Object ob = t.Assembly.CreateInstance(type);
							spec = new BoundPropertySpec( name, t, categoryName, description, defaultValue );
							spec.PropertyName = property;
							spec.ConverterTypeName = typeConverterName;
							spec.BoundObject = obj2;
							spec.BoundProperty = pInfo;
							if( complexEditor != null )
							{
								//EditorAttribute attr = new EditorAttribute( typeof(ComplexXMLValueEditor), typeof(UITypeEditor) );
								EditorAttribute attr = new EditorAttribute( Type.GetType(complexEditor), typeof(UITypeEditor) );
								spec.AttributesAsArrayList.Add( attr );
							}
							if( readonlyAttr )
							{
								spec.AttributesAsArrayList.Add( new ReadOnlyAttribute(true) );
							}
							attrType.BoundSpec = spec;
							attrType.DataProviderMethod = dataprovider;
							attributeTypes.Add( attrType );
						}
					}
					break;
				}
			}
			return attributeTypes;
		}

		public string GetNameForConverter( string[] values, bool exclusive, string name )
		{
			String finalResult = null;
			if( !typeConverters.Contains(name) )
			{

				CSharpCodeProvider codeProvider = new CSharpCodeProvider();
				ICodeCompiler compiler = codeProvider.CreateCompiler();
				CompilerParameters parameters = new CompilerParameters();

				parameters.GenerateInMemory = true;
				parameters.OutputAssembly = assemblyLocation + name;
				
				foreach (Assembly asm in AppDomain.CurrentDomain.GetAssemblies()) 
				{
					parameters.ReferencedAssemblies.Add(asm.Location);
				}
				
				string code = "using System; "
					+ "using System.ComponentModel; "
					+ "using System.Collections; "
					+ "using System.Windows.Forms; "
					+ "using System.Diagnostics; "
					+ "using jvms.presentation; "
					+ "namespace " + name + " { "
					+ "public class " + name + " : DefaultValuesStringConverter { "
					+ "public " + name + "() "
					+ "{ "
					+ "values = new object[" + values.Length + "]; ";
				for( int i = 0; i < values.Length; i++ )
				{
					code += "values[" + i + "] = \"" + values[i] +"\"; ";
				}
				code += "exclusive = " + exclusive.ToString().ToLower() 
					+ "; } } }";

				CompilerResults results = compiler.CompileAssemblyFromSource( parameters, code );
				if( results.NativeCompilerReturnValue == 0 )
				{
					Assembly aResult = results.CompiledAssembly;
					Type t = aResult.GetType( name + "." + name );
					finalResult = t.AssemblyQualifiedName;
					typeConverterAssemblies[name] = aResult;
				}
				typeConverters[name] = finalResult;
			}
			else
			{
				finalResult = (string)typeConverters[name];
			}

			return finalResult;
		}

		static Assembly MyResolver(object sender, ResolveEventArgs args) 
		{
			AppDomain domain = (AppDomain) sender;
			int index = args.Name.IndexOf(',');
			string name = args.Name.Remove( index, args.Name.Length - index );
			//MessageBox.Show( name );
      
			//byte[] rawAssembly = loadFile(name);
			//Assembly assembly = domain.Load(rawAssembly);
			Assembly assembly = (Assembly)typeConverterAssemblies[name];

			return assembly;
		}
		
		// Loads the content of a file to a byte array. 
		static byte[] loadFile(string filename) 
		{
			FileStream fs = new FileStream(filename, FileMode.Open);
			byte[] buffer = new byte[(int) fs.Length];
			fs.Read(buffer, 0, buffer.Length);
			fs.Close();
   
			return buffer;
		}

		private PropertyInfo getPropertyInfo( Object obj, String property, out Object ob2 )
		{
			PropertyInfo pInfo = null;
			int index = property.IndexOf( "." );
			String tempProp;
			while( index > 0 )
			{
				tempProp = property.Substring( 0, index );
				property = property.Substring( index + 1, property.Length - index - 1 );
				pInfo = obj.GetType().GetProperty( tempProp );
				obj = pInfo.GetValue( obj, null );
				index = property.IndexOf( "." );
			}
			pInfo = obj.GetType().GetProperty(property);
			ob2 = obj;
			return pInfo;
		}
	}
}
