using System;

using jvms.datamodel;
using jvms.datamodel.common;
using jvms.datamodel.dcd;
using jvms.datamodel.sad;
using jvms.datamodel.spd;
using jvms.datamodel.prf;
using jvms.datamodel.scd;
using jvms.presentation;
using jvms.presentation.drawablecomponent;
using jvms.presentation.relationships;
using jvms.project;
using jvms.system;

using System.Collections;

namespace jvms.validation
{
	/// <summary>
	/// 
	/// </summary>
	public class DataProvider
	{
		private static DataProvider instance = null;


		/// <summary>
		/// Constructor
		/// </summary>
		private DataProvider()
		{
		}

		/// <summary>
		/// Property to access single instance of this class.
		/// </summary>
		public static DataProvider Instance
		{
			get
			{
				if( instance == null )
				{
					instance = new DataProvider();
				}
				return instance;
			}
		}
		
		/// <summary>
		/// 
		/// </summary>
		public TypedArrayList GetUsesDevices( SCAComponent component )
		{
			return component.UsesDevices;
		}

		/// <summary>
		/// 
		/// </summary>
		public TypedArrayList GetSimpleAllocProperties()
		{
			TypedArrayList result = new TypedArrayList( typeof(Simple) );
			if( ProjectManager.Instance.Project != null )
			{
				IList components = ProjectManager.Instance.Project.Components;
				foreach( object obj in components )
				{
					if( typeof(Simple).Equals(obj.GetType()) )
					{
						Simple simple = (Simple)obj;
						if( simple.Kinds.Contains("allocation") )
						{
							if( !result.Contains(simple) )
							{
								result.Add( simple );
							}
						}
					}
				}
			}
			return result;
		}

		/// <summary>
		/// 
		/// </summary>
		public TypedArrayList GetCFDevices()
		{
			TypedArrayList result = new TypedArrayList( typeof(SCAComponent) );
			if( ProjectManager.Instance.Project != null )
			{
				IList components = ProjectManager.Instance.Project.Components;
				foreach( object obj in components )
				{
					if( typeof(SCAComponent).Equals(obj.GetType()) )
					{
						SCAComponent sca = (SCAComponent)obj;
						if( sca.Descriptor.ComponentType.Equals("device") )
						{
							result.Add( sca );
						}
					}
				}
			}
			return result;
		}

		/// <summary>
		/// 
		/// </summary>
		public TypedArrayList GetCFDeviceInstances()
		{
			TypedArrayList result = new TypedArrayList( typeof(ComponentPlacement) );
			if( ProjectManager.Instance.Project != null )
			{
				IList instances = ProjectManager.Instance.Project.DeviceConfigInstantiations;
				foreach( object obj in instances )
				{
					ComponentPlacement sca = (ComponentPlacement)obj;
					if( sca.ComponentFile is SCAComponent &&
						((SCAComponent)sca.ComponentFile).Descriptor.ComponentType.Equals( "device" ) )
					{
						result.Add( sca );
					}
				}
			}
			return result;
		}

		/// <summary>
		/// 
		/// </summary>
		public IList GetResourceFactoryInstances()
		{
			IList result = new ArrayList();
			if( ProjectManager.Instance.Project != null )
			{
				IList instances = ProjectManager.Instance.Project.ApplicationInstantiations;
				foreach( ApplicationComponentInstantiation obj in instances )
				{
					if( obj.Component is SCAComponent &&
						((SCAComponent)obj.Component).Descriptor.ComponentType.Equals( "resourcefactory" ) )
					{
						result.Add( obj );
					}
				}
			}
			return result;
		}

		/// <summary>
		/// Gets only allocation properties
		/// </summary>
		public TypedArrayList GetAllocationProperties( NonSCAComponent component )
		{
			TypedArrayList result = new TypedArrayList( typeof(SCAComponent) );
			if( component.Properties != null )
			{
				foreach( Simple simple in component.Properties.Simples )
				{
					if( simple.Kinds.Contains("allocation") )
					{
						if( !result.Contains(simple) )
						{
							result.Add( simple );
						}
					}
				}
			}
			return result;
		}

		/// <summary>
		/// Gets all simple properties associated with a component
		/// </summary>
		public TypedArrayList GetSimpleProperties( NonSCAComponent component )
		{
			TypedArrayList result = new TypedArrayList( typeof(SCAComponent) );
			if( component.Properties != null )
			{
				foreach( Simple simple in component.Properties.Simples )
				{
					if( !result.Contains(simple) )
					{
						result.Add( simple );
					}
				}
			}
			return result;
		}


		public IList GetInstancesOnView()
		{
			IList retval = new ArrayList();
			foreach( DrawableComponent obj in JVMSMain.JvmsForm.CurrentView.DrawableComponents )
			{
				if( obj is ApplicationComponentInstantiationDrawableComponent )
				{
					retval.Add( obj.getWrappedObject() );
				}
				if( obj is DeviceConfigurationComponentInstantiationDrawableComponent )
				{
					retval.Add( obj.getWrappedObject() );
				}
			}
			return retval;
		}

		public IList GetDeviceManagers()
		{
			IList retval = new ArrayList();
			foreach( NonSCAComponent nsca in ProjectManager.Instance.Project.Components )
			{
				if( typeof(SCAComponent).Equals( nsca.GetType() ) )
				{
					if( ((SCAComponent)nsca).Descriptor.ComponentType.Equals("devicemanager") )
					{
						retval.Add( nsca );
					}
				}
			}
			return retval;
		}

		public IList GetStructsForCurrentProperties()
		{
			IList retval = new ArrayList();
			DrawableComponent component = JVMSMain.JvmsForm.SelectedComponent;
			if( component is PropertiesDrawableComponent )
			{
				retval = ((PropertiesDrawableComponent)component).Properties.Structs;
			}
			return retval;
		}

		public IList GetUsesDevicesForSelectedRelationship()
		{
			IList retval = new ArrayList();
			IRelationship relationship = RelationshipManager.Instance.SelectedRelationship;
			if( relationship is DeviceUsedByRelationship )
			{
				DeviceUsedByRelationship usedByRelationship = (DeviceUsedByRelationship)relationship;
				retval = usedByRelationship.Component.Component.UsesDevices;
			}

			return retval;
		}

		/// <summary>
		/// The uses device relationship needs to connect to a port on the device.  If 
		/// the device has a simple property that matches the simple property from the
		/// uses device then we will include that port.
		/// </summary>
		/// <returns></returns>
		public IList GetPortsForUsesDeviceRelationship()
		{
			IList retval = new ArrayList();
			IRelationship relationship = RelationshipManager.Instance.SelectedRelationship;
			if( relationship is DeviceUsedByRelationship )
			{
				DeviceUsedByRelationship usedByRelationship = (DeviceUsedByRelationship)relationship;
				if( usedByRelationship.UsesDevice != null )
				{
					IList propertyRefs = usedByRelationship.UsesDevice.PropertyRefs;
					foreach( DrawableComponent dc in ViewManager.Instance.PlatCompView.DrawableComponents )
					{
						if( dc is SCAComponentDrawableComponent )
						{
							SCAComponent component = ((SCAComponentDrawableComponent)dc).Component;
							Properties props = component.Properties;
							if( props != null && component.Descriptor.ComponentType.Equals( "device" ) )
							{
								bool matchesComponent = true;
								foreach( SimpleRef simpleRef in propertyRefs )
								{
									if( simpleRef.Simple == null ||
										!props.Simples.Contains( simpleRef.Simple ) )
									{
										matchesComponent = false;
									}
								}
								if( matchesComponent )
								{
									ComponentPortEndPoint instancePort = usedByRelationship.Port;
									SoftwareComponent scd = null;
									if( usedByRelationship.Component.getWrappedObject() is ComponentPlacement )
									{
										ComponentPlacement placement = 
											(ComponentPlacement)usedByRelationship.Component.getWrappedObject();
										scd = ((SCAComponent)placement.ComponentFile).Descriptor;
									}
									else if( usedByRelationship.Component.getWrappedObject() is ApplicationComponentInstantiation )
									{
										ApplicationComponentInstantiation instance = 
											(ApplicationComponentInstantiation)usedByRelationship.Component.getWrappedObject();
										scd = ((SCAComponent)instance.Component).Descriptor;
									}

									if( scd.Features.UsesPorts.Contains( instancePort ) )
									{
										//if the instance port is a uses port than we want to 
										//include all provides ports on the component
										foreach( ComponentPortEndPoint port in component.Descriptor.Features.ProvidesPorts )
										{
											retval.Add( port );
										}
										foreach( Interface i in component.Descriptor.Features.SupportInterfaces )
										{
											ComponentPortEndPoint iPort = new ComponentPortEndPoint();
											iPort.RepositoryID = i;
											iPort.UsesName = i.Name;
											retval.Add( iPort );
										}
									}
									else
									{
										//else if the instance port is a uses port than we want
										//to include all of the provides ports on the device.
										retval =  component.Descriptor.Features.UsesPorts;
									}
								}
							}
						}
					}
				}
			}

			return retval;
		}

		/// <summary>
		/// 
		/// </summary>
		public TypedArrayList GetInterfaces()
		{
			TypedArrayList result = new TypedArrayList( typeof(Interface) );
			IList allInterfaces = (IList)JVMSProperties.Instance.getProperty( "interfaces", new ArrayList() );
			result.AddRange( allInterfaces );
			/*
			if( ProjectManager.Instance.Project != null )
			{
				IList components = ProjectManager.Instance.Project.Components;
				foreach( object obj in components )
				{
					if( typeof(SCAComponent).Equals(obj.GetType()) )
					{
						SCAComponent component = (SCAComponent)obj;
						foreach( object obj2 in component.Descriptor.Interfaces )
						{
							if( !result.Contains(obj2) )
							{
								result.Add( obj2 );
							}
						}
					}
				}
			}*/
			return result;
		}
	}
}
