using System;
using System.Collections;

using jvms.datamodel;

namespace jvms.ruleframework
{
	/// <summary>
	/// This class represents a validation rule that can be run 
	/// on a datamodel object.
	/// </summary>
	public abstract class Rule
	{
		private String name = "";
		private String description = "";
		private String category = "";
		private Project project;

		public String Category
		{
			get{ return category; }
			set{ category = value; }
		}


		/// <summary>
		/// The name of this rule.
		/// </summary>
		public String Name
		{
			get
			{
				return name;
			}
			set
			{
				name = value;
			}
		}

		/// <summary>
		/// Some descriptive text about this rule.
		/// </summary>
		public String Description
		{
			get
			{
				return description;
			}
			set
			{
				description = value;
			}
		}

		/// <summary>
		/// The project that this rule is being executed on.
		/// </summary>
		public Project Project
		{
			get
			{
				return project;
			}
			set
			{
				project = value;
			}
		}

		/// <summary>
		/// This is a flag that indicates if this rule is global.  A global rule is
		/// only run when an application or device config is validated.
		/// </summary>
		/// <returns>true if this is a local component.</returns>
		public abstract bool isLocal();

		/// <summary>
		/// This will be called by the rule engine when a rule needs to be run against an
		/// object.
		/// </summary>
		/// <param name="component">The component to run the rule on.</param>
		/// <returns>A list of validation errors.</returns>
		public abstract IList validate( Object component );

		/// <summary>
		/// This function will tell if this rule works on the type that is passed in.
		/// </summary>
		/// <param name="type">The type we are testing against.</param>
		/// <returns>true if this rule will perform checks on this type.</returns>
		public abstract bool worksOnType( Type type );
	}
}
