using System;
using System.Collections;

using jvms.datamodel.sad;
using jvms.ruleframework;

namespace jvms.rules
{
	/// <summary>
	/// Checks required field on an application.
	/// </summary>
	public class ApplicationMetaRule : Rule
	{
		/// <summary>
		/// Creates a new ApplicationMetaRule.
		/// </summary>
		public ApplicationMetaRule()
		{
			Name = "Application Syntax";
			Description = "Checks required fields on applications.";
			Category = "Application";
		}

		/// <summary>
		/// Returns true if the rule is local to a component, false if it has project wide scope.
		/// </summary>
		/// <returns>True if rule is local, false otherwise.</returns>
		public override bool isLocal()
		{
			return true;
		}

		/// <summary>
		/// Performs validation.
		/// </summary>
		/// <param name="obj">The data model object to validate.</param>
		/// <returns>A list of error messages.</returns>
		public override IList validate( Object obj )
		{
			IList errors = new ArrayList();
			Application app = (Application)obj;
			if ( app.AssemblyController == null )
			{
				errors.Add( "No assembly controller specified for application (" + app.Name + ")." );
			}
			else
			{
				if ( !app.ComponentPlacements.Contains( app.AssemblyController ) )
				{
					errors.Add( "Specified assembly controller is not a member of the application (" + app.Name + ")." );
				}
			}

			if ( !app.ID.DCEUUID.StartsWith( "DCE:" ) )
			{
				errors.Add( "ID for application must be DCE UUID (" + app.Name + ")." );
			}

			return errors;
		}

		/// <summary>
		/// Returns true if the rule can be run on the given type, false otherwise.
		/// </summary>
		/// <param name="type">The type in question.</param>
		/// <returns>True if the rule can be run on the type, false otherwise.</returns>
		public override bool worksOnType( Type type )
		{
			return type.Equals( typeof( Application ) );
		}
	}
}
