using System;
using System.Collections;

using jvms.datamodel.dcd;
using jvms.ruleframework;

namespace jvms.rules
{
	/// <summary>
	/// Checks required field on an application.
	/// Rule33 - says name should be unique for a particular service type,
	///		but I don't know what that means so this rule says all instance
	///		names should be unique.
	/// </summary>
	public class DCDAssemblyDomainRule : Rule
	{
		/// <summary>
		/// Creates a new AssemblyDomainRule.
		/// </summary>
		public DCDAssemblyDomainRule()
		{
			Name = "Platform Domain";
			Description = "Checks interactions between platform components.";
			Category = "Platform";
		}

		/// <summary>
		/// Returns true if the rule is local to a component, false if it has project wide scope.
		/// </summary>
		/// <returns>True if rule is local, false otherwise.</returns>
		public override bool isLocal()
		{
			return false;
		}

		/// <summary>
		/// Performs validation.
		/// </summary>
		/// <param name="obj">The data model object to validate.</param>
		/// <returns>A list of error messages.</returns>
		public override IList validate( Object obj )
		{
			IList errors = new ArrayList();
			ComponentPlacement cp = (ComponentPlacement)obj;
			DeviceConfiguration dcd = null;
			foreach ( DeviceConfiguration dcdTemp in Project.DeviceConfigurations )
			{
				if ( dcdTemp.Partitioning.Contains(cp) )
				{
					dcd = dcdTemp;
					break;
				}
			}
			
			if ( dcd != null )
			{
				String name = cp.Instantiation.UsageName;
				foreach ( ComponentPlacement cp2 in dcd.Partitioning )
				{
					if ( cp != cp2 )
					{
						if ( name.Equals( cp2.Instantiation.UsageName ) )
						{
							errors.Add( "Instance name must be unique. (" + name + ")" );
						}
					}
				}
			}

			return errors;
		}

		/// <summary>
		/// Returns true if the rule can be run on the given type, false otherwise.
		/// </summary>
		/// <param name="type">The type in question.</param>
		/// <returns>True if the rule can be run on the type, false otherwise.</returns>
		public override bool worksOnType( Type type )
		{
			return type.Equals( typeof( ComponentPlacement ) );
		}
	}
}
