using System;
using System.Collections;

using jvms.datamodel.dcd;
using jvms.ruleframework;

namespace jvms.rules
{
	/// <summary>
	/// Checks required field on an application.
	/// </summary>
	public class DCDMetaRule : Rule
	{
		/// <summary>
		/// Creates a new ApplicationMetaRule.
		/// </summary>
		public DCDMetaRule()
		{
			Name = "Device Configuration Syntax";
			Description = "Checks required fields on Device Configuration.";
			Category = "Platform";
		}

		/// <summary>
		/// Returns true if the rule is local to a component, false if it has project wide scope.
		/// </summary>
		/// <returns>True if rule is local, false otherwise.</returns>
		public override bool isLocal()
		{
			return true;
		}

		/// <summary>
		/// Performs validation.
		/// </summary>
		/// <param name="obj">The data model object to validate.</param>
		/// <returns>A list of error messages.</returns>
		public override IList validate( Object obj )
		{
			IList errors = new ArrayList();
			DeviceConfiguration dcd = (DeviceConfiguration)obj;
			
			if ( !dcd.ID.DCEUUID.StartsWith( "DCE:" ) )
			{
				errors.Add( "ID for device configuration must be DCE UUID (" + dcd.Name + ")." );
			}

			return errors;
		}

		/// <summary>
		/// Returns true if the rule can be run on the given type, false otherwise.
		/// </summary>
		/// <param name="type">The type in question.</param>
		/// <returns>True if the rule can be run on the type, false otherwise.</returns>
		public override bool worksOnType( Type type )
		{
			return type.Equals( typeof( DeviceConfiguration ) );
		}
	}
}
