using System;
using System.Collections;

using jvms.datamodel.common;
using jvms.datamodel.scd;
using jvms.datamodel.spd;
using jvms.ruleframework;

namespace jvms.rules
{
	/// <summary>
	/// Checks required field on an application.
	/// </summary>
	public class InterfaceSyntacticalRule : Rule
	{
		/// <summary>
		/// Creates a new ApplicationMetaRule.
		/// </summary>
		public InterfaceSyntacticalRule()
		{
			Name = "Interface Syntax";
			Description = "Checks required fields on component interfaces.";
			Category = "Component";
		}

		/// <summary>
		/// Returns true if the rule is local to a component, false if it has project wide scope.
		/// </summary>
		/// <returns>True if rule is local, false otherwise.</returns>
		public override bool isLocal()
		{
			return true;
		}

		/// <summary>
		/// Performs validation.
		/// </summary>
		/// <param name="obj">The data model object to validate.</param>
		/// <returns>A list of error messages.</returns>
		public override IList validate( Object obj )
		{
			IList errors = new ArrayList();
			SCAComponent component = (SCAComponent)obj;

			//rule18
			IList interfaces = component.Descriptor.getInterfaces();
			foreach ( Interface next in interfaces )
			{
				if( !next.RepositoryID.StartsWith( "IDL:" ) )
				{
					errors.Add( "Repository ID must be CORBA rep. ID (" + next.RepositoryID + ")." );
				}
			}


			return errors;
		}

		/// <summary>
		/// Returns true if the rule can be run on the given type, false otherwise.
		/// </summary>
		/// <param name="type">The type in question.</param>
		/// <returns>True if the rule can be run on the type, false otherwise.</returns>
		public override bool worksOnType( Type type )
		{
			return type.Equals( typeof( SCAComponent ) );
		}
	}
}
