using System;
using System.Collections;

using jvms.datamodel.prf;

using jvms.ruleframework;

namespace jvms.rules
{
	/// <summary>
	/// Summary description for SimplePropertySemanticRule.
	/// </summary>
	public class PropertiesMetaRule : Rule
	{
		/// <summary>
		/// Creates a new SimplePropertySemanticRule.
		/// </summary>
		public PropertiesMetaRule()
		{
			Name = "Properties Semantics";
			Description = "Checks for appropriate agreement within properties components.";
			Category = "Properties";
		}

		/// <summary>
		/// Returns true if the rule is local to a component, false if it has project wide scope.
		/// </summary>
		/// <returns>True if rule is local, false otherwise.</returns>
		public override bool isLocal()
		{
			return true;
		}

		/// <summary>
		/// Performs validation.
		/// </summary>
		/// <param name="obj">The data model object to validate.</param>
		/// <returns>A list of error messages.</returns>
		public override IList validate( Object obj )
		{
			IList errors = new ArrayList();
			Properties props = (Properties)obj;

			if ( props.Simples.Count + props.SimpleSequences.Count +	props.Structs.Count + 
				 props.StructSequences.Count + props.Test.Count == 0 )
			{
				errors.Add( "At least one property is required (" + props.Description + ")." );
			}
			return errors;
		}

		/// <summary>
		/// Returns true if the rule can be run on the given type, false otherwise.
		/// </summary>
		/// <param name="type">The type in question.</param>
		/// <returns>True if the rule can be run on the type, false otherwise.</returns>
		public override bool worksOnType( Type type )
		{
			return type.Equals( typeof( Properties ) );
		}
	}
}
