using System;
using System.Collections;

using jvms.datamodel.prf;

using jvms.ruleframework;

namespace jvms.rules
{
	/// <summary>
	/// Summary description for SimplePropertySemanticRule.
	/// </summary>
	public class SimplePropertySemanticRule : Rule
	{
		/// <summary>
		/// Creates a new SimplePropertySemanticRule.
		/// </summary>
		public SimplePropertySemanticRule()
		{
			Name = "Simple Semantics";
			Description = "Checks type agreement between fields on simple properties.";
			Category = "Properties";
		}

		/// <summary>
		/// Returns true if the rule is local to a component, false if it has project wide scope.
		/// </summary>
		/// <returns>True if rule is local, false otherwise.</returns>
		public override bool isLocal()
		{
			return true;
		}

		/// <summary>
		/// Performs validation.
		/// </summary>
		/// <param name="obj">The data model object to validate.</param>
		/// <returns>A list of error messages.</returns>
		public override IList validate( Object obj )
		{
			IList errors = new ArrayList();
			Properties props = (Properties)obj;
			foreach ( Simple simple in props.Simples )
			{
				//rule13
				if ( simple.Kinds.Contains( "allocation" ) )
				{
					if ( !simple.ID.DCEUUID.StartsWith( "DCE:" ) )
					{
						errors.Add( "ID for allocation simple must be DCE UUID (" + simple.Name + ")." );
					}
				}
				else
				{
					if ( simple.ID.DCEUUID.Trim().Length == 0 )
					{
						errors.Add( "ID is required for simple (" + simple.Name + ")." );
					}
				}
				
				//rule14
				if ( simple.Type.Equals( "boolean" ) )
				{
					//not sure what the rules are here.
				}
				else if ( simple.Type.Equals( "char" ) )
				{
					//allow for zero or one characters
					if ( simple.Value.Length > 1 )
					{
						errors.Add( "Value of simple must be single character (" + simple.Name + ")." );
					}
				}

				//rule 16
				if ( simple.Kinds.Contains( "test" ) &&	!simple.Type.Equals( "ulong" ) )
				{
					errors.Add( "Simple of kind 'test' must have type 'ulong' (" + simple.Name + ")." );
				}
			}
			return errors;
		}

		/// <summary>
		/// Returns true if the rule can be run on the given type, false otherwise.
		/// </summary>
		/// <param name="type">The type in question.</param>
		/// <returns>True if the rule can be run on the type, false otherwise.</returns>
		public override bool worksOnType( Type type )
		{
			return type.Equals( typeof( Properties ) );
		}
	}
}
